/*****************************************************************************
 * Copyright (C) 1997-2007, Mark Hummel
 * This file is part of Vrq.
 *
 * Vrq is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * Vrq is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 *****************************************************************************
 */
/******************************************************************************
 *
 *
 *	   cnode.cpp
 *		- methods class for declaration 
 *
 *
 ******************************************************************************
 */


#include <math.h>
#include <stack>
#include "cnode.h"
#include "cvector.h"
#include "lex.h"
#include "cportdir.h"
#include "cparam.h"
#include "cinstance.h"
#include "creg.h"
#include "cnet.h"
#include "cgenvar.h"
#include "cfref.h"
#include "cobstack.h"
#include "cport.h"
#include "cfunction.h"
#include "cgate.h"
#include "cspecify.h"
#include "cmodule.h"
#include "cevent.h"
#include "creal.h"
#include "systask.h"
#include "external.h"
#include "array.h"

extern DelayMode_t delayMode;

#define EVAL_VECTOR_BINARY( op )					\
	{								\
		CNode* l = Arg<CNode*>(0);				\
		CNode* r = Arg<CNode*>(1);				\
		CVector vl(l->width);					\
		CVector vr(r->width);					\
		vl.Signed( type==eS );					\
		vr.Signed( type==eS );					\
		l->_EvalVector(vl);					\
		r->_EvalVector(vr);					\
		op( &v, &vl, &vr );					\
	}								\

#define EVAL_VECTOR_BINARY_SELF( op )					\
	{								\
		CNode* l = Arg<CNode*>(0);				\
		CNode* r = Arg<CNode*>(1);				\
		CVector vl(l->width);					\
		CVector vr(r->width);					\
		vl.Signed( l->type==eS );				\
		vr.Signed( r->type==eS );				\
		l->_EvalVector(vl);					\
		r->_EvalVector(vr);					\
		op( &v, &vl, &vr );					\
	}								\

#define EVAL_VECTOR_BINARY_SELF_RIGHT( op )				\
	{								\
		CNode* l = Arg<CNode*>(0);				\
		CNode* r = Arg<CNode*>(1);				\
		CVector vl(l->width);					\
		CVector vr(r->width);					\
		vl.Signed( type==eS );					\
		vr.Signed( r->type==eS );				\
		l->_EvalVector(vl);					\
		r->_EvalVector(vr);					\
		op( &v, &vl, &vr );					\
	}								\
	
#define EVAL_REAL_BINARY( op )						\
	{								\
		CNode* l = Arg<CNode*>(0);				\
		CNode* r = Arg<CNode*>(1);				\
		double dl = l->_EvalReal();				\
		double dr = r->_EvalReal();				\
		op( &d, &dl, &dr );					\
	}								\

#define EVAL_VECTOR_UNARY( op )						\
	{								\
		CNode* l = Arg<CNode*>(0);				\
		CVector vl(l->width);					\
		vl.Signed( type==eS );					\
		l->_EvalVector(vl);					\
		op( &v, &vl );						\
	}								\

#define EVAL_VECTOR_UNARY_SELF( op )					\
	{								\
		CNode* l = Arg<CNode*>(0);				\
		CVector vl(l->width);					\
		vl.Signed( l->type==eS );				\
		l->_EvalVector(vl);					\
		op( &v, &vl );						\
	}								\

#define EVAL_REAL_UNARY( op )						\
	{								\
		CNode* l = Arg<CNode*>(0);				\
		double dl = l->_EvalReal();				\
		op( &d, &dl );						\
	}								\

#define EVAL_RELATIONAL( op )					\
	{							\
	    CNode* l = Arg<CNode*>(0);				\
	    CNode* r = Arg<CNode*>(1);				\
	    if( l->type == eR  ) {				\
		double dr = r->_EvalReal();			\
		double dl = l->_EvalReal();			\
		MASSERT( r->type == eR );			\
		op( &v, &dl, &dr );				\
	    } else if( l->type == eS && 			\
		       r->type == eS ) {			\
		CVector vr(r->width);				\
		vr.Signed(1);					\
		r->_EvalVector(vr);				\
		CVector vl(l->width);				\
		vl.Signed(1);					\
		l->_EvalVector(vl);				\
		op( &v, &vl, &vr );				\
	    } else {						\
		CVector vr(r->width);				\
		r->_EvalVector(vr);				\
		CVector vl(l->width);				\
		l->_EvalVector(vl);				\
		op( &v, &vl, &vr );				\
	    }							\
       }

#define EVAL_VECTOR_HOOK()					\
	{							\
	    CVector vl(Arg<CNode*>(0)->width);			\
	    CVector vrl(Arg<CNode*>(1)->width);			\
	    CVector vrr(Arg<CNode*>(2)->width); 		\
	    vl.Signed( Arg<CNode*>(0)->type==eS );		\
	    vrl.Signed( type==eS );				\
	    vrr.Signed( type==eS );				\
	    Arg<CNode*>(0)->_EvalVector(vl);			\
	    Arg<CNode*>(1)->_EvalVector(vrl);			\
	    Arg<CNode*>(2)->_EvalVector(vrr);			\
	    Hook( &v, &vl, &vrl, &vrr );			\
	}

#define EVAL_REAL_HOOK()					\
	{							\
	    CVector vl(Arg<CNode*>(0)->width);			\
	    Arg<CNode*>(0)->_EvalVector(vl);			\
	    double drr = Arg<CNode*>(2)->_EvalReal();		\
	    double drl = Arg<CNode*>(1)->_EvalReal();		\
	    Hook( &d, &vl, &drl, &drr ); 			\
	}

#define EVAL_VECTOR_PARAM_REF()					\
	{							\
	    CNode* n = Arg<CParam*>(0)->GetExpression();	\
	    n->LabelBits();					\
	    if( n->type == eR ) {				\
		v.LoadINT32((INT32)(n->_EvalReal() + .5));	\
	    } else {						\
		n->_EvalVector(v);				\
	    }							\
	}

#define EVAL_REAL_PARAM_REF()					\
	{							\
	    CNode* n = Arg<CParam*>(0)->GetExpression();	\
	    CVector v(width);					\
	    if( n->type == eR ) {				\
		d = n->_EvalReal();				\
	    } else {						\
		n->_EvalVector(v);				\
		d = v.GetINT32(); 				\
	    }							\
	}

extern CObstack* unitStack;
CObstack* CNode::stack = unitStack;
CObstack		CNode::evalHeap;
INT32			CNode::evalCount = 0;
map<CNode*,int> CNode::labelCache;  
int             CNode::labelCacheEnabled = 0;

/************************************************
 	Constructor
**************************************************/

CNode::CNode( Coord_t* aLoc, NodeOp_t aOp ) {

	/*
	 * set supplied members
	 */
	op    = aOp;
	type  = eU;
	width = 0;
        attributes = NULL;
	fixedWidth = 0;


	/*
	 * initialize node location
	 */
	locp = &loc;
	if( aLoc != NULL ) {
		loc = *aLoc;
	} else {
		loc.filename = NULL;
	} 

	/*
 	 * initial subtree if complex op
	 */
	switch( ArgCount() ) {
	case 0:
	case 1:
	case 2:
		left  = NULL;
		right = NULL;
		break;
	case 3:
		left  = _LIST( NULL, NULL, aLoc );
		right = NULL;
		break;
	case 4:
		left  = _LIST( NULL, NULL, aLoc );
		right = _LIST( NULL, NULL, aLoc );
		break;
	case 5:
		left   = _LIST( _LIST( NULL, NULL, aLoc ), NULL, aLoc );
		right  = _LIST( NULL, NULL, aLoc );
		break;
	case 6:
		left   = _LIST( _LIST( NULL, NULL, aLoc ), _LIST( NULL, NULL, aLoc), aLoc );
		right  = _LIST( NULL, NULL, aLoc );
		break;
	case 7:
		left   = _LIST( _LIST( NULL, NULL, aLoc ), _LIST( NULL, NULL, aLoc), aLoc );
		right  = _LIST( _LIST( NULL, NULL, aLoc ), NULL, aLoc );
		break;
	case 8:
		left   = _LIST( _LIST( NULL, NULL, aLoc ), _LIST( NULL, NULL, aLoc) );
		right  = _LIST( _LIST( NULL, NULL, aLoc ), _LIST( NULL, NULL, aLoc), aLoc );
		break;
	default:
		MASSERT( FALSE );	
	}

}

/***********************************************************
	EvalINT32
	- evaluate express as a 32 bit integer.
	  Must be constant.  
 **********************************************************/

INT32 CNode::EvalINT32() {
	CVector value(32);

	CNode::UseEvalStack();
	CNode* exp = this->Clone();
	MASSERT( exp->IsConstant() );
	exp->EvalVector( value );
	CNode::ResetBuildStack();
	return value.GetINT32();
}
#define DEFINE_METHODS
#include "cnode_def.h"
#undef DEFINE_METHODS

/*
 * return 0 if operation fails due to errors
 */
int	CNode::LabelBits( int suppressErrorMessages )
{
	LabelBitsInfo info = { suppressErrorMessages, 0 };
	PostVisit1( _LabelBits, (void*)&info );
	return info.error;
}

/*************************************************
 	Simplify
	- returns a new node if constants can be folded
**************************************************/

CNode*  CNode::Simplify( INT32 newWidth, NodeType_t newType )
{
	if( /*IsWidthVolatile() || !IsWidthEvaluateable() || */ 
	    IsVolatile() || !IsEvaluateable() ) {
	    return this;
	}

	if( newType == eR ) {
	    return _REAL( EvalReal() );
	}
	CVector v( newWidth );
	v.Signed( newType == eS );
	EvalVector( v, newWidth, newType );
	v.SetPreferredBase( 2 );
	v.SetWidth( newWidth );
	return _CVECTOR( v );
}

/*************************************************
 	EvalVector, _EvalVector
	- evaluates vector constant expression tree 
**************************************************/

void	CNode::EvalVector( CVector& v )
{
	EvalVector( v, 0, eU );

}

void	CNode::EvalVector( CVector& v, INT32 newWidth, NodeType_t newType )
{

	CNode::UseEvalStack();
	CNode* exp = this->Clone();
	exp->LabelBits();
	if( newType == eB ) {
	    newType = eS;
	}
	exp = exp->FixBits( newWidth, newType );
// printf( "MDH: " ); exp->Dump(stdout); printf( "\n" );

	MASSERT( exp->type != eR );

	exp->_EvalVector( v );

	CNode::ResetBuildStack();

}

/*************************************************
 	EvalReal, _EvalReal
	- evaluates real constant expression tree 
**************************************************/

double	CNode::EvalReal()
{
	double d;

	CNode::UseEvalStack();
	CNode* exp = this->Clone();
	exp->LabelBits();
	exp = exp->FixBits( 0, eR );
// printf( "MDH: " ); exp->Dump(stdout); printf( "\n" );
	MASSERT( exp->type == eR );

	d = exp->_EvalReal();
	CNode::ResetBuildStack();

	return d;
}

