/*****************************************************************************
 * Copyright (C) 1997-2007, Mark Hummel
 * This file is part of Vrq.
 *
 * Vrq is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * Vrq is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 *****************************************************************************
 */
/******************************************************************************
 *
 *
 *	   creg.cpp
 *		- class definition of register 
 *		  definition nodes
 *
 ******************************************************************************
 */

#include "creg.h"
#include "cnode.h"
#include "cvector.h"




/****************************************************
	Constructor
*****************************************************/
CReg::CReg( CSymbol* name, Coord_t* aLoc, Decl_t type, int undefined ) :
	CDecl( name, aLoc, type, Or(eFLAG_PRAGMA,eFLAG_ARRAY,eFLAG_VECTOR) ), undefined(undefined) {
}

/****************************************************
	Copy Constructor
	- perform a deep copy
*****************************************************/
CReg::CReg( const CReg& reg ) :
	CDecl::CDecl( reg )
{
     undefined = reg.undefined;
}

/****************************************************
	Dump
*****************************************************/
void	CReg::Dump( FILE* f )
{
    CNode* range = GetRange();
    if( range ) {
	CNode* msb = range->Arg<CNode*>(0);
	CNode* lsb = range->Arg<CNode*>(1);
	CVector vmsb(msb->GetWidth());
	CVector vlsb(lsb->GetWidth());
	msb->EvalVector(vmsb);
	lsb->EvalVector(vlsb);
	fprintf( f, "%s[", declName[GetType()] );
	msb->Dump( f );
	fprintf( f, ":" );
	lsb->Dump( f );
	fprintf( f, "]: %s, defined in ", GetName() );
    } else {
	fprintf( f, "%s: %s, defined in ", declName[GetType()], GetName() );
    }
    CDecl::Dump( f );
    fprintf( f, "\n" );
}	

