/*****************************************************************************
 * Copyright (C) 1997-2007, Mark Hummel
 * This file is part of Vrq.
 *
 * Vrq is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * Vrq is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 *****************************************************************************
 */

%{
/* 
 * TODO:
 * - clean up attribute/pragma processing, store pragmas in 
 *   parse tree, not in node.
 */

#define yyparse prog_parse
#define YYERROR_VERBOSE
#define YYDEBUG 2
# define YYMAXDEPTH 1000000

#define LEX_CPP

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <string.h>
#include <list>
#include "glue.h"
#include "csymbol.h"
#include "cmodule.h"
#include "cobstack.h"
#include "cinstance.h"
#include "cgate.h"
#include "cvector.h"
#include "cnode.h"
#include "cparam.h"
#include "creg.h"
#include "cgenvar.h"
#include "creal.h"
#include "cnet.h"
#include "cportdir.h"
#include "cport.h"
#include "cfref.h"
#include "csymtab.h"
#include "cfunction.h"
#include "cspecify.h"
#include "cdecl.h"
#include "cevent.h"
#include "cattr.h"
#include "lex.h"

#ifdef _WINDOWS
#define	alloca _alloca
#endif

#ifdef __MWERKS__
#include "alloca.h"
#endif // __MWERKS__

/* Used in BISON.SIM */
extern char *token_buffer;
extern CObstack* permStack;
extern CObstack* unitStack;
extern Coord_t	loc;
extern CSymtab<CDecl>	moduleSymtab;
extern CSymtab<CDecl>	portSymtab;
extern CSymtab<CDecl>	frefSymtab;
extern CSymtab<CModule>	defSymtab;
extern CNode*		codeList;

CNode* PrependComments( CNode* n );
CNode* PostpendComments( CNode* n );
CNode* GetComments();
void ValidateLeftHandComments( CNode* n, int more );
void DoTopLevelComments( void );
void DoComments( void );
int  IsExternal( CSymbol* );
void yyerror( const char* );
void error( Coord_t*, const char*, ... );
static CModule*	  currentModule;
static CModule*	  currentPrimitive;
static CFunction* currentFunction;
static CBlock*	  currentBlock;
static CSymbol*   currentInstanceType;
static CNode*	  currentInstanceParameters;
static CNode*	  currentInstanceAttributes;
static CNode	  errorNode( NULL, eERROR ); 
static CDecl*	  errorDecl = NULL;
static CSymbol*	  empty = NULL;
static CSymtab<CDecl> savePortSymtab;
static CBlock*	      saveBlock;
static CNode*	      currentAttributeList = NULL;
static CNode*	      lastAttributeList = NULL;
int	      keepMacros = 0;
#define empty (empty==NULL?empty= new(permStack) CSymbol( "", TRUE, FALSE ):empty)
#define errorDecl (errorDecl==NULL?errorDecl=new(permStack) CFref( empty, &loc ):errorDecl)
typedef struct DeclType_t {
	Decl_t		type;
	CNode*		range;
	int		scalar;
	CNode*		delay;
	StrengthPair_t	stpair;
	CNode*		pragma;
	int		dimension;
	vector<CNode*>	dimensions;
	int		scalared;
	int		vectored;
	int		signed_;
	Decl_t		subType;
	CNode*		attr;
        int	        inlined;
} DeclType_t;

CSymbol*	currentPortName;

DeclType_t	currentDeclType;

CSymbol*		gateType;
CNode*			currentGateDelay = NULL;
int		inSpecifyBlock = 0;
StrengthPair_t		currentDriveStrength;
CNet* CreateNet( CSymbol* symbol, CBlock* block,
	DeclType_t declType, int setDelay );
CGenvar* CreateGenvar( int create, CSymbol* symbol, CBlock* block, DeclType_t declType );
CReg* CreateRegister( CSymbol* symbol, CBlock* block, DeclType_t decl );
CReal* CreateReal( CSymbol* symbol, CBlock* block, DeclType_t declType );
CEvent* CreateEvent( CSymbol* symbol, CBlock* block, DeclType_t declType );
CVector* SizedStringToVector(Coord_t* refLoc, char* width, char* spec, char* value);
CSymbol* ValidateType( CSymbol* sym, int allowFref, ... );
CNode*   ValidateLvalList( CNode* n, int allowFref, ... );
void RegisterFrefs( CNode* n, void* );
CNode* PortDir2PortDef( CNode* dirList );
MacroToken* AnnotateMacroToken( Coord_t loc, CNode* exp, 
			MacroToken* m1=NULL, MacroToken* m2=NULL, 
		        MacroToken* m3=NULL, MacroToken* m4=NULL, 
			MacroToken* m5=NULL, MacroToken* m6=NULL );

/*
 * warning messages
 */
static Message* mCTTSW = Message::RegisterWarning( NULL,
		Message::eWARNING, "CTTSW",
		"Constant truncated to specified width" );
static Message* mTMBIN = Message::RegisterWarning( NULL,
		Message::eWARNING, "TMBIN",
		"Too many bits in number" );
static Message* mTMDIN = Message::RegisterWarning( NULL,
		Message::eWARNING, "TMDIN",
		"Too many digits in number" );
static Message* mWPDFS = Message::RegisterWarning( NULL, 
		Message::eWARNING, "WPDFS",
		"%s was previously defined in file '%s' line %d"
		", second definition will be ignored",
		"<decl>", "<filename>", 0 );
static Message* mRSINT = Message::RegisterWarning( NULL,
		Message::eWARNING, "RSINT",
		"range specification is not" 
	        " allowed in time declaration, ignoring range" );
static Message* mRSINI = Message::RegisterWarning( NULL, 
		Message::eWARNING, "RSINI",
		"range specification is not" 
	   	" allowed in integer declaration, ignoring range" );
/* 
 * use for lval/rval parsing
 */
class DimInfo {
public:
    list<CNode*>	dimensions;
    DimInfo() {}
    CNode* RangeSelect() {
	if( dimensions.size() == 0 ) {
	    return NULL;
	}
	CNode* last = dimensions.back();
	switch( last->GetOp() ) {
	case eSLICE:
	case ePSLICE:
	case eMSLICE:
	    return last;
	default:
	    break;
	}
	return NULL;
   }
};

/*
 * identifier structure
 */
class HIdentifier_ {
public:
    CSymbol*       symbol;
    list<CNode*>   dim;
    CNode* RangeSelect() {
	if( dim.size() == 0 ) {
	    return NULL;
	}
	CNode* last = dim.back();
	switch( last->GetOp() ) {
	case eSLICE:
	case ePSLICE:
	case eMSLICE:
	    return last;
	default:
	    break;
	}
	return NULL;
   }
};
class HIdentifier {
    vector<HIdentifier_> elements;
    int invalid;
public:
    HIdentifier( CSymbol* symbol ) : invalid(0) {
	class HIdentifier_ id;
	id.symbol = symbol;
	elements.push_back(id);
    }
    HIdentifier( CSymbol* symbol, CNode* index ) : invalid(0) {
	class HIdentifier_ id;
	id.symbol = symbol;
	id.dim.push_back(index);
	elements.push_back(id);
    }
    void Set( DimInfo* info ) {
	if( info->RangeSelect() && elements.back().RangeSelect() ) {
	    invalid = 1;
	}
	elements.back().dim.insert( elements.back().dim.end(),
		info->dimensions.begin(), info->dimensions.end() );	
    }
    void Push( HIdentifier* hid ) {
	if( elements.back().RangeSelect() ) {
	    invalid = 1;
	}
	if( elements.back().dim.size() > 1 ) {
	    invalid = 1;
	}
	if( elements.back().dim.size() == 1 && 
	    !elements.back().dim.front()->IsConstant() ) {
	    invalid = 1;
	}
	elements.insert( elements.end(), hid->elements.begin(), hid->elements.end() );
    }
    DimInfo* Dim() {
	if( !elements.back().dim.size() ) {
	    return NULL;
	}
	DimInfo* info = new DimInfo();
	info->dimensions.insert( info->dimensions.end(), 
			elements.back().dim.begin(), elements.back().dim.end() );
        return info;
    }
    int IsExternal() { return elements.size() > 1; }
    CSymbol* Symbol() { 
	if( elements.size() > 1 ) {
	    string buffer;
	    vector<HIdentifier_>::iterator ptr;
	    int first = 1;
	    for( ptr = elements.begin(); ptr != elements.end(); ++ptr ) {
		if( !first ) {
		    buffer += ".";
		}
		buffer += Identifier(ptr->symbol->GetName());
		if( ptr+1 != elements.end() && ptr->dim.size() &&
		    ptr->dim.front()->IsConstant() ) {
		    buffer += "[";
		    char index[32];
		    sprintf( index, "%d", 
				ptr->dim.front()->EvalINT32() );
		    buffer += index; 
		    buffer += "]";
		}
		first = 0;
	    }
	    CSymbol* s = CSymbol::Lookup( (char*)buffer.c_str(), TRUE, TRUE );
	    return s;
	} else {
	    return elements.back().symbol; 
	}
    }
    int ValidId( int allowBitSelect, int allowPartSelect ) {
	if( !allowBitSelect && elements.back().dim.size() != 0 ) {
	    invalid = 1;
	}
	if( !allowPartSelect && elements.back().RangeSelect() ) {
	    invalid = 1;
	}
	if( elements.back().RangeSelect() && 
	    !elements.back().RangeSelect()->IsWidthConstant() ) {
	    invalid = 1;
	}
	return !invalid;
    }
};


CNode* Val( int createFref, int rval, CSymbol* sym, DimInfo* dim );

/*
 * used for attribute processing
 */
CAttr* attribute = NULL;

/*
 * used for decl parsing
 */
list<CNode*>	dimIndexes;

/*
 * variables set to control parsing
 */
int modulesDirectlyDefined;
int modulesIncluded;
int allowRvalFrefs = 0;

/*
 * global state from lexer for compiler directives
 */
extern int compilerDirectiveCell;
extern int compilerDirectiveProtect;
extern int compilerDirectivePrecision;
extern int compilerDirectiveUnit;
extern int compilerDirectiveTimescaleValid;
extern int compilerDirectiveDefaultNetType;
extern int compilerDirectiveUnconnectedDrive;

#define NI(s) error( &loc, "UNSUPPORTED CONSTRUCT: %s", #s );

#define ABS(a) ((a)<0?-a:a)

inline MacroToken* UOP( CNode* op(CNode*,Coord_t*), MacroToken* m1, MacroToken* m2 ) {
    CNode* a2 = Macro2Node(m2);
    CNode* n = op( a2, &loc );
    return AnnotateMacroToken( loc, n, m1, m2 );
}

inline MacroToken* NO_UOP( MacroToken* m1, MacroToken* m2 ) {
    CNode* a2 = Macro2Node(m2);
    return AnnotateMacroToken( loc, a2, m1, m2 );
}

inline MacroToken* BOP( CNode* op(CNode*,CNode*,Coord_t*), 
		   MacroToken* m1, MacroToken* m2, MacroToken* m3 ) {
    CNode* a1 = Macro2Node(m1);
    CNode* a3 = Macro2Node(m3);
    CNode* n = op( a1, a3, &loc );
    return AnnotateMacroToken( loc, n, m1, m2, m3 );
}

inline MacroToken* TOP( CNode* op(CNode*,CNode*,CNode*, Coord_t*), 
		   MacroToken* m1, MacroToken* m2, MacroToken* m3,
		   MacroToken* m4, MacroToken* m5 ) {
    CNode* a1 = Macro2Node(m1);
    CNode* a3 = Macro2Node(m3);
    CNode* a5 = Macro2Node(m5);
    CNode* n = op( a1, a3, a5, &loc );
    return AnnotateMacroToken( loc, n, m1, m2, m3, m4, m5 );
}
inline MacroToken* _LINK( MacroToken* m1, MacroToken* m2 ) 
{
    CNode_pr a1 = m1->nodePtr;
    CNode_pr a2 = m2->nodePtr;
    MacroToken* result = MergeToken( m1, m2 ); 
    if( !result ) {
	result = m2;
        result->previousNodePtr = _LINK(m1->previousNodePtr,a1);
    } else {
        result->nodePtr = _LINK(a1,a2);
        result->previousNodePtr = m1->previousNodePtr;
    }
    const MacroHistory* history = CompleteToken( result );
    if( keepMacros && history ) {
        result->nodePtr = _PRAGMA( history->macroName, &loc );
    }
    return result;
}

/*
 * set to TRUE to allow wires to be automaticly declared
 */
static int autoDeclareWires = TRUE;

#include "parse.h"
#ifdef __cplusplus
    int yylex (YYSTYPE *);
#endif // __cplusplus

%}
%pure_parser

%union {
	CSymbol*	symbol;
	int		integer;	/* used for integers */
	int		boolean;
	CVector*	vector;
	CNode*		node;
	NodeOp_t	op;
	Coord_t		coord;
	Strength_t	st;
	StrengthPair_t	stpair;
	char*		string;
	MacroToken*	macro;
	char		byte;
	Edge_t		edge;
	CDecl*		decl;
	class DimInfo*	dim;
	CNode_pr        nodepair;
	class HIdentifier* hidentifier;
	class CSpecify*	spblock;
};
	


%nonassoc	LOWEST
%nonassoc	IF
%nonassoc	ELSE

%left		','
%right		'?' ':'
%left		OROR
%left		ANDAND
%left	 	ANDANDAND
%left		'|'
%left <macro>	'^' XNOR
%left		'&'
%left		LOGICAL_EQUALITY LOGICAL_INEQUALITY CASE_EQUALITY CASE_INEQUALITY
%left		LT LE GT GE
%left		LEFT_SHIFT RIGHT_SHIFT LLL RRR
%left		'+' '-'
%left		'*' '/' '%'
%left		'(' '['
%right		'!' '~'
%right		REDUCTION_NOR
%right		REDUCTION_NAND
%right		UNARY
%left  STARSTAR

%token  <macro> EOI 
%token	<macro> ':' '+' '-' '!' '~' '&' '|' ',' REDUCTION_NAND '{' '}'
%token  <macro> REDUCTION_NOR '*' '/' STARSTAR '%' ANDAND OROR '?' '(' ')' 
%token  <macro> LOGICAL_EQUALITY LOGICAL_INEQUALITY CASE_EQUALITY CASE_INEQUALITY
%token  <macro> LT GT LE GE LEFT_SHIFT RIGHT_SHIFT LLL RRR
%token  <macro> PCOLON MCOLON

%token	<symbol>	IDENTIFIER
%token	<string>	UNDEF_MACRO
%token	STARARROW
%token  EQUALARROW
%token  AUTOMATIC
%token  ENDGENERATE
%token  GENERATE
%token  GENVAR
%token  INCDIR
%token  LIBRARY
%token  LOCALPARAM
%token  NOSHOWCANCELLED
%token  PULSESTYLE_ONDETECT
%token  PULSESTYLE_ONEVENT
%token  SHOWCANCELLED
%token  UNSIGNED
%token  UWIRE


%token  <macro>  VRQ
%token  <macro>  PRAGMA
%token  UNIMPLEMENTED
%token	SPECPARAM
%token	<symbol> SETUP
%token	<symbol> HOLD
%token	<symbol> PERIOD
%token	<symbol> WIDTH
%token	<symbol> SKEW
%token	<symbol> RECOVERY
%token	<symbol> SETUPHOLD
%token	<symbol> NOCHANGE
%token  CONFIG
%token  ENDCONFIG
%token  CELL
%token  DESIGN
%token  INSTANCE
%token  LIBLIST
%token  USE
%token	MODULE
%token	MACROMODULE
%token	ENDMODULE
%token	SPECIFY
%token	ENDSPECIFY
%token	PRIMITIVE
%token	ENDPRIMITIVE
%token	TABLE
%token	ENDTABLE
%token	TASK
%token	ENDTASK
%token	FUNCTION
%token	ENDFUNCTION
%token	PARAMETER
%token	INPUT
%token	OUTPUT
%token	INOUT
%token  SIGNED
%token  REALTIME

%token	WIRE
%token	TRI
%token	WAND
%token	TRIAND
%token	WOR
%token	TRIOR
%token	TRI1
%token	TRI0
%token	SUPPLY0
%token	SUPPLY1
%token	TRIREG

%token  IFNONE

%token	SCALARED
%token	VECTORED
%token	REG
%token	TIME
%token	INTEGER
%token	REAL
%token	EVENT
%token	ASSIGN
%token	DEFPARAM

%token	GAND GNAND GNOR GXOR GXNOR GBUF GNOT GBUFIF0 GBUFIF1
%token	GNOTIF0 GNOTIF1 GNMOS GPMOS GRNMOS GRPMOS GCMOS GPULLUP
%token	GPULLDOWN GTRAN GTRANIF0 GTRANIF1 GRTRAN GRTRANIF0 GRTRANIF1

%token	INITIAL_
%token	ALWAYS
%token	IF
%token	CASE
%token	CASEZ
%token	CASEX
%token	ENDCASE
%token	FOREVER
%token	REPEAT
%token	WHILE
%token	FOR
%token	WAIT
%token	ARROW_
%token	ATTRIBUTE_START
%token	ATTRIBUTE_END
%token	DISABLE
%token	DEASSIGN
%token	FORCE
%token	RELEASE
%token	DEFAULT
%token	BEGIN_
%token	END
%token	FORK
%token	JOIN
%token	<symbol> SYSTEM_IDENTIFIER
%token	<macro> STRING
%token	<macro> NUMBER
%token	<macro> BNUMBER
%token	<macro> BASE
%token	<macro> REAL_NUMBER
%token	POSEDGE
%token	NEGEDGE
%token	<edge> EDGE
%left 	OR
%token	SMALL
%token	MEDIUM
%token	LARGE
%token STRONG0 STRONG1 PULL0 PULL1 WEAK0 WEAK1
%token	<edge> DOUBLE_DIGIT
%token  GRCMOS
%token  HIGHZ0
%token  HIGHZ1
%token	OUTPUT_SYMBOL
%token	LEVEL_SYMBOL
%token	EDGE_SYMBOL



%token	EOF_

%type   <macro>		module_item
%type   <macro>		module_item_list
%type   <macro>		non_port_module_item_list
%type   <macro>		non_port_module_item
%type   <macro>		non_port_module_item_macro
%type	<node>		module_declaration
%type	<node>		UDP
%type	<macro>		description_list
%type	<node>		description
%type	<macro>		rp
%type	<node>		inout_declaration
%type	<node>		input_declaration
%type	<node>		output_declaration
%type	<coord>		location
%type   <node>		comments
%type   <node>		task_port_list
%type   <node>		task_port_list_1
%type	<node>		task_item_declaration
%type	<node>		task_item_declaration_list_o
%type   <node>		task_item_declaration_list
%type   <node>		function_port_list
%type   <node>		function_port_list_1
%type	<node>		function_item_declaration 
%type	<node>		function_item_declaration_list
%type   <node>		tf_input_declaration
%type   <node>		tf_output_declaration
%type   <node>		tf_inout_declaration
%type	<integer>	automatic_o
%type	<nodepair>	constant_expression_clist
%type   <node>		case_generate_item
%type   <node>		case_generate_item_list 
%type	<node>	  	genvar_iteration
%type	<node>	  	genvar_initialization
%type	<node>		module_or_generate_item
%type	<node>		module_or_generate_item_list
%type	<nodepair>	module_or_generate_item_list_o
%type	<node>		module_or_generate_item_declaration
%type	<node>		net_assignment
%type	<nodepair>	list_of_net_assignments
%type	<node>		net_declaration
%type	<node>		static_declaration
%type	<node>		task_declaration
%type	<node>		function_declaration
%type	<nodepair>	gate_instance_clist
%type	<node>		gate_instance
%type	<node>		gate_description
%type	<node>		gate_instantiation
%type	<node>		UDP_or_module_instantiation
%type	<node>		parameter_override
%type	<node>		module_instance
%type	<nodepair>	module_instance_clist
%type	<nodepair>	list_of_assignments
%type	<node>		continuous_assign
%type	<node>		cont_assignment
%type	<node>		initial_statement
%type	<node>		always_statement
%type	<node>		generate_region
%type	<node>		conditional_generate_construct
%type   <node>		loop_generate_construct
%type	<node>		if_generate_construct
%type	<node>		if_generate_construct_elsepart
%type	<node>		case_generate_construct
%type	<node>		genvar_declaration
%type   <node>		generate_block
%type   <node>		generate_block_or_null
%type	<nodepair>	list_of_genvar_identifiers
%type	<boolean>	module_keyword
%type	<macro>		expression
%type	<node>		genvar_expression
%type	<macro>		primary
%type	<macro>		string
%type 	<macro>		number
%type	<macro>		constant_primary
%type	<node>		genvar_primary
%type	<macro>		constant_expression
%type	<macro>		constant_expression1
%type	<node>		param_constant_expression
%type	<macro>		mintypmax_expression
%type	<macro>		constant_mintypmax_expression
%type	<macro>		concatenation
%type	<macro>		constant_concatenation
%type   <node>		function_call
%type   <node>		constant_function_call
%type   <node>		system_function_call
%type   <node>		constant_system_function_call
%type	<symbol>	identifier
%type	<symbol> 	variable_identifier
%type	<symbol> 	module_identifier
%type	<symbol>	cell_identifier
%type	<symbol>	library_identifier
%type	<symbol>	port_identifier
%type	<symbol>	udp_identifier
%type	<symbol>	input_port_identifier
%type	<symbol>	output_port_identifier
%type	<symbol>	output_reg_port_identifier
%type	<symbol>	function_identifier
%type	<symbol>	genvar_identifier
%type	<symbol>	parameter_or_specparam_identifier
%type	<symbol>	parameter_or_specparam_or_genvar_identifier
%type	<symbol>	config_identifier
%type	<symbol>	topmodule_identifier
%type	<symbol>	instance_identifier
%type	<symbol>	generate_block_identifier
%type	<nodepair> 	expression_clist
%type	<macro> 	expression_catlist
%type	<macro> 	constant_expression_catlist
%type	<node> 		port_expression_o
%type	<node> 		port_expression
%type	<node> 		port_reference_list
%type	<node> 		port_reference
%type	<node>		statement
%type	<node>		statement_noattr
%type	<node>		assignment
%type	<node>		variable_assignment
%type	<node>		lvalue
%type	<node>		lval_normal
%type	<macro>		statement_list_macro
%type	<nodepair>	statement_list
%type	<nodepair>	statement_list_noattr
%type	<node>		block_item_declaration
%type	<node>		block_item_declaration_list
%type	<node>		seq_block
%type	<node>		seq_block_1
%type	<node>		par_block
%type	<node>		event_control
%type	<node>		event_expression
%type	<node>		statement_or_null
%type	<node>		statement_or_null_noattr
%type	<node>		statement1
%type	<node>		delay_or_event_control
%type	<node>		delayN
%type	<node>		delay_control
%type	<nodepair>	elsepart
%type	<nodepair>	case_item_plist
%type	<node>		case_expr
%type	<stpair>	drive_strength drive_strength_o
%type	<st>		strength0 strength1
%type	<node>		delay_o
%type	<node>		xparameter_value_assignment
%type	<macro>		mintypmax_expression_clist
%type	<macro>		constant_mintypmax_expression_clist
%type	<node>		named_port_connection
%type	<nodepair>	named_port_connection_clist
%type	<nodepair>	module_port_connection_clist
%type	<node>		module_port_connection
%type	<node>		list_of_module_connections_o
%type	<node>		lval_concat
%type	<node>		lval_clist
%type	<node>		system_task_enable
%type	<node>		param_expression_clist param_expression
%type	<node>		param_expression_clist1
%type	<node>		param_expression_clist2
%type	<node>		xexpression_clist xexpression
%type	<node>		constant_xexpression_clist constant_xexpression
%type	<nodepair>	terminal_clist
%type	<node>		conditional_expression
%type	<node> 		task_enable
%type	<nodepair>	port_list
%type	<node>		list_of_ports
%type	<node>		list_of_port_declarations
%type	<node>		port_o
%type	<node>		port
%type   <nodepair> 	list_of_real_identifiers real_type
%type   <nodepair> 	list_of_variable_identifiers 
%type   <node> 		variable_type
%type	<nodepair>	decl_identifiers
%type	<node>		decl_identifier
%type	<node>          param_override 
%type   <nodepair>      list_of_defparam_assignments
%type	<node>		path_declaration
%type	<nodepair>      list_of_path_inputs
%type	<nodepair>      list_of_path_outputs
%type	<node>		data_source_expression
%type	<node> 	        specify_input_terminal_descriptor
%type	<node>          specify_output_terminal_descriptor
%type	<integer>	edge_identifier
%type	<integer>	polarity_operator
%type	<integer>	polarity_operator1
%type	<integer>	path_operator
%type	<node>		conditional_port_expression
%type	<node>		path_delay_value
%type	<node>		delay_value
%type	<node>		path_description
%type	<node>          timing_input_terminal_descriptor
%type	<node>          timing_check_event_control
%type	<node>          timing_check_event_control_no_empty
%type	<node>          notify_clause
%type	<node>		system_timing_check
%type	<node>          timing_check_event
%type	<node>          controlled_timing_check_event
%type	<node>	        notify_register
%type	<node>          timing_check_condition
%type	<node>          scalar_expression	
%type	<node>          scalar_constant		
%type	<edge>		edge_list
%type	<nodepair>	list_of_event_identifiers
%type	<node>		event_identifier
%type	<dim>		dimension_range
%type	<dim>		constant_dimension_range
%type	<node>		module_ports_params_items
%type	<node>		port_declaration
%type	<node>		port_declaration_list
%type	<node>		port_declaration_list_1
%type   <node>		port_decl_identifier
%type	<nodepair>	port_decl_identifiers
%type	<stpair>	drive_charge_strength_o
%type	<stpair>	charge_strength
%type	<nodepair>	attribute_instance_list
%type	<node>		attribute_instance_list_o
%type	<node>		attribute_instance_pragma_list_o
%type	<node>		attribute_instance_list_pragma_o
%type	<node>		attribute_instance
%type   <symbol>        hierarchical_parameter_identifier
%type   <hidentifier>   hierarchical_event_identifier
%type   <symbol>        hierarchical_task_identifier
%type   <symbol>        hierarchical_function_identifier
%type   <hidentifier>   hierarchical_identifier
%type   <hidentifier>   hierarchical_identifier_1
%type   <macro>         pragma_macro
%type   <macro>         pragmas_macro
%type   <node>          pragmas
%type   <node>          pragmas_o
%type	<node>		output_port
%type	<node>		input_port
%type	<node>		input_port_list
%type   <node>		udp_input_declaration
%type   <node>		udp_output_declaration
%type   <node>		udp_reg_declaration
%type	<node>		udp_port_list
%type	<node>		udp_port_declaration
%type	<node>		udp_port_declaration_list
%type	<node>		udp_ports_declarations
%type	<node>		udp_declaration_port_list
%type   <node>          udp_input_declaration_list
%type   <node>          udp_input_declaration_list_1
%type   <node>          udp_initial_statement
%type	<node>		init_val
%type <node>           udp_entry_list
%type <node>           entry_list
%type <node>           entry
%type <node>           input_list
%type <node>           input_symbol
%type <node>           edge_indicator
%type <node>           current_state
%type <node>           next_state
%type <byte>           level_symbol
%type <byte>           output_symbol
%type <byte>           edge_symbol
%type <node>	       attr_clear
%type <node>	       last_attr
%type <node>	       list_of_param_assignments
%type <node>		specparam_declaration
%type <node>		local_parameter_declaration
%type <node>		parameter_declaration
%type <node>		param_assignment
%type <node>		list_of_parameter_declarations 
%type <node>		list_of_parameter_declarations_1 
%type <node>		specify_block 
%type <nodepair>	specify_items 
%type <node>		specify_item 
%type <node> 		cmos_switch_instance
%type <node> 		enable_gate_instance
%type <node> 		mos_switch_instance
%type <node> 		n_input_gate_instance
%type <node> 		n_output_gate_instance
%type <node> 		pass_enable_switch_instance
%type <node> 		pass_switch_instance
%type <node> 		pull_gate_instance

%start source_text
/* direct c interface 
generate_item_declaration ::= add: dip_import_export
dpi_import_export ::=
import dpi_spec_string [ dpi_function_import_property ] [ c_identifier = ] dpi_function_proto ;
| import dpi_spec_string [ dpi_task_import_property ] [ c_identifier = ] dpi_task_proto ;
| export dpi_spec_string [ c_identifier = ] function function_identifier ;
| export dpi_spec_string [ c_identifier = ] task task_identifier ;
dpi_spec_string ::= "DPI-C" | "DPI"
dpi_function_import_property ::= context | pure
dpi_task_import_property ::= context
dpi_function_proto8,9 ::= function_prototype
dpi_task_proto9 ::= task_prototype
function_prototype ::= function function_data_type function_identifier ( [ tf_port_list ] )
task_prototype ::= task task_identifier ( [ tf_port_list ] )
*/

%%
/********************************
 * Support constructs
 ********************************/
comments 
	: {$$ = GetComments(); }	
	;

location
	:{ $<coord>$ = loc; } 
	;

/*********************************
 * A.1.2 Verilog source text
 *********************************/

source_text
	: description_list EOF_
	{
	        codeList = _LINK(codeList,Macro2Node($1));
		DoTopLevelComments();
	}
	| error
	| EOF_
	;

description_list
	: description
	{ 
	    $$ = AnnotateMacroToken( loc, $1 ); 
	}
	| pragma_macro
	{ 
	    $$ = $1;
	    const MacroHistory* history = CompleteToken( $$ );
	    if( keepMacros && history ) {
	        $$->nodePtr = _PRAGMA( history->macroName, &loc );
	    }
	}
	| description_list description
	{
	    MacroToken* mt = AnnotateMacroToken( loc, $2 );
	    $$ = _LINK($1,mt);
	}
	| description_list pragma_macro
	{ $$ = _LINK( $1, $2 ); }
	;

description
	: comments module_declaration
	{ $$ = _LINK($1,$2); }
 	| comments UDP
	{ $$ = _LINK($1,$2); }
	| config_declaration
	{ NI(config_section); $$ = NULL; }
	;

module_declaration
	: attribute_instance_list_o attr_clear module_keyword
	  module_identifier
	{
		DoComments();
		MASSERT( currentModule == NULL );
		MASSERT( currentBlock == NULL );
		currentModule = defSymtab.Lookup( $4 );
		if( currentModule == NULL ) {
			currentModule = new(unitStack) CModule( $4, &loc );
		} else {
			currentModule->SetCoord( &loc );
		}
		currentModule->ModuleDirectlyDefined( modulesDirectlyDefined );
		currentModule->ModuleIncluded( modulesIncluded );
		currentModule->SetAttributes( $2 );
		currentModule->Cell( compilerDirectiveCell );
		currentModule->Protect( compilerDirectiveProtect );
		if( compilerDirectiveTimescaleValid ) {
		    currentModule->Precision( compilerDirectivePrecision );
		    currentModule->Unit( compilerDirectiveUnit );
		}
		currentModule->DefaultNetType( compilerDirectiveDefaultNetType );
		currentModule->UnconnectedDrive( compilerDirectiveUnconnectedDrive );
		currentBlock  = currentModule;
		if( currentModule->IsDefined() ) {
			Coord_t* origLoc;
			origLoc = currentModule->GetCoord();
			error( &loc, "module '%s' was previously defined"
				" in '%s' line %d", $4->GetName(), origLoc->filename,
				origLoc->lineno );
		} else {
			currentModule->SetDefined( TRUE );
			currentModule->SetMacroModule( $3 );
		}
		DoTopLevelComments();
		/*
		 * push a scope for input/output declarations
		 * and one for forward references
		 */
		moduleSymtab.PushScope(FALSE);
		frefSymtab = moduleSymtab;	
		currentModule->SetFrefSymtab( frefSymtab );
		moduleSymtab.PushScope(FALSE);
		portSymtab = moduleSymtab;	
		currentModule->SetPortSymtab( portSymtab );
		moduleSymtab.PushScope(FALSE);
		currentModule->SetSymtab( moduleSymtab );
		allowRvalFrefs = 1;
	}
	  module_ports_params_items ENDMODULE
	{
		currentModule->SetPorts( $6 );
		DoComments();
		currentModule->FixupPorts();
		moduleSymtab.PopScope();
		moduleSymtab.PopScope();
		moduleSymtab.PopScope();
		CNode* entry = _MODULE_DEF( currentModule );
		currentModule = NULL;
		currentBlock  = NULL;
		$$ = entry;
	}
	;

module_keyword
	: MODULE
	{ $$ = FALSE; }
	| MACROMODULE
	{ $$ = TRUE; }
	;

module_ports_params_items
	: list_of_ports sc {allowRvalFrefs = 0;} module_item_list_o
	{
		$$ = $1;
	}
	| module_parameter_port_list list_of_ports sc 
           {allowRvalFrefs = 0;} module_item_list_o
	{
		currentModule->InlineParamDecls(1);
		$$ = $2;
	}
	| list_of_port_declarations sc 
	  {allowRvalFrefs = 0;} non_port_module_item_list_o
	{
		currentModule->InlinePortDecls(1);
		$1->PostVisit1( RegisterFrefs, 0 );
		$$ = $1;
	}
	| module_parameter_port_list
	  list_of_port_declarations 
	  sc {allowRvalFrefs = 0;} non_port_module_item_list_o
	{
		currentModule->InlineParamDecls(1);
		currentModule->InlinePortDecls(1);
		$2->PostVisit1( RegisterFrefs, 0 );
		$$ = $2;
	}
	| module_parameter_port_list
	 '(' rp sc {allowRvalFrefs = 0;} module_item_list_o
	{ 
	    currentModule->InlineParamDecls(1);
	    $$ = NULL; 
	}
	| module_parameter_port_list sc {allowRvalFrefs = 0;} module_item_list_o
	{ 
	    currentModule->InlineParamDecls(1);
	    $$ = NULL; 
	}
	| '(' rp sc {allowRvalFrefs = 0;} non_port_module_item_list_o
	{ $$ = NULL; }
	| sc {allowRvalFrefs = 0;} non_port_module_item_list_o
	{ $$ = NULL; }
	;


/*********************************** 
 * A.1.3 Module parameters and ports
 ***********************************/

module_parameter_port_list
    : '#' '(' list_of_parameter_declarations ')' 
    { currentModule->Add( $3 ); }
    ;

list_of_ports
    : '(' port_o ',' port_list comments rp  
    { $$ = _LINK(_LIST($2,$4),$5); }
    | '(' port comments rp  
    { $$ = _LINK($2,$3); }
    ;

list_of_port_declarations
    : '(' port_declaration_list comments rp 
    { 
        CNode* n = _LINK($2,_LINK($3,_PORTLIST_END()));
	currentModule->Add(n);
	$$ = PortDir2PortDef($2); 
    }
    ;

port_list
	: port_o
	{ $$ = $1; }
	| port_list ',' port_o
	{ $$ = _LIST($1,$3); }
	| port_list ',' error
	{ $$ = $1; }
	;

port_o
	: pragmas_o comments
	{
	    CPort* port = new(unitStack) CPort( currentPortName, FALSE, &loc );
	    port->SetConnection( NULL );
	    $$ = _LINK($1,_LINK($2,_PORT_DEF(port)));
	}
 	| port
	;
	
port
	: pragmas_o comments port_expression pragmas_o
	{
		CPort*	port = new(unitStack) 
			CPort( currentPortName, FALSE, &loc );
		port->SetConnection( $3 );
		$$ = _LINK($1,_LINK($2,_LINK(_PORT_DEF(port),$4)));
	}
	| pragmas_o comments '.' port_identifier 
	  '(' port_expression_o rp pragmas_o
	{
		currentPortName = $4;
		CPort*	port = new(unitStack) 
				    CPort( $4, TRUE, &loc );
		moduleSymtab.Add( currentPortName, port );
		port->SetConnection( $6 );
		$$ = _LINK($1,_LINK($2,_LINK(_PORT_DEF( port ),$8)));
	}
	;

port_expression
	: port_reference
	| '{' port_reference_list '}'
	{
		currentPortName = NULL;
		$$ = $2;
	}
	;

port_expression_o
	: port_expression
	| /* empty */
	{
		currentPortName = NULL;
		$$ = NULL;
	}
	;

port_reference_list
	: port_reference
	{
		currentPortName = NULL;
		$$ = $1;
	}
	| port_reference_list ',' port_reference
	{
		currentPortName = NULL;
		$$ = _CAT( $1, $3, &loc );
	}
	;

port_reference
	: port_identifier
	{
		CFref*	fref;
		CDecl*	decl = moduleSymtab.Lookup( $1 );

		if( decl == NULL ) {
			fref = new(unitStack) CFref( $1, &loc );
			list<Decl_t> tl;
			tl.push_back( eINPUT );
			tl.push_back( eOUTPUT );
			tl.push_back( eINOUT );
			fref->ConstrainTypes( tl );
			frefSymtab.Add( $1, fref );
			currentBlock->Add( fref );
			decl = fref;
		}
		currentPortName = $1;
		MASSERT( decl->GetType() == eFREF );
		$$ = _FWD_REF( (CFref*)decl, &loc );
	}
	| port_identifier '[' constant_expression ']'
	{
		CFref*	fref;
		CDecl*	decl = moduleSymtab.Lookup( $1 );

		if( decl == NULL ) {
			fref = new(unitStack) CFref( $1, &loc );
			list<Decl_t> tl;
			tl.push_back( eINPUT );
			tl.push_back( eOUTPUT );
			tl.push_back( eINOUT );
			fref->ConstrainTypes( tl );
			frefSymtab.Add( $1, fref );
			currentBlock->Add( fref );
			decl = fref;
		}
		currentPortName = $1;
		MASSERT( decl->GetType() == eFREF );
		$$ = _ARRAY( _FWD_REF((CFref*)decl, &loc), Macro2Node($3), &loc );
	}
	| port_identifier '[' constant_expression ':' constant_expression ']'
	{
		CFref*	fref;
		CDecl*	decl = moduleSymtab.Lookup( $1 );

		if( decl == NULL ) {
			fref = new(unitStack) CFref( $1, &loc );
			list<Decl_t> tl;
			tl.push_back( eINPUT );
			tl.push_back( eOUTPUT );
			tl.push_back( eINOUT );
			fref->ConstrainTypes( tl );
			frefSymtab.Add( $1, fref );
			currentBlock->Add( fref );
			decl = fref;
		}
		currentPortName = $1;
		MASSERT( decl->GetType() == eFREF );
		MacroToken* mt = BOP( _SLICE, $3, $4, $5 );	
		CNode* select = Macro2Node(mt);
		$$ = _ARRAY( _FWD_REF((CFref*)decl,&loc), select, &loc );
	}
	;

/*********************************** 
 * A.1.4 Module items
 ***********************************/

module_item_list_o
	: /* empty */
	| module_item_list
	{ currentBlock->Add( Macro2Node($1) ); }
	;

module_item_list
	: module_item 
	{ $$ = $1; }
	| module_item_list module_item
	{ $$ = _LINK( $1, $2); }
	;

module_item
	: comments port_declaration sc
	{ $$ = AnnotateMacroToken( loc, _LINK($1,$2) ); }
	| non_port_module_item
	;

non_port_module_item_list_o
	: /* empty */
	| non_port_module_item_list
	{ currentBlock->Add( Macro2Node($1) ); }
	;

non_port_module_item_list
	:  non_port_module_item
	| non_port_module_item_list non_port_module_item
	{ $$ = _LINK($1,$2); }
	;

module_or_generate_item
    : attribute_instance_list_o module_or_generate_item_declaration
    { $$ = $2; }
    | attribute_instance_list_o local_parameter_declaration
    { $$ = $2; }
    | attribute_instance_list_o UDP_or_module_instantiation
    { $$ = $2; }
    | attribute_instance_list_o parameter_override
    { $$ = $2; }
    | attribute_instance_list_o continuous_assign
    { $$ = $2; }
    | attribute_instance_list_o initial_statement
    { $$ = $2; }
    | attribute_instance_list_o always_statement
    { $$ = $2; }
    | attribute_instance_list_o conditional_generate_construct
    { $$ = $2; }
    | attribute_instance_list_o loop_generate_construct
    { $$ = $2; }
    ;

module_or_generate_item_list
	: module_or_generate_item
	| module_or_generate_item_list module_or_generate_item
	{ $$ = _LINK($1,$2); }
	;

module_or_generate_item_list_o
	: /* empty */
	{ $$ = NULL; }
	| module_or_generate_item_list
	{ $$ = $1; }
	;

module_or_generate_item_declaration
    : net_declaration
    | static_declaration
    | task_declaration
    | function_declaration
    | gate_instantiation
    | genvar_declaration
    ;

non_port_module_item
	: comments non_port_module_item_macro
	{ 
	   $$ = $2;
	   $$->nodePtr = _LINK($1,Macro2Node($2)); 
	}
	;

non_port_module_item_macro
	: module_or_generate_item
	{ $$ = AnnotateMacroToken(loc,$1); }
	| generate_region
	{ $$ = AnnotateMacroToken(loc,$1); }
	| attribute_instance_list_o parameter_declaration 
	{ $$ = AnnotateMacroToken(loc,$2); }
	| attribute_instance_list_o specify_block
	{ $$ = AnnotateMacroToken(loc,$2); }
	| pragma_macro
	;

parameter_override
	: DEFPARAM attr_clear list_of_defparam_assignments sc
	{
	    $$ = PrependComments($3);
	}
	;


/*********************************** 
 * A.1.5 Configuration source text
 ***********************************/

// ??? mdh - not fully implemented
config_declaration
    : CONFIG config_identifier sc
      design_statement
      config_rule_statement_list_o 
      ENDCONFIG
    { NI(config_declaration); }
    ;

// ??? mdh - not fully implemented
design_statement
    : DESIGN lib_cell_list_o sc
    { NI(design_statement); }
    ;

// ??? mdh - not fully implemented
lib_cell_list_o
    : /* NULL */
    | lib_cell_list
    { NI(lib_cell_list); }
    ;

// ??? mdh - not fully implemented
lib_cell_list
    : lib_cell
    { NI(lib_cell_list); }
    | lib_cell_list lib_cell
    { NI(lib_cell_list); }
    ;

// ??? mdh - not fully implemented
lib_cell
    : library_identifier "." cell_identifier
    { NI(lib_cell); }
    | cell_identifier
    { NI(lib_cell); }
    ;

// ??? mdh - not fully implemented
config_rule_statement_list_o
    : /* NULL */
    | config_rule_statement_list
    { NI(config_rule_statement_list); }
    ;

// ??? mdh - not fully implemented
config_rule_statement_list
    : config_rule_statement
    | config_rule_statement_list config_rule_statement
    ;

// ??? mdh - not fully implemented
config_rule_statement
    : DEFAULT LIBLIST library_identifier_list_o
    | INSTANCE inst_name LIBLIST library_identifier_list_o
    | INSTANCE inst_name USE lib_cell colon_config_o
    | CELL lib_cell LIBLIST library_identifier_list_o
    | CELL lib_cell USE lib_cell colon_config_o
    ;

// ??? mdh - not fully implemented
colon_config_o
    : /* NULL */
    | ':' CONFIG
    ;

// ??? mdh - not fully implemented
library_identifier_list_o
    : /* NULL */
    | library_identifier_list
    ;

// ??? mdh - not fully implemented
library_identifier_list
    : library_identifier
    {}
    | library_identifier_list library_identifier
    ;

// ??? mdh - not fully implemented
inst_name
    : topmodule_identifier
    {}
    | topmodule_identifier '.' instance_identifier
    {}
    ;

UDP
	: attribute_instance_list_o PRIMITIVE udp_identifier 
	{
		DoComments();
		CDecl* decl = moduleSymtab.LookupTop( $3 );
		if( decl != NULL ) {
			Coord_t*	refLoc = decl->GetCoord();
			error( &loc, "%s was previously defined in file '%s' line %d",
				$3->GetName(), refLoc->filename,
				refLoc->lineno );
		}
		/*	
		 * create new block for function
		 */
		MASSERT( currentBlock == NULL );
		currentPrimitive = defSymtab.Lookup( $3 );
		if( currentPrimitive == NULL ) {
		        currentPrimitive = new(unitStack) CModule( $3, &loc );
		} else {
			currentPrimitive->SetCoord( &loc );
		}
		currentPrimitive->ModuleDirectlyDefined( TRUE );
		currentPrimitive->ModuleIncluded( TRUE );
		currentPrimitive->SetDefined( TRUE );
		currentPrimitive->SetPrimitive( TRUE );
		moduleSymtab.Add( $3, currentPrimitive );
		currentBlock = currentPrimitive;
		currentPrimitive->SetAttributes( currentAttributeList );
		currentAttributeList = NULL;
		DoTopLevelComments();
		/*
		 * push scope for function
		 */
		savePortSymtab = portSymtab;
		moduleSymtab.PushScope(FALSE);
		portSymtab = moduleSymtab;
		currentPrimitive->SetPortSymtab( portSymtab );	
		moduleSymtab.PushScope(FALSE);
	}
	  udp_ports_declarations
	{
		currentPrimitive->SetPorts( $5 );
		currentPrimitive->FixupPorts();
	}
	  udp_body
	  ENDPRIMITIVE
	{
		moduleSymtab.PopScope();
		moduleSymtab.PopScope();
		portSymtab = savePortSymtab;
		CNode* entry = _MODULE_DEF( currentPrimitive );
		currentPrimitive = NULL;
		currentBlock = NULL;
		$$ = entry;
	}
	;

udp_ports_declarations
	: '(' udp_declaration_port_list ')' comments sc
	{
        	CNode* n = _LINK($2,_LINK($4,_PORTLIST_END()));
	        currentPrimitive->Add(n);
		CNode* ports = PortDir2PortDef($2);
		currentPrimitive->InlinePortDecls(1);
		ports->PostVisit1( RegisterFrefs, 0 );
		$$ = ports;
	}
	| '(' udp_port_list ')' sc udp_port_declaration_list
	{ 
	   currentPrimitive->Add($5);
	   $$ = $2; 
	}
	;
	
udp_declaration_port_list
	: udp_output_declaration ',' udp_input_declaration_list 
	{ $$ = _LINK($1,$3); }
	;

udp_input_declaration_list
	: attribute_instance_list_o cleardecl udp_input_spec udp_input_declaration_list_1 
	{ $$ = $4; }
	;

udp_input_declaration_list_1
	: port_decl_identifier ',' udp_input_declaration_list_1
	{ $$ = _LINK( $1, $3 ); }
	| port_decl_identifier pragmas_o
	{ $$ = _LINK($1,$2); }
	| port_decl_identifier ','  attribute_instance_list_o  cleardecl
	  udp_input_spec udp_input_declaration_list_1
	{ $$ = _LINK( $1, $6 ); }
	;	

udp_port_list
	: output_port ',' input_port_list
	{ $$ = _LINK($1,$3); }
	;

input_port_list
	: input_port 
	{ $$ = $1; }
	| input_port_list ',' input_port 
	{ $$ = _LINK($1,$3); }
	;

input_port
	: comments input_port_identifier
	{
		CFref*	fref;
		CDecl*	decl = moduleSymtab.Lookup( $2 );

		if( decl == NULL ) {
			fref = new(unitStack) CFref( $2, &loc );
			list<Decl_t> tl;
			tl.push_back( eINPUT );
			fref->ConstrainTypes( tl );
			frefSymtab.Add( $2, fref );
			currentBlock->Add( fref );
			decl = fref;
		}
		MASSERT( decl->GetType() == eFREF );
		CNode* n = _FWD_REF( (CFref*)decl, &loc );
		CPort*	port = new(unitStack) CPort( $2, FALSE, &loc );
		port->SetConnection( n );
		$$ = _LINK($1,_PORT_DEF(port));
	}
	;

output_port
	: comments output_port_identifier
	{
		CFref*	fref;
		CDecl*	decl = moduleSymtab.Lookup( $2 );

		if( decl == NULL ) {
			fref = new(unitStack) CFref( $2, &loc );
			list<Decl_t> tl;
			tl.push_back( eOUTPUT );
			fref->ConstrainTypes( tl );
			frefSymtab.Add( $2, fref );
			currentBlock->Add( fref );
			decl = fref;
		}
		MASSERT( decl->GetType() == eFREF );
		CNode* n = _FWD_REF( (CFref*)decl, &loc );
		CPort*	port = new(unitStack) CPort( $2, FALSE, &loc );
		port->SetConnection( n );
		$$ = _LINK($1,_PORT_DEF(port));
	}
	;

udp_port_declaration_list
	: udp_port_declaration
	| udp_port_declaration_list udp_port_declaration
	{ $$ = _LINK($1,$2); }
	;

udp_port_declaration
	: udp_output_declaration sc
	{ $$ = $1; }
	| udp_input_declaration sc
	{ $$ = $1; }
	| udp_reg_declaration sc
	{ $$ = $1; }
	;

udp_output_declaration
	: attribute_instance_list_o cleardecl udp_output_spec port_decl_identifier
	{ $$ = $4; }
	// add semantic check for REG
	// ??? mdh - this doesn't look correct, constant_expression isn't used
	| attribute_instance_list_o cleardecl udp_output_spec port_decl_identifier '=' constant_expression
	{ $$ = $4; }
	;

udp_output_spec
	: OUTPUT
	{ currentDeclType.type = eOUTPUT; }
	| OUTPUT REG
	{ currentDeclType.type = eOUTPUT; 
	  currentDeclType.subType = eREG; }
	;
udp_input_spec
	: INPUT
	{ currentDeclType.type = eINPUT; }
	;
udp_reg_spec
	: REG
	{ currentDeclType.type = eREG; }
	;

udp_input_declaration
	: attribute_instance_list_o cleardecl udp_input_spec port_decl_identifiers
	{ $$ = $4; }
	;

udp_reg_declaration
	: attribute_instance_list_o cleardecl udp_reg_spec variable_identifier
	{
	    CReg* reg = CreateRegister( $4, currentBlock, currentDeclType );
	    reg->SetPragmas( currentDeclType.pragma );
	    $$ =  _REG_DECL(reg,NULL,&loc);
	}
	;

udp_body
	: udp_initial_statement TABLE udp_entry_list ENDTABLE
	{ currentBlock->Add($1);
	   DoComments(); 
	  currentBlock->Add( _TABLE($3,&loc) ); }
	| TABLE udp_entry_list ENDTABLE
	{ DoComments(); 
	  currentBlock->Add( _TABLE($2,&loc) ); }
	;

udp_entry_list
	: {lex_startUDP();} entry_list
	{ $$ = $2; }
	;

entry_list
	: entry
	{ $$ = $1; }
	| entry_list entry
	{ $$ = _LINK( $1, $2 ); }
	;

	// input_list is overly permissive to make parsing
	// possible, semantic checks need to be added.
entry
	: input_list ':' output_symbol sc
	{ char buf[2] = { $3, 0 };
	  $$ = _TABLE_ENTRY(_LINK( $1, _TABLE_SYMBOL(strdup(buf),&loc)),&loc); }
	| input_list ':' current_state ':' next_state sc
	{ $$ = _TABLE_ENTRY(_LINK(_LINK( $1, $3 ),$5),&loc); }
	;

input_list
	: input_symbol
	| input_list input_symbol
	{ $$ = _LINK($1,$2); }
	;

input_symbol
	: edge_indicator
	| level_symbol
	{ char buf[2] = { $1, 0 };
	  $$ = _TABLE_SYMBOL( strdup(buf), &loc ); }
	;

edge_indicator
	: '(' level_symbol level_symbol ')'
	{ string buf;
	  buf += "(";
	  buf += $2;
	  buf += $3;
	  buf += ")";
	  $$ = _TABLE_SYMBOL(strdup(buf.c_str()),&loc);
	}
	| edge_symbol
	{ char buf[2] = { $1, 0 };
	  $$ = _TABLE_SYMBOL(strdup(buf),&loc);
	}
	;

current_state
	: level_symbol
	{ char buf[2] = { $1, 0 };
	  $$ = _TABLE_SYMBOL(strdup(buf),&loc);
	}
	;

next_state
	: output_symbol
	{ char buf[2] = { $1, 0 };
	  $$ = _TABLE_SYMBOL(strdup(buf),&loc);
	}
	| '-'
	{  
	  $$ = _TABLE_SYMBOL(strdup("-"),&loc);
	}
	;

level_symbol
	: '0' 
	{ $$ = '0'; }
	| '1' 
	{ $$ = '1'; }
	| 'x' 
	{ $$ = 'x'; }
	| 'X' 
	{ $$ = 'x'; }
	| '?' 
	{ $$ = '?'; }
	| 'b' 
	{ $$ = 'b'; }
	| 'B'
	{ $$ = 'B'; }
	;

output_symbol
	: '0' 
	{ $$ = '0'; }
	| '1' 
	{ $$ = '1'; }
	| 'x' 
	{ $$ = 'x'; }
	| 'X'
	{ $$ = 'x'; }
	;

edge_symbol
	: 'r' 
	{ $$ = 'r'; }
	| 'R' 
	{ $$ = 'r'; }
	| 'f' 
	{ $$ = 'f'; }
	| 'F' 
	{ $$ = 'f'; }
	| 'p' 
	{ $$ = 'p'; }
	| 'P' 
	{ $$ = 'p'; }
	| 'n' 
	{ $$ = 'n'; }
	| 'N' 
	{ $$ = 'n'; }
	| '*'
	{ $$ = '*'; }
	;

udp_initial_statement
	: INITIAL_ output_reg_port_identifier '=' init_val sc
	{ $$ = _INIT(_ASSIGN(NULL,Val(FALSE,0,$2,NULL),$4,&loc), &loc); }
	;

init_val
	: scalar_constant
	{
	    if( $1->GetOp() != eVCONSTANT ||
	        ($1->Arg<CVector*>(0)->GetWidth() != 1 &&
		 *$1->Arg<CVector*>(0) != 0 && 
		 *$1->Arg<CVector*>(0) != 1) || 
		$1->Arg<CVector*>(0)->HasZ() ) {
			error( &loc, "illegal initialization constant" );
	    }
	}
	;

task_declaration
	: TASK attr_clear automatic_o IDENTIFIER sc
	{
		DoComments();
		CDecl* decl = moduleSymtab.LookupTop( $4 );
		if( decl != NULL ) {
			Coord_t*	refLoc = decl->GetCoord();
			error( &loc, "%s was previously defined in file '%s' line %d",
				$4->GetName(), refLoc->filename,
				refLoc->lineno );
		}
		/*	
		 * create new block for task
		 */
		MASSERT( currentBlock != NULL );
		currentFunction = new(unitStack) CFunction( $4, &loc, $3 );
		moduleSymtab.Add( $4, currentFunction );
		saveBlock = currentBlock;
		currentBlock = currentFunction;
		currentBlock->SetAttributes( $2 );
		/*
		 * push scope for task
		 */
		savePortSymtab = portSymtab;
		moduleSymtab.PushScope(FALSE);
		portSymtab = moduleSymtab;
		currentFunction->SetPortSymtab( portSymtab );	
		moduleSymtab.PushScope(FALSE);
	  	currentFunction->SetSymtab( moduleSymtab );
	}
	  task_item_declaration_list_o 
	  statement_or_null_noattr ENDTASK
	{
		currentFunction->Add($7);
		currentFunction->Add( PostpendComments($8) );
		currentFunction->FixupPorts();
		moduleSymtab.PopScope();
		moduleSymtab.PopScope();
		portSymtab = savePortSymtab;
		currentModule->Add( currentFunction );
		$$ = _FUNCTION_DEF( currentFunction );
		currentFunction = NULL;
		currentBlock = saveBlock;
	}
	| TASK attr_clear automatic_o IDENTIFIER
	{
		DoComments();
		CDecl* decl = moduleSymtab.LookupTop( $4 );
		if( decl != NULL ) {
			Coord_t*	refLoc = decl->GetCoord();
			error( &loc, "%s was previously defined in file '%s' line %d",
				$4->GetName(), refLoc->filename,
				refLoc->lineno );
		}
		/*	
		 * create new block for task
		 */
		MASSERT( currentBlock != NULL );
		currentFunction = new(unitStack) CFunction( $4, &loc, $3 );
		moduleSymtab.Add( $4, currentFunction );
		saveBlock = currentBlock;
		currentBlock = currentFunction;
		currentBlock->SetAttributes( $2 );
		/*
		 * push scope for task
		 */
		savePortSymtab = portSymtab;
		moduleSymtab.PushScope(FALSE);
		portSymtab = moduleSymtab;
		currentFunction->SetPortSymtab( portSymtab );	
		moduleSymtab.PushScope(FALSE);
	  	currentFunction->SetSymtab( moduleSymtab );
	}
 	'(' task_port_list ')' comments sc
	  block_item_declaration_list statement_or_null ENDTASK
	{
                CNode* n = _LINK($7,_LINK($9,_PORTLIST_END()));
		currentFunction->Add(n);
		currentFunction->Add($11);
		currentFunction->SetPorts(PortDir2PortDef($7));
		currentFunction->InlinePortDecls(1);
		currentFunction->FixupPorts();
		currentBlock->Add( PostpendComments($12) );
		moduleSymtab.PopScope();
		moduleSymtab.PopScope();
		portSymtab = savePortSymtab;
		currentModule->Add( currentFunction );
		$$ = _FUNCTION_DEF( currentFunction );
		currentFunction = NULL;
		currentBlock = saveBlock;
	}
	;

function_declaration
	: FUNCTION cleardecl automatic_o function_range_or_type IDENTIFIER sc
	{
		DoComments();
		CDecl* decl = moduleSymtab.LookupTop( $5 );
		if( decl != NULL ) {
			Coord_t*	refLoc = decl->GetCoord();
			error( &loc, "%s was previously defined in file '%s' line %d",
				$5->GetName(), refLoc->filename,
				refLoc->lineno );
		}
		/*	
		 * create new block for function
		 */
		MASSERT( currentBlock != NULL );
		currentFunction = new(unitStack) CFunction( $5, &loc, $3 );
		moduleSymtab.Add( $5, currentFunction );
		saveBlock = currentBlock;
		currentBlock = currentFunction;
		currentBlock->SetAttributes( currentDeclType.attr );
		/*
		 * push scope for function
		 */
		savePortSymtab = portSymtab;
		moduleSymtab.PushScope(FALSE);
		portSymtab = moduleSymtab;
		currentFunction->SetPortSymtab( portSymtab );	
		moduleSymtab.PushScope(FALSE);
	  	currentFunction->SetSymtab( moduleSymtab );
		/*
		 * create a register to hold output
		 */
		if( currentDeclType.type == eREAL ||
		    currentDeclType.type == eREALTIME ) {
			CReal* output = new(unitStack) CReal( $5, &loc,
				currentDeclType.type );
			currentFunction->SetOutput( output );
			moduleSymtab.Add( $5, output );
		} else {
			CReg* output = new(unitStack) CReg( $5, &loc, currentDeclType.type, FALSE );
			output->SetRange( currentDeclType.range );
			output->SetSigned( currentDeclType.signed_ );
			currentFunction->SetOutput( output );
			moduleSymtab.Add( $5, output );
		}
	}
	  function_item_declaration_list 
	  statement ENDFUNCTION
	{
		currentFunction->Add($8);
		currentBlock->Add( PostpendComments($9) );
		currentFunction->FixupPorts();
		moduleSymtab.PopScope();
		moduleSymtab.PopScope();
		portSymtab = savePortSymtab;
		currentModule->Add( currentFunction );
		$$ = _FUNCTION_DEF( currentFunction );
		currentFunction = NULL;
		currentBlock = saveBlock;
	}
	| FUNCTION cleardecl automatic_o function_range_or_type  IDENTIFIER
	{
		DoComments();
		CDecl* decl = moduleSymtab.LookupTop( $5 );
		if( decl != NULL ) {
			Coord_t*	refLoc = decl->GetCoord();
			error( &loc, "%s was previously defined in file '%s' line %d",
				$5->GetName(), refLoc->filename,
				refLoc->lineno );
		}
		/*	
		 * create new block for function
		 */
		MASSERT( currentBlock != NULL );
		currentFunction = new(unitStack) CFunction( $5, &loc, $3 );
		moduleSymtab.Add( $5, currentFunction );
		saveBlock = currentBlock;
		currentBlock = currentFunction;
		currentBlock->SetAttributes( currentDeclType.attr );
		/*
		 * push scope for function
		 */
		savePortSymtab = portSymtab;
		moduleSymtab.PushScope(FALSE);
		portSymtab = moduleSymtab;
		currentFunction->SetPortSymtab( portSymtab );	
		moduleSymtab.PushScope(FALSE);
	  	currentFunction->SetSymtab( moduleSymtab );
		/*
		 * create a register to hold output
		 */
		if( currentDeclType.type == eREAL ||
		    currentDeclType.type == eREALTIME ) {
			CReal* output = new(unitStack) CReal( $5, &loc,
				currentDeclType.type );
			currentFunction->SetOutput( output );
			moduleSymtab.Add( $5, output );
		} else {
			CReg* output = new(unitStack) CReg( $5, &loc, currentDeclType.type, FALSE );
			output->SetRange( currentDeclType.range );
			output->SetSigned( currentDeclType.signed_ );
			currentFunction->SetOutput( output );
			moduleSymtab.Add( $5, output );
		}
	}
 	'(' function_port_list ')' comments sc
	  block_item_declaration_list statement ENDFUNCTION
	{
            	CNode* n = _LINK($8,_LINK($10,_PORTLIST_END()));
	   	currentFunction->Add(n);
		currentFunction->SetPorts(PortDir2PortDef($8)); 
		currentFunction->Add($12);
		currentFunction->InlinePortDecls(1);
		currentFunction->FixupPorts();
		currentBlock->Add( PostpendComments($13) );
		moduleSymtab.PopScope();
		moduleSymtab.PopScope();
		portSymtab = savePortSymtab;
		currentModule->Add( currentFunction );
		$$ = _FUNCTION_DEF( currentFunction );
		currentFunction = NULL;
		currentBlock = saveBlock;
	}
	;


function_port_list
	: attribute_instance_pragma_list_o cleardecl
	  tf_input_spec function_port_list_1 
	{ $$ = $4; }
	;

function_port_list_1
	: port_decl_identifier ',' function_port_list_1
	{ $$ = _LIST($1,$3); }
	| port_decl_identifier pragmas_o
	{ $$ = _LINK($1,$2); }
	| port_decl_identifier ','  attribute_instance_pragma_list_o cleardecl
	  tf_input_spec function_port_list_1
	{ $$ = _LIST($1,$6); }
	;	


task_port_list
	: attribute_instance_list_o cleardecl
	  tf_input_spec task_port_list_1 
	{ $$ = $4; }
	| attribute_instance_list_o cleardecl
	  tf_output_spec task_port_list_1 
	{ $$ = $4; }
	| attribute_instance_list_o cleardecl
	  tf_inout_spec task_port_list_1 
	{ $$ = $4; }
	;

task_port_list_1
	: port_decl_identifier ',' task_port_list_1
	{ $$ = _LIST($1,$3); }
	| port_decl_identifier pragmas_o
	{ $$ = _LINK($1,$2); }
	| port_decl_identifier ','  attribute_instance_list_o cleardecl
	  tf_input_spec task_port_list_1
	{ $$ = _LIST($1,$6); }
	| port_decl_identifier ','  attribute_instance_list_o cleardecl
	  tf_output_spec task_port_list_1
	{ $$ = _LIST($1,$6); }
	| port_decl_identifier ','  attribute_instance_list_o cleardecl
	  tf_inout_spec task_port_list_1
	{ $$ = _LIST($1,$6); }
	;	


automatic_o
	: /* NULL */
	{ $$ = 0; } 
	| AUTOMATIC
	{ $$ = 1; } 
	;

task_item_declaration_list_o
	: pragmas_o
	| task_item_declaration_list pragmas_o
	{ $$ = _LINK($1,$2); }
	;

task_item_declaration_list
	: task_item_declaration
	| task_item_declaration_list task_item_declaration
	{ $$ = _LINK($1,$2); }
	;

task_item_declaration 
	: block_item_declaration
	| tf_input_declaration sc
	| tf_output_declaration sc
	| tf_inout_declaration sc
	;

function_item_declaration_list
	: function_item_declaration
	| function_item_declaration_list function_item_declaration
	{ $$ = _LINK($1,$2); }
	;

function_item_declaration 
	: block_item_declaration
	| tf_input_declaration sc
	;

tf_input_declaration
	: attribute_instance_list_pragma_o cleardecl 
	  tf_input_spec port_decl_identifiers
	{ $$ = _LINK($1,$4); }
	;

block_item_declaration
	: attribute_instance_list_pragma_o parameter_declaration
	{ $$ = _LINK($1,$2); }
	| attribute_instance_list_pragma_o local_parameter_declaration
	{ $$ = _LINK($1,$2); }
	| attribute_instance_list_pragma_o static_declaration
	{ $$ = _LINK($1,$2); }
	;

tf_input_spec
	: INPUT signed_o range_o
	{ currentDeclType.type = eINPUT; }
	| INPUT REG signed_o range_o
	{ currentDeclType.type = eINPUT;
	  currentDeclType.subType = eREG; }
	| INPUT INTEGER
	{ 
	   currentDeclType.type    = eINPUT; 
	   currentDeclType.subType = eINT;
	}
	| INPUT REAL
	{ 
	   currentDeclType.type    = eINPUT; 
	   currentDeclType.subType = eREAL;
	}
	| INPUT REALTIME
	{ 
	   currentDeclType.type    = eINPUT; 
	   currentDeclType.subType = eREALTIME;
	}
	| INPUT TIME
	{ 
	   currentDeclType.type    = eINPUT; 
	   currentDeclType.subType = eTIME;
	}
	;

tf_output_declaration
	: attribute_instance_list_pragma_o cleardecl 
	  tf_output_spec port_decl_identifiers 
	{ $$ = _LINK($1,$4); }
	;

tf_output_spec
	: OUTPUT signed_o range_o
	{ currentDeclType.type = eOUTPUT; }
	| OUTPUT REG signed_o range_o
	{ currentDeclType.type = eOUTPUT;
	  currentDeclType.subType = eREG; }
	| OUTPUT INTEGER
	{ 
	   currentDeclType.type    = eOUTPUT; 
	   currentDeclType.subType = eINT;
	}
	| OUTPUT REAL
	{ 
	   currentDeclType.type    = eOUTPUT; 
	   currentDeclType.subType = eREAL;
	}
	| OUTPUT REALTIME
	{ 
	   currentDeclType.type    = eOUTPUT; 
	   currentDeclType.subType = eREALTIME;
	}
	| OUTPUT TIME
	{ 
	   currentDeclType.type    = eOUTPUT; 
	   currentDeclType.subType = eTIME;
	}
	;

tf_inout_declaration
	: attribute_instance_list_pragma_o cleardecl 
	  tf_inout_spec port_decl_identifiers 
	{ $$ = _LINK($1,$4); }
	;

tf_inout_spec
	: INOUT signed_o range_o
	{ currentDeclType.type = eINOUT; }
	| INOUT REG signed_o range_o
	{ currentDeclType.type = eINOUT; 
	  currentDeclType.subType = eREG; }
	| INOUT INTEGER
	{ 
	   currentDeclType.type    = eINOUT; 
	   currentDeclType.subType = eINT;
	}
	| INOUT REAL
	{ 
	   currentDeclType.type    = eINOUT; 
	   currentDeclType.subType = eREAL;
	}
	| INOUT REALTIME
	{ 
	   currentDeclType.type    = eINOUT; 
	   currentDeclType.subType = eREALTIME;
	}
	| INOUT TIME
	{ 
	   currentDeclType.type    = eINOUT; 
	   currentDeclType.subType = eTIME;
	}
	;

function_range_or_type
	: /* empty */
	{
		currentDeclType.type   = eREG;
	}
	| SIGNED
	{
		currentDeclType.signed_ = TRUE;
	}
	| range
	{
		currentDeclType.type   = eREG;
	}
	| SIGNED range
	{
		currentDeclType.type    = eREG;
		currentDeclType.signed_ = TRUE;
	}
	| INTEGER
	{
		currentDeclType.scalar = TRUE;
		currentDeclType.type   = eINT;
		currentDeclType.scalar = FALSE;
		currentDeclType.signed_ = TRUE;
		currentDeclType.range   = _RANGE(_INT32(31), _INT32(0));
	}
	| REAL
	{
		currentDeclType.scalar = TRUE;
		currentDeclType.type   = eREAL;
	}
	| REALTIME
	{
		currentDeclType.scalar = TRUE;
		currentDeclType.type   = eREALTIME;
	}
	| TIME
	{
		currentDeclType.scalar = TRUE;
		currentDeclType.type   = eTIME;
		currentDeclType.scalar = FALSE;
		currentDeclType.signed_ = FALSE;
		currentDeclType.range   = _RANGE(_INT32(63), _INT32(0));
	}
	;

/*
A.2 Declarations
*/


list_of_parameter_declarations
	: cleardecl PARAMETER signed_o yrange 
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eNONE; } 
	  list_of_parameter_declarations_1 
	{ $$ = $6; }
	| cleardecl PARAMETER INTEGER
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eINT; } 
	  list_of_parameter_declarations_1 
	{ $$ = $5; }
	| cleardecl PARAMETER REAL
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eREAL; } 
	  list_of_parameter_declarations_1 
	{ $$ = $5; }
	| cleardecl PARAMETER REALTIME
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eREALTIME; } 
	  list_of_parameter_declarations_1 
	{ $$ = $5; }
	| cleardecl PARAMETER TIME
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eTIME; } 
	  list_of_parameter_declarations_1 
	{ $$ = $5; }
	;
list_of_parameter_declarations_1
	: param_assignment ',' list_of_parameter_declarations_1 
	{ 
	   $$ = _LINK($1,$3);
	   yyerrok; 
	}
	| param_assignment 
	| param_assignment ',' cleardecl PARAMETER signed_o yrange 
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eNONE; } 
	  list_of_parameter_declarations_1 
	{ 
	   $$ = _LINK($1,$8);
	   yyerrok; 
	}
	| param_assignment ',' cleardecl PARAMETER INTEGER 
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eINT; } 
	  list_of_parameter_declarations_1 
	{ 
	   $$ = _LINK($1,$7);
	   yyerrok; 
	}
	| param_assignment ',' cleardecl PARAMETER TIME 
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eTIME; } 
	  list_of_parameter_declarations_1 
	{ 
	   $$ = _LINK($1,$7);
	   yyerrok; 
	}
	| param_assignment ',' cleardecl PARAMETER REAL 
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eREAL; } 
	  list_of_parameter_declarations_1 
	{ 
	   $$ = _LINK($1,$7);
	   yyerrok; 
	}
	| param_assignment ',' cleardecl PARAMETER REALTIME 
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.inlined = TRUE;
	    currentDeclType.subType = eREALTIME; } 
	  list_of_parameter_declarations_1 
	{ 
	   $$ = _LINK($1,$7);
	   yyerrok; 
	}
	| error
	{ $$ = NULL; }
	;

cleardecl
	: /* NULL */
	  { currentDeclType.type = eNONE; 
	    currentDeclType.scalar = TRUE;
	    currentDeclType.range = NULL;
	    currentDeclType.pragma = NULL;
	    currentDeclType.signed_ = 0;
	    currentDeclType.delay = NULL;
	    currentDeclType.stpair.s0 = eUNDEFINED;
	    currentDeclType.stpair.s1 = eUNDEFINED;
	    currentDeclType.dimension = 0;
	    currentDeclType.scalared = 0;
	    currentDeclType.vectored = 0;
	    currentDeclType.attr = currentAttributeList; 
	    currentAttributeList = NULL;
	    currentDeclType.subType = eNONE; 
	    currentDeclType.inlined = FALSE; }
	;

parameter_declaration
	: cleardecl PARAMETER signed_o yrange 
	  { currentDeclType.type = ePARAM; }
	  list_of_param_assignments sc
	{ $$ = $6; } 
	| cleardecl PARAMETER INTEGER 
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.subType = eINT; }
	  list_of_param_assignments sc
	{ $$ = $5; } 
	| cleardecl PARAMETER REAL
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.subType = eREAL; }
	  list_of_param_assignments sc
	{ $$ = $5; } 
	| cleardecl PARAMETER REALTIME
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.subType = eREALTIME; }
	  list_of_param_assignments sc
	{ $$ = $5; } 
	| cleardecl PARAMETER TIME
	  { currentDeclType.type = ePARAM; 
	    currentDeclType.subType = eTIME; }
	  list_of_param_assignments sc
	{ $$ = $5; } 
	;

local_parameter_declaration
	: cleardecl LOCALPARAM signed_o yrange 
	  { currentDeclType.type = eLOCALPARAM; }
	  list_of_param_assignments sc
	{ $$ = $6; } 
	| cleardecl LOCALPARAM INTEGER 
	  { currentDeclType.type = eLOCALPARAM; 
	    currentDeclType.subType = eINT; }
	  list_of_param_assignments sc
	{ $$ = $5; } 
	| cleardecl LOCALPARAM REAL
	  { currentDeclType.type = eLOCALPARAM; 
	    currentDeclType.subType = eREAL; }
	  list_of_param_assignments sc
	{ $$ = $5; } 
	| cleardecl LOCALPARAM REALTIME
	  { currentDeclType.type = eLOCALPARAM; 
	    currentDeclType.subType = eREALTIME; }
	  list_of_param_assignments sc
	{ $$ = $5; } 
	| cleardecl LOCALPARAM TIME
	  { currentDeclType.type = eLOCALPARAM; 
	    currentDeclType.subType = eTIME; }
	  list_of_param_assignments sc
	{ $$ = $5; } 
	;

list_of_param_assignments
	: param_assignment
	| list_of_param_assignments ',' param_assignment
	{ 
	    $$ = _LINK($1,$3);
	    yyerrok;
	}
	| error
	{ $$ = NULL; }
	;

/* Note: this isn't in the bnf in the LRM, but is in the examples */
param_constant_expression
	: constant_expression ':' constant_expression ':' constant_expression
	{
		$$ = Macro2Node(TOP( _MTM, $1, $2, $3, $4, $5 ));
	}
	| constant_expression
	{ $$ = Macro2Node($1); }
	;

param_assignment
	: IDENTIFIER '=' param_constant_expression
	{
		CParam* 	param;
		CDecl*		decl;

		DoComments();
		decl = moduleSymtab.LookupTop( $1 );
		if( decl != NULL && decl->GetType() != eFREF ) {
			Coord_t*	refLoc = decl->GetCoord();
			error( &loc, "%s was previously defined in file '%s' line %d",
				$1->GetName(), refLoc->filename,
				refLoc->lineno );
			$$ = &errorNode;
		} else {
			param = new(unitStack) CParam( $1, &loc, 
						currentDeclType.type );
			param->SetPragmas( currentDeclType.pragma );
			param->SetRange( currentDeclType.range );
			param->SetSigned( currentDeclType.signed_ );
		        param->SetInlined( currentDeclType.inlined );
			param->SetExpression( $3);
			param->Specify( inSpecifyBlock );
			param->SetAttributes( currentDeclType.attr ?
						currentDeclType.attr->Clone() :
						NULL );
			param->SetSubType( currentDeclType.subType );
			MASSERT( currentBlock != NULL );
			currentBlock->Add( param );
			moduleSymtab.Add( $1, param );
			param->SetDeclStatementCreated();
			if( inSpecifyBlock ) {
			    $$ = _SPECPARAM_DECL(param,&loc);
			} else {
			    $$ = _PARAM_DECL(param,&loc);
			}
		}
	}
	;

static_declaration
	: cleardecl reg_spec list_of_variable_identifiers sc
	{
		$$ = PrependComments($3);
	}
	| cleardecl event_spec list_of_event_identifiers sc
	{
		$$ = PrependComments($3);
	}
	| cleardecl real_spec list_of_real_identifiers sc
	{
		$$ = PrependComments($3);
	}
	;
port_declaration_list
	: attribute_instance_list_pragma_o cleardecl
	  inout_type port_declaration_list_1 
	{ $$ = _LINK( $1,$4); }
	| attribute_instance_list_pragma_o cleardecl 
	  input_type port_declaration_list_1 
	{ $$ = _LINK( $1,$4); }
	| attribute_instance_list_pragma_o cleardecl
	  output_type port_declaration_list_1 
	{ $$ = _LINK( $1,$4); }
	;

port_declaration_list_1
	: port_decl_identifier ',' port_declaration_list_1
	{ $$ = _LINK($1,$3); }
	| port_decl_identifier pragmas_o
	{ $$ = _LINK($1,$2); }
	| port_decl_identifier ',' attribute_instance_pragma_list_o cleardecl
	  inout_type port_declaration_list_1
	{ $$ = _LINK($1,$6); }
	| port_decl_identifier ','  attribute_instance_pragma_list_o cleardecl
	  input_type port_declaration_list_1
	{ $$ = _LINK($1,$6); }
	| port_decl_identifier ','  attribute_instance_pragma_list_o cleardecl
	  output_type port_declaration_list_1
	{ $$ = _LINK($1,$6); }
	;	

port_declaration
	: attribute_instance_list_o cleardecl inout_declaration
	{ $$ = $3; }
	| attribute_instance_list_o cleardecl input_declaration
	{ $$ = $3; }
	| attribute_instance_list_o cleardecl output_declaration
	{ $$ = $3; }

inout_declaration
	: inout_type port_decl_identifiers 
	{ $$ = $2; }
	;

inout_type
	: INOUT net_type_o signed_o range_o
	{ currentDeclType.type = eINOUT; } 
	;

input_declaration
	: input_type port_decl_identifiers 
	{ $$ = $2; }
	;

input_type
	: INPUT net_type_o signed_o range_o
	  { currentDeclType.type = eINPUT; } 
	;

output_declaration
	: output_type port_decl_identifiers 
	{ $$ = $2; }
	;

output_type
	: OUTPUT net_type_o signed_o range_o 
	{ currentDeclType.type = eOUTPUT; } 
	| OUTPUT REG signed_o range_o
	  { currentDeclType.type = eOUTPUT;
	    currentDeclType.subType = eREG; } 
	| OUTPUT INTEGER signed_o range_o
	  { currentDeclType.type = eOUTPUT;
	    currentDeclType.subType = eINT; }
	| OUTPUT TIME signed_o range_o
	  { currentDeclType.type = eOUTPUT;
	    currentDeclType.subType = eTIME; }
	;

port_decl_identifiers
	: port_decl_identifier
	{ $$ = $1; }
	| port_decl_identifiers ',' port_decl_identifier
		{ yyerrok;
		  $$ = _LIST($1,$3);
		}
	| error
	{ $$ = &errorNode; }
	| port_decl_identifiers error port_decl_identifier
	{ yyerrok;
	  $$ = _LIST($1,$3);
        }
	| port_decl_identifiers ',' error
	{ $$ = $1; }
	;

port_decl_identifier
	: comments IDENTIFIER
	{
		CPortDir* 	port = NULL;
		CDecl*		decl;
		CPort*		p;

		decl = portSymtab.LookupTop( $2 );
		if( decl != NULL ) {
			Coord_t*	refLoc = decl->GetCoord();
			error( &loc, "%s was previously defined in file '%s' line %d",
				$2->GetName(), refLoc->filename,
				refLoc->lineno );
		} 
		/* 
		 * create new port even if already declared
		 */
		MASSERT( currentModule != NULL ||
			 currentPrimitive != NULL );
		port = new(unitStack) CPortDir( $2, &loc, currentDeclType.type );
		port->SetRange( currentDeclType.range );
		port->SetSigned( currentDeclType.signed_ );
		port->SetPortType( currentDeclType.subType );
		port->SetAttributes( currentDeclType.attr );
		portSymtab.Add( $2, port );
		port->SetDeclStatementCreated();
		DeclType_t newType;
		newType = currentDeclType;
		newType.type = currentDeclType.subType;
		newType.subType = eNONE;
		CReg* reg;
		CReal* real;
		switch( newType.type ) {
		case eREG:
		case eTIME:
		case eINT:
		    reg = CreateRegister( $2, 
		    	currentBlock, newType );
		    moduleSymtab.Add( $2, reg );
		    reg->SetRange( currentDeclType.range );
		    port->SetDecl( reg );
		    break;
		case eREALTIME:
		case eREAL:
		    real = CreateReal( $2, 
		    	currentBlock, newType );
		    moduleSymtab.Add( $2, real );
		    port->SetDecl( real );
		    break;
		case eNONE:
		    {
		    CFref* fref = new(unitStack) CFref( $2, &loc );
		    list<Decl_t> tl;
		    if( currentFunction ) {
		        tl.push_back( eREG );
			tl.push_back( eREAL );
		        tl.push_back( eREALTIME );
		    } else {
		    	tl.push_back( eNET );
		        tl.push_back( eREG );
		    }
		    fref->ConstrainTypes( tl );
		    moduleSymtab.Add( $2, fref );
		    fref->SetRangeInfo( FALSE, currentDeclType.range );
		    currentBlock->Add( fref );
		    port->SetDecl( fref );
		    }
		    break;
		default: 
			    CNet* net = CreateNet( $2, currentBlock, 
			    	       newType, FALSE );
			    moduleSymtab.Add( $2, net );
			    net->SetRange( currentDeclType.range );
			    port->SetDecl( net );
		}
	        $$ = _LINK($1,_PORT_DECL( port, &loc ));
	}
	;

reg_spec
	: REG signed_o yrange
	{
		currentDeclType.type      = eREG;
	}
	/* 
         * note range isn't valid will either integers or time
	 * I've included
	 * it to allow compatibility with existing 
	 * simulator. Issue a warning that the 
	 */
	| INTEGER range_o
	{
		if( currentDeclType.range ) {
		    message( &loc, mRSINI );
		}
		currentDeclType.type   = eINT;
		currentDeclType.scalar = FALSE;
		currentDeclType.range    = _RANGE(_INT32(31),_INT32(0));
	}
	| TIME range_o
	{
		if( currentDeclType.range ) {
		    message( &loc, mRSINT );
		}
		currentDeclType.type    = eTIME;
		currentDeclType.scalar  = FALSE;
		currentDeclType.range    = _RANGE(_INT32(63),_INT32(0));
	}
	;

real_spec
	: REAL
	{
		currentDeclType.type    = eREAL;
		currentDeclType.scalar  = FALSE;
		currentDeclType.range     = NULL;
		currentDeclType.pragma = NULL;
		currentDeclType.scalared = 0;
		currentDeclType.vectored = 0;
		currentDeclType.signed_ = 0;
	}
	| REALTIME
	{
		currentDeclType.type    = eREALTIME;
		currentDeclType.scalar  = FALSE;
		currentDeclType.range   = NULL;
		currentDeclType.pragma = NULL;
		currentDeclType.scalared = 0;
		currentDeclType.vectored = 0;
		currentDeclType.signed_ = 0;
	}
	;

event_spec
	: EVENT
	{
		currentDeclType.type = eDEVENT;
	}
	;

net_declaration
	: cleardecl netspec list_of_net_assignments sc
	{ $$ = $3; }
	| cleardecl netspec decl_identifiers sc
	{ $$ = PrependComments($3); }
	| cleardecl trireg_spec list_of_net_assignments sc
	{ $$ = $3; }
	| cleardecl trireg_spec decl_identifiers sc
	{ $$ = PrependComments($3); }
	;

netspec
	: net_type drive_strength_o scalared_vectored_o signed_o range_o delay_o 
	{
	    currentDeclType.delay  = $6; 
	    currentDeclType.stpair = $2; 
	}
	;

trireg_spec
	: TRIREG
	  drive_charge_strength_o scalared_vectored_o signed_o range_o delay_o
	{
	  currentDeclType.type = eTRIREG;
	  currentDeclType.delay = $6;
	  currentDeclType.stpair = $2;
	}
	;

net_type_o
	: WIRE
	{ currentDeclType.subType = eWIRE; }
	| TRI
	{ currentDeclType.subType = eTRI; }
	| WAND
	{ currentDeclType.subType = eWAND; }
	| TRIAND
	{ currentDeclType.subType = eTRIAND; }
	| WOR
	{ currentDeclType.subType = eWOR; }
	| TRIOR
	{ currentDeclType.subType = eTRIOR; }
	| TRI1
	{ currentDeclType.subType = eTRI1; }
	| TRI0
	{ currentDeclType.subType = eTRI0; }
	| SUPPLY0
	{ currentDeclType.subType = eSUPPLY0; }
	| SUPPLY1
	{ currentDeclType.subType = eSUPPLY1; }
	| /* NULL */
	{ currentDeclType.subType = eNONE; }
	;

net_type
	: WIRE
	{ currentDeclType.type = eWIRE; }
	| TRI
	{ currentDeclType.type = eTRI; }
	| WAND
	{ currentDeclType.type = eWAND; }
	| TRIAND
	{ currentDeclType.type = eTRIAND; }
	| WOR
	{ currentDeclType.type = eWOR; }
	| TRIOR
	{ currentDeclType.type = eTRIOR; }
	| TRI1
	{ currentDeclType.type = eTRI1; }
	| TRI0
	{ currentDeclType.type = eTRI0; }
	| SUPPLY0
	{ currentDeclType.type = eSUPPLY0; }
	| SUPPLY1
	{ currentDeclType.type = eSUPPLY1; }
	;

scalared_vectored_o
	: SCALARED
	{
		currentDeclType.scalared = 1;
		currentDeclType.vectored = 0;
	}
	| VECTORED
	{
		currentDeclType.scalared = 0;
		currentDeclType.vectored = 1;
	}
	| /* empty */
	{
		currentDeclType.scalared = 0;
		currentDeclType.vectored = 0;
	}
	;

continuous_assign
	: assign_spec attr_clear list_of_assignments sc
	{ 
	   $$ = $3; 
	}
	;

assign_spec
	: ASSIGN drive_strength_o delay_o
	{
		currentDeclType.delay = $3;
		currentDeclType.stpair = $2;
	}
	;

list_of_net_assignments
	: net_assignment
	{ $$ = $1; }
	| list_of_net_assignments ',' net_assignment
	{ $$ = _LINK($1,$3); }
	| error
	{ $$ = &errorNode; }
	| list_of_net_assignments error
	{ $$ = $1; }
	| list_of_net_assignments error net_assignment
	{ $$ = _LINK($1,$3); }
	| list_of_net_assignments ',' error
	{ $$ = $1; }
	;

net_assignment
	: IDENTIFIER '='
	{
	    CNet* net = CreateNet( $1, currentBlock, currentDeclType, FALSE );
	    $<decl>$ = net;
	}
	  expression
	  {

		CNode*		n1;
		CNode*		n;
		CDecl*          decl = $<decl>3;
		CNet*           net = (CNet*)decl;
    		n1 = _NET_REF( net ,&loc );
		StrengthPair_t* pair =
			(StrengthPair_t*) unitStack->Alloc( sizeof(StrengthPair_t) );
		*pair = currentDeclType.stpair;
		n = _CASSIGN( pair, currentDeclType.delay, n1, Macro2Node($4), &loc );
		if( currentDeclType.type != eTRIREG &&
		    ( currentDeclType.stpair.s0 == eSMALL ||
		      currentDeclType.stpair.s0 == eMEDIUM ||
		      currentDeclType.stpair.s0 == eLARGE ) ) {
		    error( &loc, "charge strength cannot be applied"
			     " to %s %s in file '%s' line %d",
			     declName[currentDeclType.type],
			     $1->GetName(), loc.filename,
			     loc.lineno );
		}
		if( currentDeclType.type == eTRIREG &&
		    ( currentDeclType.stpair.s0 == eSMALL ||
		      currentDeclType.stpair.s0 == eMEDIUM ||
		      currentDeclType.stpair.s0 == eLARGE ) ) {
		    error( &loc, "charge strength cannot be applied"
			     " to assignment trireg %s in file '%s' line %d",
			     $1->GetName(), loc.filename,
			     loc.lineno );
		}
		if( net != NULL ) {
			CNode* sd = _NET_DECL(net,n,&loc);
			$$ = PrependComments(sd);
		} else {
			$$ = PrependComments(n);
		}
	}
	;

list_of_assignments
	: cont_assignment
	{ $$ = $1; }
	| list_of_assignments ',' cont_assignment
	{ 
	    $$ = _LINK($1,$3);
   	    yyerrok;
	}
	| error
	{ $$ = &errorNode; }
	| list_of_assignments error
	{ $$ = $1; }
	| list_of_assignments error cont_assignment
	{ 
	    $$ = _LINK($1,$3);
   	    yyerrok;
	}
	| list_of_assignments ',' error
	{ $$ = $1; }
	;

cont_assignment
	: lvalue last_attr '='
	  expression
	{
		CNode*		n;
		StrengthPair_t* pair;
		
		ValidateLvalList( $1, TRUE, eNET, eOUTPUT, eINOUT, eNONE );
		pair = (StrengthPair_t*) unitStack->Alloc( sizeof(StrengthPair_t) );
		*pair = currentDeclType.stpair;
		n = _CASSIGN( pair, currentDeclType.delay, $1, Macro2Node($4), &loc );
		n->SetAttributes($2?$2->Clone():NULL);
		$$ = PrependComments(n);
	}
	;

list_of_defparam_assignments
	: param_override
	{ $$ = $1; }
	| list_of_defparam_assignments ',' param_override
	{ $$ = _LIST($1,$3); }
	;

param_override
	: hierarchical_parameter_identifier last_attr '=' constant_expression
 	{
	    $$ = _DEFPARAM( _EXTERNAL_REF($1), Macro2Node($4), &loc );
	    if( $2 ) {
	        $$->SetAttributes($2?$2->Clone():NULL);
	    }
	}
	;

list_of_variable_identifiers
	: variable_type
	{ $$ = $1; }
	| list_of_variable_identifiers ',' variable_type
		{
		  yyerrok;
		  $$ = _LIST($1,$3);
		}
	| error
	{ $$ = &errorNode; }
	| list_of_variable_identifiers error
	{ $$ = $1; }
	| list_of_variable_identifiers error variable_type
	{
		yyerrok;
		$$ = _LIST($1,$3);
	}
	| list_of_variable_identifiers ',' error
	{ $$ = $1; }
	;

variable_type
	: IDENTIFIER
	{
	    CReg* reg = CreateRegister( $1, currentBlock, currentDeclType );
	    reg->SetPragmas( currentDeclType.pragma );
	    $$ = _REG_DECL(reg,NULL,&loc);
	}
	| IDENTIFIER dimension_decl
	{
	    CReg* reg = CreateRegister( $1, currentBlock, currentDeclType );
	    reg->SetPragmas( currentDeclType.pragma );
	    $$ = _REG_DECL(reg,NULL,&loc);
	}
	| IDENTIFIER '=' constant_expression
	{
	    CReg* reg = CreateRegister( $1, currentBlock, currentDeclType );
	    reg->SetPragmas( currentDeclType.pragma );
	    CNode* n = _ASSIGN( NULL, _REG_REF(reg, &loc), Macro2Node($3), &loc );
	    $$ = _REG_DECL( reg, n, &loc );
	}
	;

event_identifier
	: IDENTIFIER
	{
	    CEvent* event = CreateEvent( $1, currentBlock, currentDeclType );
	    event->SetPragmas( currentDeclType.pragma );
	    $$ = _EVENT_DECL(event,&loc);
	}
	| IDENTIFIER dimension_decl
	{
	    CEvent* event = CreateEvent( $1, currentBlock, currentDeclType );
	    event->SetPragmas( currentDeclType.pragma );
	    $$ = _EVENT_DECL(event,&loc);
	}
	;
list_of_event_identifiers
	: event_identifier
	{ $$ = $1; }
	| list_of_event_identifiers ',' event_identifier
	{ $$ = _LIST($1,$3); }
	;

decl_identifier
	: IDENTIFIER
	{
	    if( currentDeclType.type != eTRIREG &&
		( currentDeclType.stpair.s0 != eUNDEFINED ||
		  currentDeclType.stpair.s1 != eUNDEFINED) ) {
		error( &loc, "strength cannot be applied"
			 " to non-assignment %s in file '%s' line %d",
			 $1->GetName(), loc.filename,
			 loc.lineno );
	    }
	    if( currentDeclType.type == eTRIREG &&
		currentDeclType.stpair.s0 != eSMALL && 
		currentDeclType.stpair.s0 != eMEDIUM &&
		currentDeclType.stpair.s0 != eLARGE &&
		currentDeclType.stpair.s0 != eUNDEFINED ) {
		error( &loc, "drive strength cannot be applied"
			 " to non-assignment trireg %s in file '%s' line %d",
			 $1->GetName(), loc.filename,
			 loc.lineno );
	    }
	    CNet* net = CreateNet( $1, currentBlock, currentDeclType, TRUE );
	    $$ = _NET_DECL(net, NULL,&loc);
	}
	| IDENTIFIER dimension_decl
	{
	    if( currentDeclType.type != eTRIREG &&
		( currentDeclType.stpair.s0 != eUNDEFINED ||
		  currentDeclType.stpair.s1 != eUNDEFINED) ) {
		error( &loc, "strength cannot be applied"
			 " to non-assignment %s in file '%s' line %d",
			 $1->GetName(), loc.filename,
			 loc.lineno );
	    }
	    if( currentDeclType.type == eTRIREG &&
		currentDeclType.stpair.s0 != eSMALL && 
		currentDeclType.stpair.s0 != eMEDIUM &&
		currentDeclType.stpair.s0 != eLARGE &&
		currentDeclType.stpair.s0 != eUNDEFINED ) {
		error( &loc, "drive strength cannot be applied"
			 " to non-assignment trireg %s in file '%s' line %d",
			 $1->GetName(), loc.filename,
			 loc.lineno );
	    }
	    CNet* net = CreateNet( $1, currentBlock, currentDeclType, TRUE );
	    $$ = _NET_DECL(net, NULL, &loc);
	}
	;
decl_identifiers
	: decl_identifier
	{ $$ = $1; }
	| decl_identifiers ',' decl_identifier
	{ $$ = _LIST($1,$3); }
	;

real_type
	: IDENTIFIER
	{
	    CReal* real = CreateReal( $1, currentBlock, currentDeclType );
	    real->SetPragmas( currentDeclType.pragma );
	    $$ = _REAL_DECL(real,NULL,&loc);
	}
	| IDENTIFIER dimension_decl
	{
	    CReal* real = CreateReal( $1, currentBlock, currentDeclType );
	    real->SetPragmas( currentDeclType.pragma );
	    $$ = _REAL_DECL(real,NULL,&loc);
	}
	| IDENTIFIER '=' constant_expression
	{
	    CReal* real = CreateReal( $1, currentBlock, currentDeclType );
	    real->SetPragmas( currentDeclType.pragma );
	    CNode* n = _ASSIGN( NULL, _REAL_REF(real,&loc), Macro2Node($3), &loc );
	    $$ = _REAL_DECL(real,n,&loc);
	}
	;
list_of_real_identifiers
	: real_type
	| list_of_real_identifiers ',' real_type
	{ $$ = _LIST($1,$3); }
	;

drive_charge_strength_o
	: /* empty */
	{ $$.s0 = eUNDEFINED; $$.s1 = eUNDEFINED; }
	| charge_strength
	| drive_strength
	;

charge_strength
	: '(' SMALL rp
	{ $$.s0 = eSMALL; $$.s1 = eSMALL; }
	| '(' MEDIUM rp
	{ $$.s0 = eMEDIUM; $$.s1 = eMEDIUM; }
	| '(' LARGE rp
	{ $$.s0 = eLARGE; $$.s1 = eLARGE; }
	;

drive_strength_o
	: /* empty */
	{ $$.s0 = eUNDEFINED; $$.s1 = eUNDEFINED; }
	| drive_strength
	;

drive_strength
	: '(' strength0 ',' strength1 rp
	{
		$$.s1 = $4;
		$$.s0 = $2;
	}
	| '(' strength1 ',' strength0 rp
	{
		$$.s0 = $4;
		$$.s1 = $2;
	}
	;

strength0
	: STRONG0
	{ $$ = eSTRONG; }
	| PULL0
	{ $$ = ePULL; }
	| WEAK0
	{ $$ = eWEAK; }
	| HIGHZ0
	{ $$ = eHIGHZ; }
	| SUPPLY0
	{ $$ = eSUPPLY; }
	;

strength1
	: STRONG1
	{ $$ = eSTRONG; }
	| PULL1
	{ $$ = ePULL; }
	| WEAK1
	{ $$ = eWEAK; }
	| HIGHZ1
	{ $$ = eHIGHZ; }
	| SUPPLY1
	{ $$ = eSUPPLY; }
	;


signed_o 
	: SIGNED
	{ currentDeclType.signed_ = 1; }
	| /* empty */
	{ currentDeclType.signed_ = 0; }
	;
yrange
	: /* empty */
	| pragmas
	{ currentDeclType.pragma = $1; }
	| range
	| range pragmas
	{ currentDeclType.pragma = $2; }
	;


range_o
	: range
	| /* empty */
	;

range
	: '[' constant_expression ':' constant_expression rb
	{
		currentDeclType.scalar = FALSE;
		currentDeclType.range = Macro2Node(BOP(_RANGE, $2, $3, $4 ));
	}
	;

/*
A.3 Primitive instances
*/
gate_instantiation
        : cmos_switchtype attr_clear
	{ currentDriveStrength.s0 = eUNDEFINED;
	  currentDriveStrength.s1 = eUNDEFINED; 
	  currentGateDelay = NULL; }
          cmos_switch_instance
	{ $$ = $4; }
        | n_input_gatetype attr_clear
	{ currentDriveStrength.s0 = eUNDEFINED;
	  currentDriveStrength.s1 = eUNDEFINED; 
	  currentGateDelay = NULL; }
          n_input_gate_instance
	{ $$ = $4; }
        | n_output_gatetype attr_clear
	{ currentDriveStrength.s0 = eUNDEFINED;
	  currentDriveStrength.s1 = eUNDEFINED; 
	  currentGateDelay = NULL; }
          n_output_gate_instance
	{ $$ = $4; }
        | enable_gatetype attr_clear
	{ currentDriveStrength.s0 = eUNDEFINED;
	  currentDriveStrength.s1 = eUNDEFINED; 
	  currentGateDelay = NULL; }
          enable_gate_instance
	{ $$ = $4; }
        | mos_switchtype attr_clear
	{ currentDriveStrength.s0 = eUNDEFINED;
	  currentDriveStrength.s1 = eUNDEFINED; 
	  currentGateDelay = NULL; }
          mos_switch_instance 
	{ $$ = $4; }
        | pulltype attr_clear
	{ currentDriveStrength.s0 = eUNDEFINED;
	  currentDriveStrength.s1 = eUNDEFINED; 
	  currentGateDelay = NULL; }
          pull_gate_instance
	{ $$ = $4; }
        | pass_en_switchtype attr_clear
	{ currentDriveStrength.s0 = eUNDEFINED;
	  currentDriveStrength.s1 = eUNDEFINED; 
	  currentGateDelay = NULL; }
          pass_enable_switch_instance
	{ $$ = $4; }
        | pass_switchtype attr_clear
	{ currentDriveStrength.s0 = eUNDEFINED;
	  currentDriveStrength.s1 = eUNDEFINED; 
	  currentGateDelay = NULL; }
          pass_switch_instance
	{ $$ = $4; }
        ;

cmos_switchtype
	: GCMOS
	{ gateType = CSymbol::Lookup( "cmos", TRUE ); }
	| GRCMOS
	{ gateType = CSymbol::Lookup( "rcmos", TRUE ); }
        ;

n_input_gatetype
        : GAND
	{ gateType = CSymbol::Lookup( "and", TRUE ); }
	| GNAND	
	{ gateType = CSymbol::Lookup( "nand", TRUE ); }
	| GNOR
	{ gateType = CSymbol::Lookup( "nor", TRUE ); }
        /* is a LOGIC but is also a keyword for event expressions */
	| OR
	{ gateType = CSymbol::Lookup( "or", TRUE ); }
	| GXOR
	{ gateType = CSymbol::Lookup( "xor", TRUE ); }
	| GXNOR
	{ gateType = CSymbol::Lookup( "xnor", TRUE ); }
	;

n_output_gatetype
	: GBUF
	{ gateType = CSymbol::Lookup( "buf", TRUE ); }
	| GNOT
	{ gateType = CSymbol::Lookup( "not", TRUE ); }
	;

enable_gatetype
	: GBUFIF0
	{ gateType = CSymbol::Lookup( "bufif0", TRUE ); }
	| GBUFIF1
	{ gateType = CSymbol::Lookup( "bufif1", TRUE ); }
	| GNOTIF0
	{ gateType = CSymbol::Lookup( "notif0", TRUE ); }
	| GNOTIF1
	{ gateType = CSymbol::Lookup( "notif1", TRUE ); }
	;

mos_switchtype
	: GNMOS
	{ gateType = CSymbol::Lookup( "nmos", TRUE ); }
	| GPMOS
	{ gateType = CSymbol::Lookup( "pmos", TRUE ); }
	| GRNMOS
	{ gateType = CSymbol::Lookup( "rnmos", TRUE ); }
	| GRPMOS
	{ gateType = CSymbol::Lookup( "rpmos", TRUE ); }
	;

pulltype
	: GPULLUP
	{ gateType = CSymbol::Lookup( "pullup", TRUE ); }
	| GPULLDOWN
	{ gateType = CSymbol::Lookup( "pulldown", TRUE ); }
	;

pass_en_switchtype
	: GTRANIF0		
	{ gateType = CSymbol::Lookup( "tranif0", TRUE ); }
	| GTRANIF1		
	{ gateType = CSymbol::Lookup( "tranif1", TRUE ); }
	| GRTRANIF0		
	{ gateType = CSymbol::Lookup( "rtranif0", TRUE ); }
	| GRTRANIF1		
	{ gateType = CSymbol::Lookup( "rtranif1", TRUE ); }
        ;

pass_switchtype
	: GTRAN
	{ gateType = CSymbol::Lookup( "tran", TRUE ); }
	| GRTRAN
	{ gateType = CSymbol::Lookup( "rtran", TRUE ); }
	;

// ??? - mdh this are place holders, need to clean and extend this
cmos_switch_instance
	: gate_description
	;
enable_gate_instance
	: gate_description
	;
mos_switch_instance
	: gate_description
	;
n_input_gate_instance
	: gate_description
	;
n_output_gate_instance
	: gate_description
	;
pass_enable_switch_instance
	: gate_description
	;
pass_switch_instance
	: gate_description
	;
pull_gate_instance
	: gate_description
	;


gate_description
        :   gate_instance_clist sc
	{ $$ = $1; }
        | delayN
		{ currentDriveStrength.s0 = eUNDEFINED;
		  currentDriveStrength.s1 = eUNDEFINED; 
		  currentGateDelay = $1; }
          gate_instance_clist sc
	{ $$ = $3; }
        | drive_strength { currentDriveStrength = $1; } delayN
	  { currentGateDelay = $3; } gate_instance_clist sc
	{ $$ = $5; }
        | drive_strength { currentDriveStrength = $1; }
          gate_instance_clist sc
	{ $$ = $3; }
        ;

gate_instance_clist
        : gate_instance
	{ $$ = $1; }
        | gate_instance_clist ',' gate_instance
	{ $$ = _LINK($1,$3); }
        ;

gate_instance
        : IDENTIFIER cleardecl range_o last_attr '(' terminal_clist rp
	{
		CGate*	gate;
		gate = new(unitStack) CGate(
			$1, gateType,
			currentDriveStrength.s0, currentDriveStrength.s1,
			currentGateDelay, $6, &loc );
		gate->SetAttributes( $4?$4->Clone():NULL );
		if( !currentDeclType.scalar ) {
		    gate->SetRange(currentDeclType.range);
		}
		MASSERT( currentModule != NULL );
		$$ = PrependComments(_GATE_REF( gate, &loc ));
	}
        | '(' last_attr terminal_clist rp
	{
		CGate*	gate;
		gate = new(unitStack) CGate(
			NULL,
			gateType,
			currentDriveStrength.s0, currentDriveStrength.s1,
			currentGateDelay, $3, &loc );
		gate->SetAttributes( $2 );
		MASSERT( currentModule != NULL );
		$$ = PrependComments(_GATE_REF( gate, &loc ));
	}
    ;

/*
A.4 Module instantiation and generate construct
*/


UDP_or_module_instantiation
	: IDENTIFIER attr_clear
	{
		DoComments();
		currentInstanceType = $1;
		currentInstanceAttributes = $2;
	}
	  xparameter_value_assignment
	{
		currentInstanceParameters = $4;
	}
	  module_instance_clist sc
	{
		currentInstanceType = NULL;
		$$ = $6;
	}
	;

xparameter_value_assignment
	: /* empty */
	{ $$ = NULL; }
	| pragmas '#' '(' param_expression_clist ')'
	{ $$ = _LINK($1,$4); }
	| pragmas '#' '(' param_expression_clist ')' pragmas
	{ $$ = _LINK($1 ,_LINK($4,$6)); }
	| '#' '(' param_expression_clist ')' pragmas
	{ $$ = _LINK($3,$5); }
	| '#' '(' param_expression_clist ')'
	{ $$ = $3; }
	// ??? mdh - need to add pragma code here
	| '#' delay_value pragmas
	{ $$ = _LINK($2,$3); }
	| pragmas '#' delay_value
	{ $$ = _LINK($1,$3); }
	| pragmas '#' delay_value pragmas
	{ $$ = _LINK($1,_LINK($3,$4)); }
	| '#' delay_value 
	{ $$ = $2; }
	;

delay_value
	: IDENTIFIER
	{ $$ = Val( FALSE, 1, $1, NULL ); }
	| number
	{ $$ = Macro2Node($1); }
	;

mintypmax_expression_clist
	: mintypmax_expression
	| mintypmax_expression_clist ',' mintypmax_expression
	{
		$$ = BOP(_ELIST, $1, $2, $3 );
	}
	;

constant_mintypmax_expression_clist
	: constant_mintypmax_expression
	| constant_mintypmax_expression_clist ',' constant_mintypmax_expression
	{
		$$ = BOP(_ELIST, $1, $2, $3 );
	}
	;



terminal_clist
	: expression
	{ $$ = _ARG( NULL, Macro2Node($1), &loc ); }
	| terminal_clist ',' expression
	{
		CNode* arg1 = _ARG( NULL, Macro2Node($3), &loc );
		$$ = _ELIST( $1, arg1 );
	}
	| error
	{ $$ = NULL; }
	| terminal_clist error
	{ $$ = $1; }
	| terminal_clist error expression
	{
		CNode* arg1 = _ARG( NULL, Macro2Node($3), &loc );
		$$ = _ELIST( $1, arg1 );
	 	yyerrok;
	}
	| terminal_clist ',' error
	{ $$ = $1; }
	;


module_instance_clist
	: module_instance
	{ $$ = $1; }
	| module_instance_clist ',' module_instance
	{ 
	    $$ = _LINK($1,$3);
	    yyerrok; 
	}
	| error
	{ $$ = &errorNode; }
	| module_instance_clist error
	| module_instance_clist error module_instance
		{ yyerrok; }
	| module_instance_clist ',' error
	;

module_instance
	: IDENTIFIER comments cleardecl range_o '(' list_of_module_connections_o rp
	{
		CInstance*	instance;
		instance = new(unitStack) CInstance( $1,
			currentInstanceType,
			currentInstanceParameters, PostpendComments($6), &loc );
		if( !currentDeclType.scalar ) {
		    instance->SetRange( currentDeclType.range );
		}
		instance->SetAttributes( currentDeclType.attr );
		MASSERT( currentModule != NULL );
		currentModule->Add( instance );
		$$ = _LINK($2,_INSTANCE_REF( instance, &loc ));
	}
	| '(' { DoComments(); } list_of_module_connections_o rp
	{
		CInstance*	instance;
		instance = new(unitStack) CInstance(
			CSymbol::GenSymbol( "_instance_" ),
			currentInstanceType,
			currentInstanceParameters, PostpendComments($3), &loc );
		MASSERT( currentModule != NULL );
		currentBlock->Add( instance );
		$$ = PrependComments(_INSTANCE_REF( instance, &loc ));
	}
	;

list_of_module_connections_o
	: module_port_connection_clist
	{ $$ = $1; }
	| named_port_connection_clist pragmas_o
	{ $$ = _LINK($1,$2); }
	;

module_port_connection_clist
	: module_port_connection
	{ $$ = $1; }
	| module_port_connection_clist ',' module_port_connection
	{
	    CNode* l;
	    CNode* r;
	    if( $1 && $1->GetOp() == eLIST ) {
		l = $1;
	    } else if ( $1 && $1->GetOp() != eARG ) {
		CNode* n = _ARG( NULL, NULL, &loc );
		l = _LINK( $1, n );
	    } else if( $1 ) {
		l = $1;
	    } else {
		l = _ARG( NULL, NULL, &loc );
	    }
	    if( $3 && $3->GetOp() == eLIST ) {
		r = $3;
	    } else if ( $3 && $3->GetOp() != eARG ) {
		CNode* n = _ARG( NULL, NULL, &loc );
		r = _LINK( $3, n );
	    } else if( $3 ) {
		r = $3;
	    } else {
		r = _ARG( NULL, NULL, &loc );
	    }
	    $$ = _LINK( l, r );
	}
	;

module_port_connection
	: pragmas_o attribute_instance_list_o attr_clear expression pragmas_o
	{
		CNode* n = _ARG( NULL, Macro2Node($4), &loc );
		$$ = PrependComments(_LINK(n,$5));
		n->SetAttributes( _LINK($1,$3) );
	}
	| pragmas_o attribute_instance_list_o attr_clear
	{
		if( $3 || $1 ) {
		    CNode* n = _ARG( NULL, NULL, &loc );
		    $$ = PrependComments(n);
		    n->SetAttributes(_LINK($1,$3));
		} else {
		    $$ = NULL;
		}
	}
	;

named_port_connection_clist
	: named_port_connection
	{ $$ = $1; }
	| named_port_connection_clist ',' named_port_connection
	{ $$ = _LIST( $1, $3, &loc ); }
	;

named_port_connection
	: pragmas_o attribute_instance_list_o '.' IDENTIFIER '(' attr_clear expression rp
	{
		CNode* n = _ARG( $4, Macro2Node($7), &loc );
		$$ = PrependComments(n);
		n->SetAttributes(_LINK($1,$6));
	}
	| pragmas_o attribute_instance_list_o '.' IDENTIFIER '(' attr_clear rp
	{
		CNode* n = _ARG( $4, NULL, &loc );
		$$ = PrependComments(n);
		n->SetAttributes(_LINK($1,$6));
	}
	;

/*
A.4.2 Generate construct
*/

genvar_declaration
	: cleardecl GENVAR {$<node>$=GetComments();}
	  list_of_genvar_identifiers sc
	{ $$ = _LINK($<node>3,$4); }
	;

list_of_genvar_identifiers
	: genvar_identifier
	{
	    CDecl* genvar = CreateGenvar( FALSE, $1, currentBlock, currentDeclType );
	    MASSERT( !currentDeclType.pragma );
	    $$ = _GENVAR_DECL((CGenvar*)genvar, &loc);
	}
	| list_of_genvar_identifiers ',' genvar_identifier
	{
	    CDecl* genvar = CreateGenvar( FALSE, $3, currentBlock, currentDeclType );
	    MASSERT( !currentDeclType.pragma );
	    $$ = _LINK($1,_GENVAR_DECL((CGenvar*)genvar,&loc));
	}
	;


generate_region
	: GENERATE location comments
	{ moduleSymtab.PushScope( FALSE ); 
	  CBlock*	block;
	  MASSERT( currentBlock != NULL );
	  block = new(unitStack) CBlock( NULL, &$<coord>2, eGENVARBLOCK );
	  currentBlock->Add( block );
	  block->SetParent( currentBlock );
	  currentBlock = block;
	}
	  module_or_generate_item_list_o ENDGENERATE
	{
	  CNode* n = _BLOCK_REF( currentBlock, $5, &$<coord>2 );
	  currentBlock->SetSymtab( moduleSymtab );
	  currentBlock = currentBlock->GetParent();
	  moduleSymtab.PopScope();
	  $$ = _LINK($<node>3,n);
	}
	;

loop_generate_construct
	: FOR attr_clear comments location { moduleSymtab.PushScope( FALSE ); }
	  '(' genvar_initialization ';' 
	  genvar_expression ';' genvar_iteration ')'
	{
	    CSymbol* sym = $7->Arg<CNode*>(1)->Arg<CGenvar*>(0)->GetSymbol();
	    CParam* param = new(unitStack) CParam( sym, &loc, eLOCALPARAM );
	    param->SetExpression( Val( FALSE, 1, sym, NULL ) );
	    param->SetSubType( eINT );
	    moduleSymtab.Add( sym, param );
	    param->SetDeclStatementCreated();
	}
          generate_block
	{
	    moduleSymtab.PopScope();
	    CNode* n =  _GFOR( $7, $9, $11, $14, &$4 );
	    n->SetAttributes( $2 );
	    $$ = _LINK( $3, n );
	}
	;

genvar_initialization 
	: genvar_identifier location '=' constant_expression
	{   CNode* n = Val( FALSE, 0, $1, NULL ); 
	    $$ = _ASSIGN( NULL, n, Macro2Node($4), &$2 ); 
	}
	;

/* Need to add attribute instance to express */
genvar_expression
	: genvar_primary
	| '+' genvar_primary %prec UNARY
	{
		$$ = $2;
	}
	| '-' genvar_primary %prec UNARY
	{
		$$ = _NEG( $2, &loc );
	}
	| '!' genvar_primary %prec UNARY
	{
		$$ = _NOT( $2, &loc );
	}
	| '~' genvar_primary %prec UNARY
	{
		$$ = _COM( $2, &loc );
	}
	| '&' genvar_primary %prec UNARY
	{
		$$ = _RAND( $2, &loc );
	}
	| REDUCTION_NAND genvar_primary %prec UNARY
	{
		$$ = _RNAND( $2, &loc );
	}
	| '|' genvar_primary %prec UNARY
	{
		$$ = _ROR( $2, &loc );
	}
	| REDUCTION_NOR genvar_primary %prec UNARY
	{
		$$ = _RNOR( $2, &loc );
	}
	| '^' genvar_primary %prec UNARY
	{
		$$ = _RXOR( $2, &loc );
	}
	| XNOR genvar_primary %prec UNARY
	{
		$$ = _RXNOR( $2, &loc );
	}
	| genvar_expression '+' genvar_expression
	{
		$$ = _ADD( $1, $3, &loc );
	}
	| genvar_expression '-' genvar_expression
	{
		$$ = _SUB( $1, $3, &loc );
	}
	| genvar_expression '*' genvar_expression
	{
		$$ = _MUL( $1, $3, &loc );
	}
	| genvar_expression '/' genvar_expression
	{
		$$ = _DIV(  $1, $3, &loc );
	}
	| genvar_expression STARSTAR genvar_expression
	{
		$$ = _POW(  $1, $3, &loc );
	}
	| genvar_expression '%' genvar_expression
	{
		$$ = _MOD( $1, $3, &loc );
	}
	| genvar_expression LOGICAL_EQUALITY genvar_expression
	{
		$$ = _EQ( $1, $3, &loc );
	}
	| genvar_expression LOGICAL_INEQUALITY genvar_expression
	{
		$$ = _NE( $1, $3, &loc );
	}
	| genvar_expression CASE_EQUALITY genvar_expression
	{
		$$ = _CEQ( $1, $3, &loc );
	}
	| genvar_expression CASE_INEQUALITY genvar_expression
	{
		$$ = _CNE( $1, $3, &loc );
	}
	| genvar_expression ANDAND genvar_expression
	{
		$$ = _LAND( $1, $3, &loc );
	}
	| genvar_expression OROR genvar_expression
	{
		$$ = _LOR( $1, $3, &loc );
	}
	| genvar_expression LT genvar_expression
	{
		$$ = _LT( $1, $3, &loc );
	}
	| genvar_expression LE genvar_expression
	{
		$$ = _LE( $1, $3, &loc );
	}
	| genvar_expression GT genvar_expression
	{
		$$ = _GT( $1, $3, &loc );
	}
	| genvar_expression GE genvar_expression
	{
		$$ = _GE( $1, $3, &loc );
	}
	| genvar_expression '&' genvar_expression
	{
		$$ = _AND( $1, $3, &loc );
	}
	| genvar_expression '|' genvar_expression
	{
		$$ = _OR( $1, $3, &loc );
	}
	| genvar_expression '^' genvar_expression
	{
		$$ = _XOR( $1, $3, &loc );
	}
	| genvar_expression XNOR genvar_expression
	{
		$$ = _XNOR( $1, $3, &loc );
	}
	| genvar_expression LEFT_SHIFT genvar_expression
	{
		$$ = _LSH( $1, $3, &loc );
	}
	| genvar_expression RIGHT_SHIFT genvar_expression
	{
		$$ = _RSH( $1, $3, &loc );
	}
	| genvar_expression LLL genvar_expression
	{
		$$ = _LSHA( $1, $3, &loc );
	}
	| genvar_expression RRR genvar_expression
	{
		$$ = _RSHA( $1, $3, &loc );
	}
	| genvar_expression '?' genvar_expression ':' genvar_expression
	{
		$$ = _HOOK( $1, $3, $5, &loc );
	}
	;

genvar_iteration
	: genvar_identifier {$<coord>$=loc;} '=' genvar_expression
	{ 
// ??? mdh - TRUE nor FALSE is the right first arg here
	    CNode* lv = Val( TRUE, 0, $1, NULL );
	    $$ = _ASSIGN(  NULL, lv, $4, &$<coord>2 ); 
	}
	;

genvar_primary
	: number
	{ $$ = Macro2Node($1); }
	| parameter_or_specparam_or_genvar_identifier %prec LOWEST
	{ $$ = Val( FALSE, 1, $1, NULL ); }
	// ??? mdh - this requires a semantic check as a genver is not allowed here
	| parameter_or_specparam_or_genvar_identifier constant_dimension_range
	{  $$ = Val( FALSE, 1, $1, $2 ); }
	| constant_concatenation
	{ $$ = Macro2Node($1); }
	| constant_function_call
	/*
	 * note an expression list is only sometimes valid
	 * here, a semantic check needs to be added
	 */
	| constant_system_function_call
	| '(' constant_mintypmax_expression_clist rp
	{
		$$ = Macro2Node($2);
	}
	| '(' error rp
	{
		$$ = &errorNode;
	}
	| string
	{ $$ = Macro2Node($1); }
	;

conditional_generate_construct
	: if_generate_construct
	| case_generate_construct
	;

if_generate_construct
	: IF attr_clear '(' constant_expression ')'  
	  comments generate_block_or_null 
	  if_generate_construct_elsepart
	{ 
	   CNode* n = _GIF( Macro2Node($4), $7, $8, &loc ); 
	   n->SetAttributes( $2 );
	   $$ = _LINK($6,n); 
	}
	;

if_generate_construct_elsepart
	: /* empty */		%prec IF
	{ $$ = NULL; }
	| ELSE generate_block_or_null 
	{ $$ = $2; }
	;

case_generate_construct
	: CASE attr_clear comments location
	  '(' constant_expression ')' case_generate_item_list ENDCASE 
	{
	   CNode* n = _GCASE( Macro2Node($6), $8, &$4 );
	   n->SetAttributes( $2 );
	   $$ = _LINK( $3, n );
	}
	;

case_generate_item 
	: constant_expression_clist ':' comments
	  generate_block_or_null
	{ $$ = _LINK( $3, _CASEITEM( $1, $4, &loc ) ); }
	| DEFAULT ':' comments generate_block_or_null
	{ $$ = _LINK( $3, _CASEITEM( NULL, $4, &loc ) ); }
	| DEFAULT comments generate_block_or_null
	{ $$ = _LINK( $2, _CASEITEM( NULL, $3, &loc ) ); }
	;

case_generate_item_list 
	: case_generate_item
	{ $$ = $1; }
	| case_generate_item_list case_generate_item
	{ $$ = _LINK( $1, $2 ); }
	;

constant_expression_clist
	: constant_expression
	{ $$ = Macro2Node($1); }
	| constant_expression_clist ',' constant_expression
	{ $$ = _ELINK( $1, Macro2Node($3)); }
	;

generate_block
	: module_or_generate_item
	 { $$ = $1; }
	| BEGIN_ module_or_generate_item_list_o END
	 { $$ = $2; }
	| BEGIN_ ':' {$<coord>$=loc;}
          generate_block_identifier {$<node>$=GetComments();}
	{ 
	  CBlock*	block;
	  MASSERT( currentBlock != NULL );
	  block = new(unitStack) CBlock( $4, &$<coord>3 );
	  currentBlock->Add( block );
	  block->SetParent( currentBlock );
	  currentBlock = block;
	  moduleSymtab.PushScope( FALSE );
	}
	  module_or_generate_item_list_o END
	{
	  CNode* n = _BLOCK_REF( currentBlock, PrependComments($7), &$<coord>3 );
	  currentBlock->SetSymtab( moduleSymtab );
	  currentBlock = currentBlock->GetParent();
	  moduleSymtab.PopScope();
	  $$ = _LINK($<node>5,n);
	}
	;

generate_block_or_null 
	: {moduleSymtab.PushScope(FALSE);} generate_block
	{ 
	  moduleSymtab.PopScope(); 
	  $$ = $2;
        }
	| sc
	{ $$ = NULL; }
	; 

/*
A.5 Behavioral Statements
*/

initial_statement
	: INITIAL_ attr_clear location comments statement
	{
		CNode* n = _INIT( $5, &$3 );
		n->SetAttributes($2);
		$$ = _LINK($4,n);
	}
	;

always_statement
	: ALWAYS attr_clear location comments statement
	{
		CNode* n = _ALWAYS( $5, &$3 );
		n->SetAttributes($2);
		$$ = _LINK($4,n);
	}
	;


statement_or_null
	: pragmas attribute_instance_list_o statement1
	{ $$ = _LINK($1,$3); }
	| attribute_instance_list_o statement1
	{ $$ = $2; }
	| attribute_instance_list_o attr_clear sc
	{ $$ = $2; }
	;

statement_or_null_noattr
	:  statement1
	| sc
	{ $$ = NULL; }
	;

statement
	: pragmas attribute_instance_list_o statement1
	{ $$ = _LINK($1,$3); }
	| attribute_instance_list_o statement1
	{ $$ = $2; }
	;

statement_noattr
	: pragmas statement1
	{ $$ = _LINK($1,$2); }
	| statement1
	;

statement_list_noattr
	: /* empty */
	{ $$ = PrependComments(NULL); }
	| statement_noattr statement_list
	{ $$ = _LINK( $1, $2 ); }
	;

statement_list
	: statement_list_macro
	{ $$ = Macro2Node($1); }
	;

statement_list_macro
	: comments 
	{ $$ = AnnotateMacroToken( loc, $1); }
	| statement_list_macro attribute_instance_list_o statement1
	{	
	    MacroToken* mt = AnnotateMacroToken( loc, $3 );
	    $$ = _LINK( $1, mt);
	    yyerrok;
	}
	| statement_list_macro pragma_macro
	{	
	    $$ = _LINK( $1, $2 );
	    yyerrok;
	}
	| statement_list_macro error
	{ $$ = $1; }
	;

statement1
	: assignment comments sc
	{
		$$ = _LINK( $2, $1 );
	}
	| IF attr_clear comments location 
	  '(' expression rp statement_or_null  elsepart
	{
		CNode* n = _IF( Macro2Node($6), _LINK($8,$9.head), $9.tail, &$4 );
		n->SetAttributes( $2 );
		$$ = _LINK( $3, n );
	}
	| CASE attr_clear comments location '(' expression rp
			case_item_plist pragmas_o ENDCASE
	{
	  	CNode* n = _CASE( Macro2Node($6), _LINK($8,$9), &$4 );
	        n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}
	| CASEX attr_clear comments location '(' expression rp
			case_item_plist pragmas_o ENDCASE
	{
	  	CNode* n = _CASEX( Macro2Node($6), _LINK($8,$9), &$4 );
	        n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}
	| CASEZ attr_clear comments location '(' expression rp
			case_item_plist pragmas_o ENDCASE
	{
	  	CNode* n = _CASEZ( Macro2Node($6), _LINK($8,$9), &$4 );
	        n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}
	| FOREVER attr_clear comments location statement
	{
		CNode* n = _FOREVER( $5, &$4 );
		n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}
	| REPEAT attr_clear comments location '(' expression rp statement
	{
		CNode* n = _REPEAT( Macro2Node($6), $8, &$4 );
		n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}
	| WHILE attr_clear comments location '(' expression rp statement
	{
		CNode* n = _WHILE( Macro2Node($6), $8, &$4 );
		n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}
	| FOR attr_clear comments location '(' variable_assignment sc  expression sc variable_assignment rp statement
	{
		CNode* n = _FOR(  $6, Macro2Node($8), $10, $12, &$4 );
		n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}
	| delay_control attr_clear comments location statement_or_null
	{
		CNode* n = _DELAY( $1, $5, &$4 );
		n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}

	| event_control attr_clear comments location statement_or_null
	{ 	
		CNode* n = _EVENT( $1, $5, &$4 );
		n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}

	| lvalue '=' attr_clear comments location
	  delay_or_event_control expression sc
	{
		ValidateLvalList( $1, TRUE, eREG, eREAL, eNONE );
		CNode* n = _ASSIGN( $6, $1, Macro2Node($7), &$5 );
	        n->SetAttributes($3);
		$$ = _LINK( $4, n );
	}

	| lvalue LE attr_clear comments expression sc
	{
		ValidateLvalList( $1, TRUE, eREG, eREAL, eNONE );
		CNode* n = _NBASSIGN( NULL, $1, Macro2Node($5), &loc );
	        n->SetAttributes($3);
		$$ = _LINK( $4, n );
	}

	| lvalue  LE attr_clear comments location
	  delay_or_event_control expression sc
	{
		ValidateLvalList( $1, TRUE, eREG, eREAL, eNONE );
		CNode* n = _NBASSIGN( $6, $1, Macro2Node($7), &$5 );
	        n->SetAttributes($3);
		$$ = _LINK( $4, n );
	}

	| WAIT attr_clear comments location
	  '(' conditional_expression rp statement_or_null
	{
		CNode* n =_WAIT( $6, $8, &$4 );
		n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}

	| ARROW_ attr_clear comments location 
	  hierarchical_event_identifier sc
	{
	  	if( !$5->ValidId( 1, 0 ) ) {
	       	    error( &loc, "illegal identifier reference" );
	  	}
		CNode* n = Val( FALSE, 1, $5->Symbol(), $5->Dim() ) ;
		n->SetAttributes($2);
	   	$$ = _LINK( _TRIGGER( n, &$4 ), $3 );
	  	delete $3; 
	}
	| ARROW_ attr_clear comments 
	  location hierarchical_event_identifier dimension_range sc
	{
		$5->Set( $6 );
	  	if( !$5->ValidId( 1, 0 ) ) {
	       	    error( &loc, "illegal identifier reference" );
	  	}
		CNode* n = Val( FALSE, 1, $5->Symbol(), $5->Dim() ) ;
		n->SetAttributes($2);
	   	$$ = _LINK( _TRIGGER( n, &$4 ), $3 );
	  	delete $5; 
	}
	| seq_block
	| par_block
	| task_enable
	| system_task_enable
	| DISABLE attr_clear
	  hierarchical_task_identifier comments sc
	{
		CNode* n = _DISABLE( $3, &loc );
		n->SetAttributes($2);
		$$ = _LINK( $4, n );
	}
	| ASSIGN attr_clear comments location lvalue '=' expression sc
	{
		CNode* n = _PASSIGN( $5, Macro2Node($7), &$4 );
	        n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}

	| DEASSIGN attr_clear lvalue comments sc
	{
		CNode* n = _DEASSIGN( $3, &loc );
	        n->SetAttributes($2);
		$$ = _LINK( $4, n );
	}
	| FORCE	attr_clear lvalue '=' comments expression sc
	{
		CNode* n = _FORCE( $3, Macro2Node($6), &loc );
	        n->SetAttributes($2);
		$$ = _LINK( $5, n );
	}
	| RELEASE attr_clear lvalue comments sc
	{
		CNode* n = _RELEASE( $3, &loc );
	        n->SetAttributes($2);
		$$ = _LINK( $4, n );
	}
	;

elsepart
	: /* empty */		%prec IF
	{ $$.head = NULL; $$.tail = NULL; }
	| ELSE comments statement_or_null
	{ $$.head = $2; $$.tail = $3; }
	;

variable_assignment
	: lvalue '=' expression
	{
		ValidateLvalList( $1, TRUE, eREG, eREAL, eNONE );
		CNode* n = _ASSIGN( NULL, $1, Macro2Node($3), &loc );
		$$ = n;
	}
	;

assignment
	: lvalue '=' attr_clear location expression
	{
		ValidateLvalList( $1, TRUE, eREG, eREAL, eNONE );
		CNode* n = _ASSIGN( NULL, $1, Macro2Node($5), &$4 );
		n->SetAttributes($3);
		$$ = n;
	}
	;

case_item_plist
	: case_expr
	{ $$ = $1; }
	| case_item_plist case_expr
	{	
		$$ = _LINK( $1, $2 );
		yyerrok;
	}
	| error
	{ $$ = &errorNode; }
	| case_item_plist error
	;

case_expr
	: pragmas_o expression_clist ':' comments 
					statement_or_null
	{ 
	  CNode* preamble = _LINK( $1, $4 );
	  $$ = _LINK( preamble, _CASEITEM( $2, $5, &loc ) ); }
	| pragmas_o DEFAULT ':'  comments statement_or_null
	{ 
	  CNode* preamble = _LINK( $1, $4 );
	  $$ = _LINK( preamble, _CASEITEM( NULL, $5, &loc ) ); }
	| pragmas_o DEFAULT comments statement_or_null
	{ 
	  CNode* preamble = _LINK( $1, $3 );
	  $$ = _LINK( preamble, _CASEITEM( NULL, $4, &loc ) ); }
	;

expression_clist
	: expression
	{ $$ = Macro2Node($1); }
	| expression_clist ',' expression
	{
		$$ = _ELIST( $1, Macro2Node($3), &loc );
	}
	;

expression_catlist
	: expression
	| expression_catlist ',' expression
	{
		$$ = BOP(_CAT,$1,$2,$3);
	}
	| error
	{ $$ = AnnotateMacroToken(loc,&errorNode); }
	| expression_catlist error
	{ $$ = $1; }
	| expression_catlist error expression
	{ $$ = AnnotateMacroToken(loc, _CAT( Macro2Node($1), Macro2Node($3))); }
	| expression_catlist ',' error
	{ $$ = $1; }
	;

constant_expression_catlist
	: constant_expression
	| constant_expression_catlist ',' constant_expression
	{
		$$ = BOP(_CAT, $1, $2, $3 );
	}
	| constant_expression_catlist error
	{ $$ = $1; }
	| constant_expression_catlist error constant_expression
	{ $$ = AnnotateMacroToken(loc, _CAT( Macro2Node($1), Macro2Node($3))); }
	| constant_expression_catlist ',' error
	{ $$ = $1; }
	;

param_expression_clist
	: param_expression
	| constant_expression ':' constant_expression ':' constant_expression
	{
		$$ = Macro2Node(TOP(_MTM, $1, $2, $3, $4, $5 ));
	}
	| constant_expression ':' constant_expression ':' 
			constant_expression ',' param_expression_clist2
	{
		$$ = _ELIST(Macro2Node(TOP( _MTM, $1, $2, $3, $4, $5 )), $7 );
	}
	| param_expression ',' param_expression_clist1
	{
		$$ = _ELIST( $1, $3 );
	}
	;

param_expression_clist2
	: constant_expression ':' constant_expression ':' constant_expression
	{
		$$ = Macro2Node(TOP(_MTM, $1, $2, $3, $4, $5 ));
	}
	| constant_expression ':' constant_expression ':' 
			constant_expression ',' param_expression_clist2
	{
		$$ = _ELIST(Macro2Node(TOP( _MTM, $1, $2, $3, $4, $5 )), $7 );
	}
	;
param_expression_clist1
	: param_expression
	| param_expression ',' param_expression_clist1
	{
		$$ = _ELIST( $1, $3 );
	}
	;
param_expression
	: constant_expression
	{ $$ = Macro2Node($1); }
	| pragmas_o '.' IDENTIFIER '(' constant_expression rp
	{
		CNode* n = _ARG( $3, Macro2Node($5), &loc );
		$$ = PrependComments(_LINK($1,n));
	}
	;
xexpression_clist
	: xexpression
	| xexpression_clist ',' xexpression
	{
		$$ = _ELIST( $1, $3 );
	}
	;
constant_xexpression_clist
	: constant_xexpression
	| constant_xexpression_clist ',' constant_xexpression
	{
		$$ = _ELIST( $1, $3 );
	}
	;

xexpression
	: /* empty */
	{ $$ = NULL; }
	| expression
	{ $$ = Macro2Node($1); }
	;

constant_xexpression
	: /* empty */
	{ $$ = NULL; }
	| constant_expression
	{ $$ = Macro2Node($1); }
	;


seq_block
	: BEGIN_ comments seq_block_1
	{ $$ = _LINK($2,$3); }
	;	

seq_block_1
	:  attr_clear location statement_list END
	{ 
		if( $1 || ($3 && $3->GetOp() == eLIST) ) {
		    $$ = _BLOCK_REF( NULL, PostpendComments($3), &$2 );
		    $$->SetAttributes($1);
		} else {
		    $$ = PostpendComments($3);
		}
	}
	| attr_clear ':' location IDENTIFIER
	{
		CBlock*	block;
		MASSERT( currentBlock != NULL );
		block = new(unitStack) CBlock( $4, &$3 );
		currentBlock->Add( block );
		block->SetParent( currentBlock );
		currentBlock = block;
	        moduleSymtab.PushScope( FALSE );
		
	}
 	  block_item_declaration_list statement_list_noattr END
	{
		currentBlock->Add($6);
		CNode* n = _BLOCK_REF( currentBlock, 
					PostpendComments($7), &$3 );
		n->SetAttributes($1);
	        currentBlock->SetSymtab( moduleSymtab );
		currentBlock = currentBlock->GetParent();
		moduleSymtab.PopScope();
		$$ = n;
	}
	;

par_block
	: FORK attr_clear comments location 
	{
		CBlock*	block;
		MASSERT( currentBlock != NULL );
		block = new(unitStack) CBlock( NULL, &$4, ePARALLELBLOCK );
		currentBlock->Add( block );
		block->SetParent( currentBlock );
		currentBlock = block;
		
	}
 	  statement_list JOIN
	{
		CNode* n = _BLOCK_REF( currentBlock, 
					PostpendComments($6), &$4 );
		n->SetAttributes( $2 );
	        currentBlock->SetSymtab( moduleSymtab );
		currentBlock = currentBlock->GetParent();
		$$ = _LINK($3,n);
	}
	| FORK attr_clear ':' comments location
	  IDENTIFIER
	{
		CBlock*	block;
		MASSERT( currentBlock != NULL );
		block = new(unitStack) CBlock( $6, &$5, ePARALLELBLOCK );
		currentBlock->Add( block );
		block->SetParent( currentBlock );
		currentBlock = block;
	        moduleSymtab.PushScope( FALSE );
		
	}
 	  block_item_declaration_list statement_list_noattr JOIN
	{
		currentBlock->Add($8);
		CNode* n = _BLOCK_REF( currentBlock, 
					PostpendComments($9), &$5 );
		n->SetAttributes( $2 );
	        currentBlock->SetSymtab( moduleSymtab );
		currentBlock = currentBlock->GetParent();
		moduleSymtab.PopScope();
		$$ = _LINK($4,n);
	}
	;

block_item_declaration_list
	: /* empty */
	{ $$ = NULL; }
	| block_item_declaration_list block_item_declaration
	{ $$ = _LINK($1,$2); }
	;

task_enable
	: hierarchical_task_identifier comments sc
	{
		CNode* n = _TASK_ENABLE( $1, NULL, &loc );
		$$ = _LIST( $2, n );
	}
	| hierarchical_task_identifier comments 
	  '(' expression_clist rp sc
	{
		CNode* n = _TASK_ENABLE( $1, $4, &loc );
		$$ = _LIST( $2, n );
	}
	;

system_task_enable
	: SYSTEM_IDENTIFIER attr_clear comments sc
	{
		CNode* n = _SYSTASK_CALL( $1, NULL, &loc );
		n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}
	| SYSTEM_IDENTIFIER attr_clear comments 
	  '(' xexpression_clist rp sc
	{
		CNode* n = _SYSTASK_CALL( $1, $5, &loc );
		n->SetAttributes($2);
		$$ = _LINK( $3, n );
	}
	;

lvalue
	: lval_normal
	| lval_concat
	;

lval_normal
	: hierarchical_identifier
	{ 
	  if( !$1->ValidId( 1, 1 ) ) {
	       error( &loc, "illegal identifier reference" );
	  }
	  $$ = Val( TRUE, 0, $1->Symbol(), $1->Dim() ); 
	  delete $1; 
	}
	| hierarchical_identifier dimension_range
	{  
	  $1->Set( $2 );
	  if( !$1->ValidId( 1, 1 ) ) {
	       error( &loc, "illegal identifier reference" );
	  }
	  $$ = Val( TRUE, 0, $1->Symbol(), $1->Dim() ); 
	  delete $1; 
	}
	;

delay_or_event_control
	: delay_control
	{ $$ = _DELAY_CONTROL($1); }
	| event_control
	{ $$ = _EVENT_CONTROL($1); }
	| REPEAT location '(' expression ')' event_control
	{ $$ = _REPEAT_CONTROL( Macro2Node($4), _EVENT_CONTROL($6), &$2 ); }
	;
/*
A.6 Specify Section
*/

specify_block
	: SPECIFY
	{
		DoComments();
		CSpecify* specifyBlock = new(unitStack) CSpecify( &loc );
		specifyBlock->SetParent( currentBlock );
		inSpecifyBlock = 1;
		$<spblock>$ = specifyBlock;
	}	
	specify_items comments ENDSPECIFY
	{
		inSpecifyBlock = 0;
		$$ = _SPECIFY_REF( $<spblock>2, _LINK($3,$4), &loc );
	}
	;

specify_items
	: /* empty */
	{ $$ = NULL; }
	| specify_items specify_item
	{ $$ = _LINK( $1, $2 ); }
	| specify_items error
	{ $$ = $1; }
	;
specify_item	
	: comments specparam_declaration
	{ $$ = _LINK($1,$2); }
	| comments path_declaration
	{ $$ = _LINK($1,$2); }
	| comments system_timing_check
	{ $$ = _LINK($1,$2); }
	;
specparam_declaration
	: SPECPARAM cleardecl
	{ currentDeclType.type = ePARAM; }
	list_of_param_assignments sc
	{ $$ = $4; }
	;

path_declaration
	: path_description '=' path_delay_value sc
	{
	    $$ = _PATH_ASSIGN( NULL, $1, $3, &loc );
	}	
	| IF '(' conditional_port_expression rp path_description '=' path_delay_value sc
	{
	    $$ = _PATH_ASSIGN( $3, $5, $7, &loc );
	}	
	| IFNONE path_description '=' path_delay_value sc
	{
	    $$ = _IFNONE_PATH_ASSIGN( $2, $4, &loc );
	}	
	;
path_description
	/* all types of path descriptors are bundled together - its was the
	   only way I could get bison to parse correctly */
	: '('
	  list_of_path_inputs path_operator '('
	  list_of_path_outputs ':'
	  data_source_expression rp rp
	{
	    $$ = _PATH( 0, $2, 0, $3, $5, 0, $7, &loc );
	}
	| '(' edge_identifier
	  list_of_path_inputs path_operator '('
	  list_of_path_outputs ':'
	  data_source_expression rp rp
	{
	    $$ = _PATH( $2, $3, 0, $4, $6, 0, $8, &loc );
	}
	| '('
	  list_of_path_inputs path_operator '('
	  list_of_path_outputs polarity_operator1
	  data_source_expression rp rp
	{
	    $$ = _PATH( 0, $2, 0, $3, $5, $6, $7, &loc );
	}
	| '(' edge_identifier
	  list_of_path_inputs path_operator '('
	  list_of_path_outputs polarity_operator1
	  data_source_expression rp rp
	{
	    $$ = _PATH( $2, $3, 0, $4, $6, $7, $8, &loc );
	}
	| '(' list_of_path_inputs  path_operator
					 list_of_path_outputs rp
	{
	    $$ = _PATH( 0, $2, 0, $3, $4, 0, NULL, &loc );
	}
	| '(' list_of_path_inputs  polarity_operator path_operator
					 list_of_path_outputs rp
	{
	    $$ = _PATH( 0, $2, $3, $4, $5, 0, NULL, &loc );
	}
	;

list_of_path_inputs
	: specify_input_terminal_descriptor
	{ $$ = $1; }
	| list_of_path_inputs ','specify_input_terminal_descriptor
	{ yyerrok;
	  $$ = _LIST( $1, $3 );
	}
	| error
	{ $$ = &errorNode; }
	;
list_of_path_outputs
	: specify_output_terminal_descriptor
	{ $$ = $1; }
	| list_of_path_outputs ','specify_output_terminal_descriptor
	{ yyerrok;
	  $$ = _LIST( $1, $3 );
	}
	| error
	{ $$ = &errorNode; }
	;
specify_input_terminal_descriptor
	: IDENTIFIER
	{ $$ = Val( FALSE, 1, $1, NULL ); }
	| IDENTIFIER dimension_range
	{  $$ = Val( FALSE, 1, $1, $2 ); }
	;
specify_output_terminal_descriptor
	: IDENTIFIER
	{ $$ = Val( FALSE, 1, $1, NULL ); }
	| IDENTIFIER dimension_range
	{  $$ = Val( FALSE, 1, $1, $2 ); }
	;
path_delay_value
	/* note this is more permisive that it should be
	 * it requires later semantic checking
	 */
	: mintypmax_expression_clist
	{ $$ = Macro2Node($1); }
	;
polarity_operator
	:	'+'
	{ $$ = 1; }
	|	'-'
	{ $$ = -1; }
	;
polarity_operator1
	:	PCOLON	
	{ $$ = 1; }
	|	MCOLON	
	{ $$ = -1; }
	|	'+' ':'	
	{ $$ = 1; }
	|	'-' ':'	
	{ $$ = -1; }
	;
path_operator
	:	STARARROW
	{ $$ = 0; }
	|	EQUALARROW
	{ $$ = 1; }
	;
conditional_port_expression
	: expression	/* according to the BNF this should be more elaborate */
		/* if further checking for conformance is required it 	
		   could be done in a later pass */
	{ $$ = Macro2Node($1); }
	;
data_source_expression
	: expression
	{ $$ = Macro2Node($1); }
	;
edge_identifier
	: POSEDGE
	{ $$ = 1; }
	| NEGEDGE
	{ $$ = -1; }
	;

system_timing_check
	:	SETUP '(' timing_check_event ',' timing_check_event ','
			expression notify_clause rp sc
	{
		CNode* n1 = _ELIST( _ELIST( $3, $5 ), _ELIST( Macro2Node($7), $8 ) );
		$$ = _TIMING_CALL( $1, n1, &loc );
	}
	|	HOLD '(' timing_check_event ',' timing_check_event ','
			expression  notify_clause rp sc
	{
		CNode* n1 = _ELIST( _ELIST( $3, $5 ), _ELIST( Macro2Node($7), $8 ) );
		$$ = _TIMING_CALL( $1, n1, &loc );
	}
	|	PERIOD '(' controlled_timing_check_event ','
			expression notify_clause rp sc
	{
		CNode* n1 = _ELIST( _ELIST( $3, Macro2Node($5) ), $6 );
		$$ = _TIMING_CALL( $1, n1, &loc );
	}
	|	WIDTH '(' controlled_timing_check_event ','
			expression  rp sc
	{
		CNode* n1 = _ELIST( $3, Macro2Node($5) );
		$$ = _TIMING_CALL( $1, n1, &loc );
	}
	|	WIDTH '(' controlled_timing_check_event ','
			expression ',' expression notify_clause  rp sc
	{
		CNode* n1 = _ELIST( _ELIST( $3, Macro2Node($5) ), _ELIST( Macro2Node($7), $8 ) );
		$$ = _TIMING_CALL( $1, n1, &loc );
	}
	|	SKEW '(' timing_check_event ',' timing_check_event ','
			expression notify_clause rp sc
	{
		CNode* n1 = _ELIST( _ELIST( $3, $5 ), _ELIST( Macro2Node($7), $8 ) );
		$$ = _TIMING_CALL( $1, n1, &loc );
	}
	|	RECOVERY '(' controlled_timing_check_event ','  timing_check_event ','
			expression notify_clause rp sc
	{
		CNode* n1 = _ELIST( _ELIST( $3, $5 ), _ELIST( Macro2Node($7), $8 ) );
		$$ = _TIMING_CALL( $1, n1, &loc );
	}
	|	SETUPHOLD '(' timing_check_event ',' timing_check_event ','
			expression ',' expression notify_clause rp sc
	{
		CNode* n1 = _ELIST( _ELIST( _ELIST( $3, $5 ), _ELIST( Macro2Node($7), Macro2Node($9) ) ), $10 );
		$$ = _TIMING_CALL( $1, n1, &loc );
	}
	|	NOCHANGE '(' controlled_timing_check_event ','  timing_check_event ','
			expression ',' expression rp sc
	{
		CNode* n1 = _ELIST( _ELIST( $3, $5 ), _ELIST( Macro2Node($7), Macro2Node($9) ) );
		$$ = _TIMING_CALL( $1, n1, &loc );
	}
	;		
notify_clause
	:	/* EMPTY */
	{ $$ = NULL; }
	|	',' notify_register
	{ $$ = $2; }
	;
notify_register
	:	IDENTIFIER
	{ $$ = Val( TRUE, 1, $1, NULL ); }
	;
timing_check_event
	:	timing_check_event_control timing_input_terminal_descriptor	
	{
		if( $1 ) {
		    $1->Arg<CNode*>(0) = $2;
		    $$ = $1;
		} else {
		    $$ = $2;
		}
	}
	|	timing_check_event_control timing_input_terminal_descriptor	
		ANDANDAND timing_check_condition	
	{
	   if( $1 ) {
	      $1->Arg<CNode*>(0) = $2;
	      $$ = $1;
	   } else {
	      $$ = $2;
	   }
	   $$ = _ANDANDAND( $$, $4 );
	}
	;
controlled_timing_check_event
	:	timing_check_event_control_no_empty timing_input_terminal_descriptor	
	{
	        $1->Arg<CNode*>(0) = $2;
		$$ = $1;
	}
	|	timing_check_event_control_no_empty timing_input_terminal_descriptor	
		ANDANDAND timing_check_condition	
	{
	   $1->Arg<CNode*>(0) = $2;
	   $$ = $1;
	   $$ = _ANDANDAND( $$, $4 );
	}
	;
timing_check_event_control
	: /* EMPTY */
	{ $$ = NULL; }
	| POSEDGE
	{ $$ = _POSEDGE( NULL, &loc ); }
	| NEGEDGE
	{ $$ = _NEGEDGE( NULL, &loc ); }
	| EDGE '[' edge_list rb
	{ $$ = _EDGE( NULL, $3, &loc ); }
	;
timing_check_event_control_no_empty
	: POSEDGE
	{ $$ = _POSEDGE( NULL, &loc ); }
	| NEGEDGE
	{ $$ = _NEGEDGE( NULL, &loc ); }
	| EDGE '[' edge_list rb
	{ $$ = _EDGE( NULL, $3, &loc ); }
	;
edge_list
	:	DOUBLE_DIGIT
	{ $$ = $1; }
	|	DOUBLE_DIGIT ',' edge_list
	{ $$ = (Edge_t) ($1 | $3); }
	|	error
	{ $$ = (Edge_t)0; }
	;
timing_check_condition
	:	scalar_expression
	|	'(' timing_check_condition ')'
	{ $$ = $2; }
	|	'~' scalar_expression %prec UNARY
	{ $$ = _COM($2,&loc); }
	|	scalar_expression LOGICAL_EQUALITY scalar_constant
	{ $$ = _EQ( $1, $3 ); }
	|	scalar_expression CASE_EQUALITY scalar_constant
	{ $$ = _CEQ( $1, $3 ); }
	|	scalar_expression CASE_INEQUALITY scalar_constant
	{ $$ = _CNE( $1, $3 ); }
	|	scalar_expression LOGICAL_INEQUALITY scalar_constant
	{ $$ = _NE( $1, $3 ); }
	;	
scalar_expression	/* expression must be a Scalar Net */
	:	IDENTIFIER	
	{ $$ = Val( FALSE, 1, $1, NULL ); }
	;
scalar_constant	
	: number
	{ $$ = Macro2Node($1); }
	;

timing_input_terminal_descriptor
	: identifier
	{ $$ = Val( FALSE, 1, $1, NULL ); }
	| identifier dimension_range
	{  $$ = Val( FALSE, 1, $1, $2 ); }
	;	

/*********************************
 * A.6.5 Timing control statements
 *********************************/


delayN : delay_control
	;

delay_control
	: '#' NUMBER
	{
		CVector* v = SizedStringToVector( &loc, NULL, NULL, Macro2Text($2) );
		CNode* n = _VCONSTANT( v, &loc );
		$$ = Macro2Node( AnnotateMacroToken( loc, n, $2 ) );
	}
	| '#' REAL_NUMBER
	{
		CNode* n = _RCONSTANT( Macro2Text($2), &loc );
		$$ = Macro2Node( AnnotateMacroToken( loc, n, $2 ) );
	}
	| '#' hierarchical_identifier
	{
	  if( !$2->ValidId( 1, 1 ) ) {
	       error( &loc, "illegal identifier reference" );
	  }
	  $$ = Val( 0, 1, $2->Symbol(), $2->Dim() ); 
	  delete $2; 
	}
	| '#' '(' mintypmax_expression_clist rp
	{ $$ = Macro2Node($3); }
	;

event_control
	: '@' hierarchical_event_identifier 
	{  
	  if( !$2->ValidId( 0, 0 ) ) {
	       error( &loc, "illegal identifier reference" );
	  }
	  $$ = Val( 0, 1, $2->Symbol(), $2->Dim() ); 
	  delete $2; 
	}
	| '@' '('
	  event_expression rp
	{ $$ = $3; }
	| '@' '*'
	{ $$ = NULL; }
	| '@' '(' '*' rp
	{ $$ = NULL; }
	| '@' ATTRIBUTE_START rp
	{ $$ = NULL; }
	;

event_expression
	: expression
	{ $$ = Macro2Node($1); }
	| POSEDGE expression
	{ $$ = _POSEDGE( Macro2Node($2), &loc ); }
	| NEGEDGE expression
	{ $$ = _NEGEDGE( Macro2Node($2), &loc ); }
	| event_expression OR event_expression
	{ $$ = _EVOR( $1, $3, &loc ); }
	| event_expression ',' event_expression
	{ $$ = _EVOR( $1, $3, &loc ); }
	;

/*
A.7 Expressions
*/

lval_concat
	: '{' lval_clist '}'
	{ $$ = $2; }
	;

lval_clist
	: lval_normal
	{}
	| lval_concat
	| lval_clist ',' lval_normal
	{ $$ = _CAT( $1, $3, &loc ); }
	| lval_clist ',' lval_concat
	{ $$ = _CAT( $1, $3, &loc ); }
	| error
	{ $$ = &errorNode; }
	| lval_clist error
	{ $$ = &errorNode; }
	| lval_clist error lval_normal
	{ $$ = &errorNode; }
	| lval_clist error lval_concat
	{ $$ = &errorNode; }
	| lval_clist ',' error
	{ $$ = &errorNode; }
	;

conditional_expression
	: expression
	{ $$ = Macro2Node($1); }
	;

constant_expression1
	: expression	
	{ 
	    /*
	     * this construct is used in places where
	     * the grammer causes a conflict
	     */
	    if( !Macro2Node($1)->IsConstant() ) {
	        error( &loc, "Constant expression expected" );
	    }
	    $$ = $1;
	}
	;

constant_expression
	: constant_primary	
	| '+' constant_primary %prec UNARY
	{
		$$ = NO_UOP( $1, $2 );
	}
	| '-' constant_primary %prec UNARY
	{
		$$ = UOP( _NEG, $1, $2 );
	}
	| '!' constant_primary %prec UNARY
	{
		$$ = UOP( _NOT, $1, $2 );
	}
	| '~' constant_primary %prec UNARY
	{
		$$ = UOP( _COM, $1, $2 );
	}
	| '&' constant_primary %prec UNARY
	{
		$$ = UOP( _RAND, $1, $2 );
	}
	| REDUCTION_NAND constant_primary %prec UNARY
	{
		$$ = UOP( _RNAND, $1, $2 );
	}
	| '|' constant_primary %prec UNARY
	{
		$$ = UOP( _ROR, $1, $2 );
	}
	| REDUCTION_NOR constant_primary %prec UNARY
	{
		$$ = UOP( _RNOR, $1, $2 );
	}
	| '^' constant_primary %prec UNARY
	{
		$$ = UOP( _RXOR, $1, $2 );
	}
	| XNOR constant_primary %prec UNARY
	{
		$$ = UOP( _RXNOR, $1, $2 );
	}
	| constant_expression '+' constant_expression
	{
		$$ = BOP( _ADD, $1, $2, $3 );
	}
	| constant_expression '-' constant_expression
	{
		$$ = BOP( _SUB, $1, $2, $3 );
	}
	| constant_expression '*' constant_expression
	{
		$$ = BOP( _MUL, $1, $2, $3 );
	}
	| constant_expression '/' constant_expression
	{
		$$ = BOP( _DIV, $1, $2, $3 );
	}
	| constant_expression STARSTAR constant_expression
	{
		$$ = BOP( _POW, $1, $2, $3 );
	}
	| constant_expression '%' constant_expression
	{
		$$ = BOP( _MOD, $1, $2, $3 );
	}
	| constant_expression LOGICAL_EQUALITY constant_expression
	{
		$$ = BOP( _EQ, $1, $2, $3 );
	}
	| constant_expression LOGICAL_INEQUALITY constant_expression
	{
		$$ = BOP( _NE, $1, $2, $3 );
	}
	| constant_expression CASE_EQUALITY constant_expression
	{
		$$ = BOP( _CEQ, $1, $2, $3 );
	}
	| constant_expression CASE_INEQUALITY constant_expression
	{
		$$ = BOP( _CNE, $1, $2, $3 );
	}
	| constant_expression ANDAND constant_expression
	{
		$$ = BOP( _LAND, $1, $2, $3 );
	}
	| constant_expression OROR constant_expression
	{
		$$ = BOP( _LOR, $1, $2, $3 );
	}
	| constant_expression LT constant_expression
	{
		$$ = BOP( _LT, $1, $2, $3 );
	}
	| constant_expression LE constant_expression
	{
		$$ = BOP( _LE, $1, $2, $3 );
	}
	| constant_expression GT constant_expression
	{
		$$ = BOP( _GT, $1, $2, $3 );
	}
	| constant_expression GE constant_expression
	{
		$$ = BOP( _GE, $1, $2, $3 );
	}
	| constant_expression '&' constant_expression
	{
		$$ = BOP( _AND, $1, $2, $3 );
	}
	| constant_expression '|' constant_expression
	{
		$$ = BOP( _OR, $1, $2, $3 );
	}
	| constant_expression '^' constant_expression
	{
		$$ = BOP( _XOR, $1, $2, $3 );
	}
	| constant_expression XNOR constant_expression
	{
		$$ = BOP( _XNOR, $1, $2, $3 );
	}
	| constant_expression LLL constant_expression
	{
		$$ = BOP( _LSHA, $1, $2, $3 );
	}
	| constant_expression RRR constant_expression
	{
		$$ = BOP( _RSHA, $1, $2, $3 );
	}
	| constant_expression LEFT_SHIFT constant_expression
	{
		$$ = BOP( _LSH, $1, $2, $3 );
	}
	| constant_expression RIGHT_SHIFT constant_expression
	{
		$$ = BOP( _RSH, $1, $2, $3 );
	}
	| constant_expression '?' constant_expression ':' constant_expression
	{
		$$ = TOP( _HOOK, $1, $2, $3, $4, $5 );
	}
	;

constant_primary
	: number
	| parameter_or_specparam_identifier %prec LOWEST
	{ $$ = AnnotateMacroToken( loc, Val( TRUE, 1, $1, NULL ) ); }
	| parameter_or_specparam_identifier constant_dimension_range
	{  $$ = AnnotateMacroToken( loc, Val( TRUE, 1, $1, $2 ) ); }
	| constant_concatenation
	| constant_function_call
	{  $$ = AnnotateMacroToken( loc, $1 ); }
	/*
	 * note an expression list is only sometimes valid
	 * here, a semantic check needs to be added
	 */
	| constant_system_function_call
	{  $$ = AnnotateMacroToken( loc, $1 ); }
	| '(' constant_mintypmax_expression_clist rp
	{ 
            CNode* a2 = Macro2Node($2);
            $$ = AnnotateMacroToken( loc, a2, $1, $2, $3 );
	}
	| '(' error rp
	{  $$ = AnnotateMacroToken( loc, &errorNode ); }
	| string
	;

mintypmax_expression
	: expression
	| expression ':' expression ':' expression
	{
		$$ = TOP( _MTM, $1, $2, $3, $4, $5 );
	}
    ;

constant_mintypmax_expression
	: constant_expression
	| constant_expression ':' constant_expression ':' constant_expression
	{
		$$ = TOP( _MTM, $1, $2, $3, $4, $5 );
	}
    ;

expression
	: primary	
	| '+' primary %prec UNARY
	{
		$$ = NO_UOP( $1, $2 );
	}
	| '-' primary %prec UNARY
	{
		$$ = UOP( _NEG, $1, $2 );
	}
	| '!' primary %prec UNARY
	{
		$$ = UOP( _NOT, $1, $2 );
	}
	| '~' primary %prec UNARY
	{
		$$ = UOP( _COM, $1, $2 );
	}
	| '&' primary %prec UNARY
	{
		$$ = UOP( _RAND, $1, $2 );
	}
	| REDUCTION_NAND primary %prec UNARY
	{
		$$ = UOP( _RNAND, $1, $2 );
	}
	| '|' primary %prec UNARY
	{
		$$ = UOP( _ROR, $1, $2 );
	}
	| REDUCTION_NOR primary %prec UNARY
	{
		$$ = UOP( _RNOR, $1, $2 );
	}
	| '^' primary %prec UNARY
	{
		$$ = UOP( _RXOR, $1, $2 );
	}
	| XNOR primary %prec UNARY
	{
		$$ = UOP( _RXNOR, $1, $2 );
	}
	| expression '+' expression
	{
		$$ = BOP( _ADD, $1, $2, $3 );
	}
	| expression '-' expression
	{
		$$ = BOP( _SUB, $1, $2, $3 );
	}
	| expression '*' expression
	{
		$$ = BOP( _MUL, $1, $2, $3 );
	}
	| expression '/' expression
	{
		$$ = BOP( _DIV, $1, $2, $3 );
	}
	| expression STARSTAR expression
	{
		$$ = BOP( _POW, $1, $2, $3 );
	}
	| expression '%' expression
	{
		$$ = BOP( _MOD, $1, $2, $3 );
	}
	| expression LOGICAL_EQUALITY expression
	{
		$$ = BOP( _EQ, $1, $2, $3 );
	}
	| expression LOGICAL_INEQUALITY expression
	{
		$$ = BOP( _NE, $1, $2, $3 );
	}
	| expression CASE_EQUALITY expression
	{
		$$ = BOP( _CEQ, $1, $2, $3 );
	}
	| expression CASE_INEQUALITY expression
	{
		$$ = BOP( _CNE, $1, $2, $3 );
	}
	| expression ANDAND expression
	{
		$$ = BOP( _LAND, $1, $2, $3 );
	}
	| expression OROR expression
	{
		$$ = BOP( _LOR, $1, $2, $3 );
	}
	| expression LT expression
	{
		$$ = BOP( _LT, $1, $2, $3 );
	}
	| expression LE expression
	{
		$$ = BOP( _LE, $1, $2, $3 );
	}
	| expression GT expression
	{
		$$ = BOP( _GT, $1, $2, $3 );
	}
	| expression GE expression
	{
		$$ = BOP( _GE, $1, $2, $3 );
	}
	| expression '&' expression
	{
		$$ = BOP( _AND, $1, $2, $3 );
	}
	| expression '|' expression
	{
		$$ = BOP( _OR, $1, $2, $3 );
	}
	| expression '^' expression
	{
		$$ = BOP( _XOR, $1, $2, $3 );
	}
	| expression XNOR expression
	{
		$$ = BOP( _XNOR, $1, $2, $3 );
	}
	| expression LLL expression
	{
		$$ = BOP( _LSHA, $1, $2, $3 );
	}
	| expression RRR expression
	{
		$$ = BOP( _RSHA, $1, $2, $3 );
	}
	| expression LEFT_SHIFT expression
	{
		$$ = BOP( _LSH, $1, $2, $3 );
	}
	| expression RIGHT_SHIFT expression
	{
		$$ = BOP( _RSH, $1, $2, $3 );
	}
	| expression '?' expression ':' expression
	{
		$$ = TOP( _HOOK, $1, $2, $3, $4, $5 );
	}
	;

/*
UNARY_OPERATOR is one of the following tokens:
	+	   -	   !	   ~
	&	   ~&	  |	   ^|	  ^	   ~^

BINARY_OPERATOR is one of the following tokens:
	+	   -	   *	   /	   %
	==	  !=	  ===	 !==
	&&	  ||
	<	   <=	  >	   >=
	&	   |	   ^	   ^~
	>>	  <<

STRING is text enclosed in "" and contained on one line
*/

primary
	: number
	| hierarchical_identifier %prec LOWEST
	{ 
	  if( !$1->ValidId( 1, 1 ) ) {
	       error( &loc, "illegal identifier reference" );
	  }
	  $$ = AnnotateMacroToken( loc, Val( TRUE, 1, $1->Symbol(), $1->Dim() ) ); 
	  delete $1; 
	}
	| hierarchical_identifier dimension_range
	{  
	  $1->Set( $2 );
	  if( !$1->ValidId( 1, 1 ) ) {
	       error( &loc, "illegal identifier reference" );
	  }
	  $$ = AnnotateMacroToken( loc, Val( TRUE, 1, $1->Symbol(), $1->Dim() ) ); 
	  delete $1; 
	}
	| concatenation
	| function_call
	{  $$ = AnnotateMacroToken( loc, $1 ); }
	/*
	 * note an expression list is only sometimes valid
	 * here, a semantic check needs to be added
	 */
	| system_function_call
	{  $$ = AnnotateMacroToken( loc, $1 ); }
	| '(' mintypmax_expression_clist rp
	{ 
            CNode* a2 = Macro2Node($2);
            $$ = AnnotateMacroToken( loc, a2, $1, $2, $3 );
	}
	| '(' error rp
	{  $$ = AnnotateMacroToken( loc, &errorNode ); }
	| string
	;

string
	: STRING
	{
            int len = strlen( Macro2Text($1) );
            if( !len ) {
                 len = 1;
            }
            CVector* vector = CVector::AllocFromHeap( unitStack, len*8 );
            vector->LoadString( Macro2Text($1) );
	    CNode* n = _VCONSTANT( vector, &loc );
	    $$ = AnnotateMacroToken( loc, n, $1 );
	}
	;

number
	: NUMBER
	{
		CVector* v = SizedStringToVector( &loc, NULL, NULL, Macro2Text($1) );
		CNode* n = _VCONSTANT( v, &loc );
		$$ = AnnotateMacroToken( loc, n, $1 );
	}
	| REAL_NUMBER
	{
		CNode* n = _RCONSTANT( Macro2Text($1), &loc );
		$$ = AnnotateMacroToken( loc, n, $1 );
	}
	| NUMBER BASE BNUMBER
	{
		CVector* v = SizedStringToVector( &loc, Macro2Text($1), Macro2Text($2), Macro2Text($3) );
		CNode* n = _VCONSTANT( v, &loc );
		$$ = AnnotateMacroToken( loc, n, $1, $2, $3 );
	}
	| BASE BNUMBER
	{
		CVector* v = SizedStringToVector( &loc, NULL, Macro2Text($1), Macro2Text($2) );
		CNode* n = _VCONSTANT( v, &loc );
		$$ = AnnotateMacroToken( loc, n, $1, $2 );
	}
	| UNDEF_MACRO
	{
		CNode* n = _MACRO_EXPR( $1, NULL, &loc);
		$$ = AnnotateMacroToken( loc, n );
	}
	;

dimension_range :
	  '[' expression ']' dimension_range
	{ $4->dimensions.push_front(Macro2Node($2)); $$ = $4; }
	| '[' constant_expression1 ':' constant_expression1 ']'
	{ 
	    MacroToken* mt = BOP(_SLICE, $2, $3, $4 );
	    $$ = new DimInfo; 
	    $$->dimensions.push_front(Macro2Node(mt)); 
	}
	| '[' expression PCOLON constant_expression ']'
	{ 
	    MacroToken* mt = BOP(_PSLICE, $2, $3, $4 );
	    $$ = new DimInfo; 
	    $$->dimensions.push_front(Macro2Node(mt)); 
	}
	| '[' expression MCOLON constant_expression ']'
	{ 
	    MacroToken* mt = BOP(_MSLICE, $2, $3, $4 );
	    $$ = new DimInfo; 
	    $$->dimensions.push_front(Macro2Node(mt)); 
	}
	| '[' expression ']'
	{ $$ = new DimInfo; $$->dimensions.push_front(Macro2Node($2)); }
	;

constant_dimension_range :
	  '[' constant_expression ']' constant_dimension_range
	{ $4->dimensions.push_front(Macro2Node($2)); $$ = $4; }
	| '[' constant_expression ':' constant_expression ']'
	{ 
	    MacroToken* mt = BOP(_SLICE, $2, $3, $4 );
	    $$ = new DimInfo; 
	    $$->dimensions.push_front(Macro2Node(mt)); 
	}
	| '[' constant_expression PCOLON constant_expression ']'
	{ 
	    MacroToken* mt = BOP(_PSLICE, $2, $3, $4 );
	    $$ = new DimInfo; 
	    $$->dimensions.push_front(Macro2Node(mt)); 
	}
	| '[' constant_expression MCOLON constant_expression ']'
	{ 
	    MacroToken* mt = BOP(_MSLICE, $2, $3, $4 );
	    $$ = new DimInfo; 
	    $$->dimensions.push_front(Macro2Node(mt)); 
	}
	| '[' constant_expression ']'
	{ $$ = new DimInfo; $$->dimensions.push_front(Macro2Node($2)); }
	;

dimension_decl :
	  '[' constant_expression ':' constant_expression ']' dimension_decl
	{ 
	    MacroToken* mt = BOP(_RANGE, $2, $3, $4 );
	    dimIndexes.push_front(Macro2Node(mt));
	}
	| '[' constant_expression ':' constant_expression ']'
	{ 
	    MacroToken* mt = BOP(_RANGE, $2, $3, $4 );
	    dimIndexes.push_front(Macro2Node(mt));
	}
	;
    


/*
NUMBER
	Numbers can be specified in decimal, hexadecimal, octal or
	binary, may optionally start with a + or -, and can be given
	in one of two forms.  One form is a normal decimal number
	specified as a sequence of the digits 0 to 9. The other takes
	the form

		ss..s 'f nn..n

	where the characters ss..s are decimal digits and specify the
	size of the constant in terms of the exact number of bits
	that make up the constant. The size specification is
	optional.  The character f represents the base format and
	must be one of d, h, o, or b, for the bases decimal,
	hexadecimal, octal and binary respectively.  The characters
	nn..n represent the constants value, and must follow the base
	specification and be one of the following characters

	0123456789abcdefABCDEFxXzZ

	White space is allowed between ss..s and the characters 'f,
	and between 'f and nn..n.

BASE is one of the following tokens:
	'b	 'B	 'o	 'O	 'd	 'D	 'h	 'H
*/

concatenation
	: '{' expression_catlist '}'
	{
	    CNode* a2 = Macro2Node($2);
	    if( a2->GetOp() != eCAT ) {
	        CNode* n = _UCAT( a2, &loc );
                $$ = AnnotateMacroToken( loc, n, $1, $2, $3 );
	    } else {
		$$ = AnnotateMacroToken( loc, a2, $1, $2, $3 );;
	    }
	}
	| '{' constant_expression1 '{' expression_catlist '}' '}'
	{
		CNode* a2 = Macro2Node($2);
		CNode* a4 = Macro2Node($4);
		CNode* n = _REP( a2, a4, &loc );
                $$ = AnnotateMacroToken( loc, n, $1, $2, $3, $4, $5, $6 );
	}
	;

constant_concatenation
	: '{' constant_expression_catlist '}'
	{
	    CNode* a2 = Macro2Node($2);
	    if( a2->GetOp() != eCAT ) {
	        CNode* n = _UCAT( a2, &loc );
                $$ = AnnotateMacroToken( loc, n, $1, $2, $3 );
	    } else {
		$$ = AnnotateMacroToken( loc, a2, $1, $2, $3 );;
	    }
	}
	| '{' constant_expression '{' constant_expression_catlist '}' '}'
	{
		CNode* a2 = Macro2Node($2);
		CNode* a4 = Macro2Node($4);
		CNode* n = _REP( a2, a4, &loc );
                $$ = AnnotateMacroToken( loc, n, $1, $2, $3, $4, $5, $6 );
	}
	;

function_call
	: hierarchical_function_identifier '(' xexpression_clist rp
	{
		$$ = _FUNCTION_CALL( $1, $3, currentBlock, &loc );
	}
	; 

constant_function_call
	: function_identifier '(' constant_xexpression_clist rp
	{
		$$ = _FUNCTION_CALL( $1, $3, currentBlock, &loc );
	}
	; 

system_function_call
	: SYSTEM_IDENTIFIER '(' xexpression_clist rp
	{
		$$ = _SYSTASK_CALL( $1, $3, &loc );
	}
	| SYSTEM_IDENTIFIER 		%prec LOWEST
	{
		$$ = _SYSTASK_CALL( $1, NULL, &loc );
	}
	;

constant_system_function_call
	: SYSTEM_IDENTIFIER '(' constant_xexpression_clist rp
	{
		$$ = _SYSTASK_CALL( $1, $3, &loc );
	}
	| SYSTEM_IDENTIFIER 		%prec LOWEST
	{
		$$ = _SYSTASK_CALL( $1, NULL, &loc );
	}
	;

/*
A.8 General
*/

identifier
	: IDENTIFIER
	;

hierarchical_identifier_1
	: IDENTIFIER  %prec LOWEST
	{ $$ = new HIdentifier( $1 ); }
	| IDENTIFIER '[' expression ']' 
	{ $$ = new HIdentifier( $1, Macro2Node($3) ); }
	| IDENTIFIER '[' expression ':' constant_expression ']' 
	{
	    CNode* slice = Macro2Node(BOP(_SLICE, $3, $4, $5)); 
	    $$ = new HIdentifier( $1, slice ); 
	}
	| IDENTIFIER '[' expression PCOLON constant_expression ']' 
	{ $$ = new HIdentifier( $1, Macro2Node(BOP(_PSLICE,$3,$4,$5)) ); }
	| IDENTIFIER '[' expression MCOLON constant_expression ']' 
	{ $$ = new HIdentifier( $1, Macro2Node(BOP(_MSLICE,$3,$4,$5)) ); }
	;

hierarchical_identifier
	: hierarchical_identifier_1 '.' hierarchical_identifier
	{ 
	  $$ = $1;
	  $$->Push( $3 );
	  delete $3;
        }
	| hierarchical_identifier_1 
	{ $$ = $1; }
	;

hierarchical_parameter_identifier 
	: hierarchical_identifier 
	{ 
	  if( !$1->ValidId( 0, 0 ) ) {
	       error( &loc, "illegal identifier reference" );
	  }
	  $$ = $1->Symbol(); 
	  delete $1; 
	}
	;
hierarchical_event_identifier     
	: hierarchical_identifier 
	{ $$ = $1; }
	;
hierarchical_task_identifier      
	: hierarchical_identifier 
	{ 
	  if( !$1->ValidId( 0, 0 ) ) {
	       error( &loc, "illegal identifier reference" );
	  }
	  $$ = $1->Symbol(); 
	  delete $1; 
	}
	;
hierarchical_function_identifier  
	: hierarchical_identifier 
	{ 
	  if( !$1->ValidId( 0, 0 ) ) {
	       error( &loc, "illegal identifier reference" );
	  }
	  $$ = $1->Symbol(); 
	  delete $1; 
	}
	;

/*
IDENTIFIER
	An identifier is any sequence of letters, digits and the
	underscore (_) symbol, except that the first must be a letter
	or the underscore. .her and lower case letters are
	considered to be different (unless the.her case option is
	used when compiling).  Identifiers may be of any size, i.e.
	all characters are significant.
	Escaped identifiers start with the \ character and provide a
	means of including any of the printable ascii characters in
	an identifier.  An escaped identifier ends with white space.
	The leading \ character is not considered to be part of the
	identifier.
*/

delay_o
	: /* empty */
	{
	  $$ = NULL;
	}
	| delay_control
	{
	  $$ = _DELAY_CONTROL($1);
	}
	;

/* 
    A.9.1 Attributes
*/

attribute_instance_list_pragma_o
    : /* NULL */
    { $$ = NULL; }
    | attribute_instance_list
    { currentAttributeList = $1;
      $$ = NULL; }
    | attribute_instance_list pragmas
    { $$ = $2; }
    | pragmas
    { $$ = $1; }
    ;

attribute_instance_pragma_list_o
    : /* NULL */
    { $$ = NULL; }
    | attribute_instance_list
    { currentAttributeList = $1;
      $$ = NULL; }
    | attribute_instance_list pragmas
    { currentAttributeList = _LINK( currentAttributeList, $2 ); 
      $$ = NULL; 
    }
    | pragmas
    { currentAttributeList = _LINK( currentAttributeList, $1 );
      $$ = NULL;
    }
    ;

attribute_instance_list_o
    : /* NULL */
    { currentAttributeList = NULL; 
      $$ = NULL; }
    | attribute_instance_list
    { currentAttributeList = $1;
      $$ = NULL; }
    ;

attribute_instance_list
    : attribute_instance
    { $$ = $1; }
    | attribute_instance_list  attribute_instance
    { $$ = _LINK( $1, $2 ); }
    ;

attribute_instance
    : ATTRIBUTE_START 
      { attribute = new(unitStack) CAttr( &loc ); }
      attr_spec_list ATTRIBUTE_END
    { $$ = _ATTRIBUTE( attribute, &loc ); attribute = NULL; } 
    ;

attr_clear
    : /* NULL */
    { 
	$$ = currentAttributeList;
	lastAttributeList = currentAttributeList;
	currentAttributeList = NULL;
    }
    ;
	
last_attr
    : /* NULL */
    { 
	$$ = lastAttributeList;
    }
    ;
    ;

attr_spec_list
    : attr_spec
    | attr_spec_list ',' attr_spec
    ;

attr_spec
    : IDENTIFIER
    { 
	attribute->AddAttribute( $1->GetName(), NULL );
    }
    | identifier '=' expression
    { 
		attribute->AddAttribute( $1->GetName(), Macro2Node($3) );
		if( !Macro2Node($3)->IsConstant() ) {
		    error( &loc, "Constant expression expected" );
		}
	    }
	    ; 
	/*
	 * A.9.3 Identifiers
	 */

	variable_identifier
	    : IDENTIFIER
	    ;

	udp_identifier
		: IDENTIFIER
		{ $$ = ValidateType( $1, FALSE, eUDP, eNONE ); }
		;

	module_identifier
		: IDENTIFIER
		{ $$ = ValidateType( $1, FALSE, eMODULE, eNONE ); }
		;

	topmodule_identifier
	    : IDENTIFIER
		{ $$ = ValidateType( $1, FALSE, eMODULE, eNONE ); }
	    ;

	instance_identifier
	    : IDENTIFIER
		{ $$ = ValidateType( $1, FALSE, eINSTANCE, eNONE ); }
	    ;

	config_identifier
	    : IDENTIFIER
	    ;

	library_identifier
	    : IDENTIFIER
	    ;

	cell_identifier
	    : IDENTIFIER
	    ;

	port_identifier
	    : IDENTIFIER
		{ $$ = ValidateType( $1, TRUE, ePORT, eNET, eNONE ); }
	    ;

	input_port_identifier
	    : IDENTIFIER
		{ $$ = ValidateType( $1, FALSE, eINPUT, eNONE ); }
	    ;

	output_port_identifier
	    : IDENTIFIER
		{ $$ = ValidateType( $1, FALSE, eOUTPUT, eNONE ); }
	    ;

	output_reg_port_identifier
	    : IDENTIFIER
		{ $$ = ValidateType( $1, FALSE, eREG, eREAL, eNONE ); }
	    ;

	function_identifier
	    : IDENTIFIER
		{ $$ = ValidateType( $1, FALSE, eFUNCTION, eNONE ); }
	    ;

	genvar_identifier
	    : IDENTIFIER
		{ $$ = ValidateType( $1, FALSE, eGENVAR, eNONE ); }
	    ;

	generate_block_identifier
	    : IDENTIFIER
	    ;

	parameter_or_specparam_identifier
	    : IDENTIFIER 
		{ $$ = ValidateType( $1, allowRvalFrefs,  eLOCALPARAM, ePARAM, eNONE ); }
	    ;

	parameter_or_specparam_or_genvar_identifier
	    : IDENTIFIER 
		{ $$ = ValidateType( $1, FALSE, eLOCALPARAM, ePARAM, eGENVAR, eNONE ); }
	    ;

	/*
	"Important" terminals
	*/

	rp
		: ')'
			{ yyerrok; $$ = $1; }
		;

	rb
		: ']'
			{ yyerrok; }
		| error
			{ yyerrok; }
		;

	sc
		: ';'
			{ yyerrok; }
		;

pragma_macro
    : comments PRAGMA  
    { $$ = AnnotateMacroToken( loc, _LINK($1,_PRAGMA(Macro2Text($2),&loc)), $2 ); }
    | comments VRQ
    { $$ = AnnotateMacroToken( loc, _LINK($1,_VRQ(Macro2Text($2), &loc)) ,$2 ); }
    | comments EOI
    { 
        $$ = AnnotateMacroToken( loc, $1 ,$2 );
    }
    ;

pragmas_macro
    : pragmas_macro comments pragma_macro
    {
	$3->nodePtr = _LINK( $2, $3->nodePtr );
	$$ = _LINK( $1, $3 );
    }
    | comments pragma_macro  
    {   $$ = $2;
        $$->nodePtr =_LINK($1,Macro2Node($2)); 
    }
    ;

pragmas
    : pragmas_macro
    { $$ = Macro2Node($1); }
    ;

pragmas_o
        : comments
        | pragmas
        ;

	%%
	static int MoreComments( CNode* n )
	{
		switch( n->GetOp() ) {
		case eCOMMENT:
			return TRUE;
		case eLIST:
			return MoreComments( n->Arg<CNode*>(1) ) |
			       MoreComments( n->Arg<CNode*>(0) );
		default:
			return FALSE;
		}
	}
	void ValidateLeftHandComments( CNode* n, int more )
	{
		switch( n->GetOp() ) {
		case eCOMMENT:
			return;
		case eLIST:
			{
			CNode* r = n->Arg<CNode*>(1);
			CNode* l = n->Arg<CNode*>(0);
			if( more && r != NULL && r->GetOp() == eCOMMENT ) {
				error( r->GetCoord(),
					"Illegal synopsys comment placement\n" );
			}
			ValidateLeftHandComments( l, MoreComments(r)||more );
			ValidateLeftHandComments( r, more );
			}
			break;
		default:
			return;
		}
	}
	void DoTopLevelComments( void ) {
		char* comment = lex_getComments();
		if( strlen(comment) > 1 ) {
			CNode* n = _COMMENT( comment, &loc );
			if( codeList == NULL ) {
				codeList = n;
			} else {
				CNode* link = _LIST( codeList, n );
				codeList = link;
			}
		}
	}

	CNode* PrependComments( CNode* n ) {
		char* comment = lex_getComments();
		if( strlen(comment) > 1 ) {
			if( n == NULL ) {
				return _COMMENT(comment, &loc);
			}
			return _LIST( _COMMENT(comment, &loc), n );
		}
		return n;
	}

	CNode* PostpendComments( CNode* n ) {
		char* comment = lex_getComments();
		if( strlen(comment) > 1 ) {
			if( n == NULL ) {
				return _COMMENT(comment, &loc);
			}
			return _LIST( n, _COMMENT(comment, &loc) );
		}
		return n;
	}

	CNode* GetComments()
	{
		char* comment = lex_getComments();
		if( strlen(comment) > 1 ) {
			return _COMMENT(comment, &loc);
		}
		return NULL;
	}

	void DoComments( void ) {
		char* comment = lex_getComments();
		if( strlen(comment) > 1 ) {
			CNode* n = _COMMENT(comment, &loc);
			if( currentBlock != NULL ) {
				currentBlock->Add(n);
			} else {
				codeList = _LIST( codeList, n );
			}
		}
	}

	int IsExternal( CSymbol* sym )
	{
		return sym->Escaped() && strchr( sym->GetName(), '.' ) != NULL;
	}

	void _ValidateType( int allowFref, CDecl* decl,  list<Decl_t>& tl )
	{
		/*	
		 * if frefs are allowed then constrain
		 * fref types
		 */
		if( allowFref && decl->GetType() == eFREF ) {
			((CFref*)decl)->ConstrainTypes( tl );
			return;
		}
		
		list<Decl_t>::iterator ptr;
		for( ptr = tl.begin(); ptr != tl.end(); ++ptr ) {
		    Decl_t t = *ptr;
		    if( decl->GetType() == t || decl->GetClass() == t ) {
			break;
		    }
		    if( t == eNONE ) {
			error( &loc, "%s is illegal variable type\n", decl->GetName() );
			break;
		    }
		} 
		return;
	}
	void	_ValidateLvalList( CNode*n, int allowFref, list<Decl_t>& tl )
	{
		switch( n->GetOp() ) {
		case eCAT:
		    _ValidateLvalList( n->Arg<CNode*>(0), allowFref, tl );
		    _ValidateLvalList( n->Arg<CNode*>(1), allowFref, tl );
		    break;
		case eARRAY:
		    _ValidateLvalList( n->Arg<CNode*>(0), allowFref, tl );    
		    break;
		case eNET_REF:
		case eREG_REF:
		case eREAL_REF:
		case ePARAM_REF:
		case ePORT_REF:
		case eFWD_REF:
		case eEVENT_REF:
		case eGENVAR_REF:
		    _ValidateType( allowFref, n->Arg<CDecl*>(0), tl );    
		    break;
		default: ;
		}
	}
	CNode*   ValidateLvalList( CNode* n, int allowFref, ... )
	{
		va_list ap;
		va_start( ap, allowFref );
		list<Decl_t> tl;
		while(1) {
		    Decl_t t = (Decl_t)va_arg(ap, int);
		    tl.push_back( t );
		    if( t == eNONE ) {
			break;
		    }
		} 
		va_end( ap );
		_ValidateLvalList( n, allowFref, tl );
		return n;
	}
	CSymbol*  ValidateType( CSymbol* sym, int allowFref, ... )
	{
		va_list ap;
		va_start( ap, allowFref );
		list<Decl_t> tl;
		while(1) {
		    Decl_t t = (Decl_t)va_arg(ap, int);
		    tl.push_back( t );
		    if( t == eNONE ) {
			break;
		    }
		} 
		va_end( ap );
		CDecl*	decl = moduleSymtab.Lookup( sym );
		if( !decl ) {
		    /*	
		     * create Fref if allowed
		     */
		    if( allowFref ) {
			CFref* fref = new(unitStack) CFref( sym, &loc );
			currentBlock->Add( fref );
			frefSymtab.Add( sym, fref );
		    }
		    return sym;
		}
		_ValidateType( allowFref, decl, tl );    
		return sym;
	}
	CNode* Val( int createFref, int rval, CSymbol* sym, DimInfo* dim )
	{
		// ??? mdh - Frefs/Ext need to be fixed for arrays
		// ??? mdh - Add type validation for Frefs/Ext
		CDecl*	decl = moduleSymtab.Lookup( sym );
		Decl_t  subType;
		CNode*  result;
		int	illegal = 0;
		int 	ext = IsExternal( sym );
		int	hasSelect = dim && (dim->dimensions.size() > 0 && 
				          (dim->dimensions.back()->GetOp() == eSLICE || 
				           dim->dimensions.back()->GetOp() == ePSLICE || 
				           dim->dimensions.back()->GetOp() == eMSLICE) );
		/*
		 * Note: this is not definitive as an array can have a width of 1
		 *       however for inferencing widths, assume a width of 1 is not
		 * 	 an array reference, but a select.
		 */
		int	isArray = dim && dim->dimensions.size() > 1; 

		if( ext ) {
			result = _EXTERNAL_REF( sym );
			goto DONE_identifier1;
		}
		if( decl == NULL ) {
			CFref* fref = new(unitStack) CFref( sym, &loc );
			fref->Lval( !rval );
			CBlock* root = currentBlock;
			if( currentFunction == NULL ) {
			    while( root->GetParent() ) {
				root = root->GetParent();
			    }
			}
			root->Add( fref );
			moduleSymtab.Add( sym, fref );
			decl = fref;
		}

		/*
		 * accumulate info about fref for inferencing
		 */
		if( decl->GetType() == eFREF ) {
		    CFref* fref = dynamic_cast<CFref*>(decl);
		    if( dim ) {
			if( dim->dimensions.size() > 0 ) {
			     if( dim->dimensions.back()->GetOp() == 
							eSLICE ) {
		                 fref->SetRangeInfo( isArray,  
			               _RANGE(dim->dimensions.back()->Arg<CNode*>(0)->Clone(),
			               dim->dimensions.back()->Arg<CNode*>(1)->Clone()) );
			     } else if( dim->dimensions.back()->GetOp() != ePSLICE &&
				        dim->dimensions.back()->GetOp() != eSLICE ) {
		                 fref->SetRangeInfo( isArray,  
			               _RANGE(dim->dimensions.back()->Clone(),
			               dim->dimensions.back()->Clone()) );
			     }
			}
		    }
		}

		subType = (decl->GetType() == ePARAM || decl->GetType() == eLOCALPARAM)
				 ? ((CParam*)decl)->GetSubType(): decl->GetType();
		switch( subType ) {
		case eREAL:
			if( dim && hasSelect ) {
				illegal = 1;
				break;
			}
		default: ;
		}

		switch( decl->GetType() ) {
		case eWIRE:
		case eTRI:
		case eWAND:
		case eTRIAND:
		case eWOR:
		case eTRIOR:
		case eTRI1:
		case eTRI0:
		case eTRIREG:
		case eSUPPLY0:
		case eSUPPLY1:
			result = _NET_REF( dynamic_cast<CNet*>(decl), &loc );
			break;
		case eTIME:
		case eINT:
		case eREG:
			result = _REG_REF( dynamic_cast<CReg*>(decl), &loc );
			break;
		case eGENVAR:
			result = _GENVAR_REF( dynamic_cast<CGenvar*>(decl), 
									&loc );
			break;
		case eREALTIME:
		case eREAL:
			result = _REAL_REF( dynamic_cast<CReal*>(decl), &loc );
			break;
		case eLOCALPARAM:
		case ePARAM:
			result = _PARAM_REF( dynamic_cast<CParam*>(decl), &loc );
			break;
		case eINPUT:
		case eOUTPUT:
		case eINOUT:
			result = _PORT_REF( dynamic_cast<CPortDir*>(decl), &loc );
			break;
		case eDEVENT:
			result = _EVENT_REF( dynamic_cast<CEvent*>(decl), &loc );
			break;
		case eFREF:
			result =  _FWD_REF( dynamic_cast<CFref*>(decl), &loc );
			if( !rval ) {
			    dynamic_cast<CFref*>(decl)->Lval(1);
			}
			break;
		default:
			illegal = 1;
		}
		
		if( illegal ) {
			if( rval ) {
			    error( &loc, "%s is an illegal rval\n", sym->GetName() );
			} else {
			    error( &loc, "%s is an illegal lval\n", sym->GetName() );
			}
			result = &errorNode;
		}

	DONE_identifier1:
		if( dim && hasSelect ) {
		    switch( dim->dimensions.back()->GetOp() ) {
		    case eSLICE:
		    case ePSLICE:
		    case eMSLICE:
			if( !dim->dimensions.back()->IsWidthVolatile() &&
			    dim->dimensions.back()->IsWidthEvaluateable() &&
			    dim->dimensions.back()->GetWidth() == 0 ) {
			    error( &loc, "%s part select cannot have zero width\n", sym->GetName() );
			}
		    default:
			break;
		    }
		}
		
		CNode* index     = NULL;
		int    depth     = 0;
		int    declDims = ext ? (dim ? dim->dimensions.size() : 0) : 
					decl->GetNumberOfDimensions();
		while( dim && dim->dimensions.size() ) {
		    if( depth > declDims && decl != errorDecl ) {
			error( &loc, "reference %s has too many dimensions\n", sym->GetName() );
		    } else {
			index = _ELINK(index,dim->dimensions.front());
		    }
		    dim->dimensions.pop_front();
		    depth++;
		}
		if( index ) {
		    result = _ARRAY( result, index, &loc );
		} 
		if( dim ) {
		    delete dim;
		}

		return result;
	}

	CNet* CreateNet( CSymbol* symbol, CBlock* block,
		DeclType_t declType, int setDelay  )
	{
	    CNet* 		net = NULL;
	    CDecl*		decl;

	    decl = moduleSymtab.LookupTop( symbol );
	    if( decl != NULL && decl->GetType() != eFREF ) {
		Coord_t*	refLoc = decl->GetCoord();
		if( decl->GetType() != declType.type ) {
		    decl = NULL;
		    error( &loc, "%s was previously defined in file '%s' line %d",
					symbol->GetName(), refLoc->filename,
					refLoc->lineno );
		} else {
		    message( &loc, mWPDFS,
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
		}
		dimIndexes.clear();
	    }
	    if( decl && decl->GetType() == eFREF &&
	        !((CFref*)decl)->IsTypeValid( eNET ) ) {
		Coord_t*	refLoc = decl->GetCoord();
		error( &loc, "%s was previously defined in file '%s' line %d",
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
	    }
	    if( decl == NULL || decl->GetType() == eFREF ) {
		MASSERT( block != NULL );
		net = new(unitStack) CNet( symbol, &loc, declType.type, FALSE );
		net->SetNumberOfDimensions( dimIndexes.size() );
		int depth = 0;
		while( dimIndexes.size() ) {
		    net->SetIndex( depth, dimIndexes.front() );
		    dimIndexes.pop_front();
		    depth++;
		}
		net->SetRange( declType.range );
		net->SetSigned( declType.signed_ );
		net->SetVectored( declType.vectored );
		net->SetScalared( declType.scalared );
		if( net->GetType() == eTRIREG ) {
		    net->SetStrength( currentDeclType.stpair );
		}
		if( setDelay ) {
		    net->SetDelay( declType.delay );
		}
		net->SetAttributes( declType.attr?declType.attr->Clone():NULL );
		net->SetDeclStatementCreated();
		block->Add( net );
		moduleSymtab.Add( symbol, net );
		decl = net;
	    }
	    return (CNet*)decl;
	}

	CGenvar* CreateGenvar( int createLParam, CSymbol* symbol, 
				CBlock* block, DeclType_t declType )
	{
	    CGenvar* 	genvar;
	    CDecl*	decl;

	    decl = moduleSymtab.LookupTop( symbol );
	    if( createLParam && decl == NULL ) {
		error( &loc, "%s was not previously defined",
		       symbol->GetName() );
	    }
	    if( !createLParam && decl != NULL ) {
		Coord_t* refLoc = decl->GetCoord();
		if( decl->GetType() != eGENVAR ) {
		    decl = NULL;
		    error( &loc, "%s was previously defined in file '%s' line %d",
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
		} else {
		    message( &loc, mWPDFS,
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
		}
	    }
	    if( decl == NULL ) {
		MASSERT( currentBlock != NULL );
		genvar = new(unitStack) CGenvar( symbol, &loc );
		genvar->SetAttributes( declType.attr );
		block->Add( genvar );
		moduleSymtab.Add( symbol, genvar );
		genvar->SetDeclStatementCreated();
		decl = genvar;
	    }
	    return (CGenvar*)decl;
	}

	CEvent* CreateEvent( CSymbol* symbol, CBlock* block, DeclType_t declType )
	{
	    CEvent* e;
	    CDecl*  decl;

	    decl = moduleSymtab.LookupTop( symbol );
	    if( decl != NULL ) {
		Coord_t* refLoc = decl->GetCoord();
		if( decl->GetType() != declType.type ) {
		    decl = NULL;
		    error( &loc, "%s was previously defined in file '%s' line %d",
			   symbol->GetName(), refLoc->filename,
			   refLoc->lineno );
		} else {
		    message( &loc, mWPDFS,
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
		}
		dimIndexes.clear();
		/*
		 * return dummy event
		 */
		decl = new(unitStack) CEvent( symbol, &loc );
	    } 
	    if( decl == NULL ) {
		MASSERT( currentBlock != NULL );
		e = new(unitStack) CEvent( symbol, &loc );
		e->SetNumberOfDimensions( dimIndexes.size() );
		e->SetAttributes( declType.attr );
		int depth = 0;
		while( dimIndexes.size() ) {
		    e->SetIndex( depth, dimIndexes.front() );
		    dimIndexes.pop_front();
		    depth++;
		}
		block->Add( e );
		moduleSymtab.Add( symbol, e );
		e->SetDeclStatementCreated();
		decl = e;
	    }
	    return (CEvent*)decl;
	}


	CReal* CreateReal( CSymbol* symbol, CBlock* block, DeclType_t declType )
	{
	    CReal* 	real;
	    CDecl*	decl;

	    decl = moduleSymtab.LookupTop( symbol );
	    if( decl != NULL && decl->GetType() != eFREF ) {
		Coord_t*	refLoc = decl->GetCoord();
		if( decl->GetType() != declType.type ) {
		    decl = NULL;
		    error( &loc, "%s was previously defined in file '%s' line %d",
					symbol->GetName(), refLoc->filename,
					refLoc->lineno );
		} else {
		    message( &loc, mWPDFS,
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
		}
		dimIndexes.clear();
	    }
	    if( decl && decl->GetType() == eFREF &&
	        !((CFref*)decl)->IsTypeValid( eREAL ) ) {
		Coord_t*	refLoc = decl->GetCoord();
		error( &loc, "%s was previously defined in file '%s' line %d",
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
	    }
	    if( decl == NULL || decl->GetType() == eFREF ) {
		MASSERT( currentBlock != NULL );
		real = new(unitStack) CReal( symbol, &loc, declType.type );
		real->SetNumberOfDimensions( dimIndexes.size() );
		real->SetAttributes( declType.attr );
		int depth = 0;
		while( dimIndexes.size() ) {
		    real->SetIndex( depth, dimIndexes.front() );
		    dimIndexes.pop_front();
		    depth++;
		}
		MASSERT( !declType.range );
		block->Add( real );
		moduleSymtab.Add( symbol, real );
		real->SetDeclStatementCreated();
		decl = real;
	    }
	    return (CReal*)decl;
	}
	CReg* CreateRegister( CSymbol* symbol, CBlock* block, DeclType_t declType )
	{
	    CReg* 	reg;
	    CDecl*	decl;

	    decl = moduleSymtab.LookupTop( symbol );
	    if( decl != NULL && decl->GetType() != eFREF ) {
		Coord_t* refLoc = decl->GetCoord();
		if( decl->GetType() != declType.type ) {
		    decl = NULL;
		    error( &loc, "%s was previously defined in file '%s' line %d",
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
		} else {
		    message( &loc, mWPDFS,
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
		}
		dimIndexes.clear();
	    }
	    if( decl && decl->GetType() == eFREF &&
	        !((CFref*)decl)->IsTypeValid( eREG ) ) {
		Coord_t* refLoc = decl->GetCoord();
		error( &loc, "%s was previously defined in file '%s' line %d",
			     symbol->GetName(), refLoc->filename,
			     refLoc->lineno );
	    }
	    if( decl == NULL || decl->GetType() == eFREF ) {
		MASSERT( currentBlock != NULL );
		reg = new(unitStack) CReg( symbol, &loc, declType.type, FALSE );
		reg->SetNumberOfDimensions( dimIndexes.size() );
		int depth = 0;
		while( dimIndexes.size() ) {
		    reg->SetIndex( depth, dimIndexes.front() );
		    dimIndexes.pop_front();
		    depth++;
		}
		reg->SetRange( declType.range );
		reg->SetSigned( declType.signed_ );
		reg->SetAttributes( declType.attr );
		block->Add( reg );
		moduleSymtab.Add( symbol, reg );
		reg->SetDeclStatementCreated();
		decl = reg;
	    }
	    return (CReg*)decl;
	}

MacroToken* AnnotateMacroToken( Coord_t loc, CNode* exp, 
	MacroToken* m1, MacroToken* m2, MacroToken* m3,
	MacroToken* m4, MacroToken* m5, MacroToken* m6 )
{
	MacroToken* mm = m1; 
	if( mm && m2 ) {
		mm = MergeToken( mm, m2 );
	}
	if( mm && m3 ) {
		mm = MergeToken( mm, m3 );
	}
	if( mm && m4 ) {
		mm = MergeToken( mm, m4 );
	}
	if( mm && m5 ) {
		mm = MergeToken( mm, m5 );
	}
	if( mm && m6 ) {
		mm = MergeToken( mm, m6 );
	}
	if( mm ) {
		const MacroHistory* history = CompleteToken( mm );
	        if( keepMacros && history ) {
	            exp = _MACRO_EXPR( history->macroName, exp, &loc );
		}
	} else {
	    mm = AllocateNullMacroToken();
	}
	mm->text = NULL;
	mm->nodePtr = exp;
	mm->previousNodePtr = NULL;
	return mm;
}

	CVector* SizedStringToVector(Coord_t* refLoc, char* widthspec, char* spec, char* value)
	{
	    int		fail = 0;
	    char 	c1 = ' ';
	    char 	c2 = ' ';
	    INT32	width;
	    CVector*    vector;

	    if( spec ) {
		if( spec[1] && spec[2] ) {
		    c1 = spec[1];
		    c2 = spec[2];
		} else if( spec[1] ) {
		    c2 = spec[1];
		}
	    }
	    INT32  sized;
	    if( widthspec == NULL ) {
		    width = 32;
		    sized = FALSE;
	    } else {
		    /*
		     * strip '_' from widthspec
		     */
		    char* pd = widthspec;
		    char* ps = widthspec;
		    while( *ps ) {
			if( *ps == '_' ){
			    ps++;
			} else if( ps != pd ) {
			    *pd++ = *ps++;
			} else {
			    ++pd;
			    ++ps;
			}
		    }
		    *pd = 0;
		    width = atoi( widthspec );
		    sized = TRUE;
		    if( width <= 0 ) {
			width = 1;
			fail  = 1;
		    }
	    }
	    vector = CVector::AllocFromHeap( unitStack, width );
	    vector->Sized( sized );
	    vector->Based( spec != NULL );
	    while( *value && strchr( " \t\f\n", *value ) ) value++;
	    int tooManyDigits = FALSE;
	    int digits = 0;
	    for( char* p = value; *p; ++p ) {
		if( *p != '_' ) {
		    digits++;
		}
	    }
	    switch( c1 ) {
	    case ' ':
		    vector->Signed( c2 == ' ' );
		    break;
	    case 's':
	    case 'S':
		    vector->Signed( 1 );
		    break;
	    default:
		    vector->Signed( 0 );
		    break;
	    }
	    switch( c2 ) {
	    case 'h':
	    case 'H':
		    fail |= !vector->LoadHex( value );
		    tooManyDigits |= (digits > (width+3)/4);
		    break;
	    case ' ':
	    case 'd':
	    case 'D':
		    fail |= !vector->LoadDecimal( value );
		    break;
	    case 'o':
	    case 'O':
		    fail |= !vector->LoadOctal( value );
		    tooManyDigits |= (digits > (width+2)/3);
		    break;
	    case 'b':
	    case 'B':
		    fail |= !vector->LoadBinary( value );
		    tooManyDigits |= (digits > width);
		    break;
	    default:
		    MASSERT( FALSE );
	    }
	    if( fail )  {
		error( &loc, "Illegal number %s%s%s in file '%s' line %d",
				(widthspec?widthspec:""),		
				(spec?spec:""),		
				value,
				refLoc->filename, refLoc->lineno );
	    } else if( tooManyDigits ) { 
		message( &loc, mTMDIN );
	    } else if( vector->Overflowed() && c2 != 'd' && c2 != 'D' && c2 != ' ' )  {
		message( &loc, mTMBIN );
	    } else if( vector->Overflowed() ) {
		message( &loc, mCTTSW );
	    }
	    return vector;
	}
	void RegisterFrefs( CNode* n, void* ) 
	{
	    if( n->GetOp() == eFWD_REF ) {
		CFref* fref = n->Arg<CFref*>(0);
		frefSymtab.Add( fref->GetSymbol(), fref );
		currentBlock->Add( fref );
	    }
	    if( n->GetOp() == ePORT_DEF ) {
		CPort* port = n->Arg<CPort*>(0);
		if( port->GetConnection() ) {
		    port->GetConnection()->PostVisit1( RegisterFrefs, 0 );
		}
	    }
	}
/*********************************************************
	PortDir2PortDef
	- convert a list of PortDir to a list of PortDef
 ********************************************************/
CNode* PortDir2PortDef( CNode* dirList )
{

    if( !dirList ) {
	return NULL;
    }
    switch( dirList->GetOp() ) {
    case eLIST:
	{
	    CNode* c = dirList;
	    CNode* n = NULL;
	    while( 1 ) {
	        n = _LINK(n,PortDir2PortDef(c->Arg<CNode*>(0)));
		if( !c->Arg<CNode*>(1) ||
		    c->Arg<CNode*>(1)->GetOp() != eLIST ) {
	            n = _LINK(n,PortDir2PortDef(c->Arg<CNode*>(1)));
		    break;
		}
		c = c->Arg<CNode*>(1);
	    }
	    return n;
        }
    case ePORT_DECL:
	{
    	    CPortDir* dir = dirList->Arg<CPortDir*>(0);
	    CPort* p = new(unitStack) CPort( dir->GetSymbol(), 
				             FALSE, dir->GetCoord() );
	    CFref* fref = new(unitStack) CFref( dir->GetSymbol(),
				                dir->GetCoord() );
	    currentBlock->Add( fref );
	    p->SetConnection(_FWD_REF( fref, dir->GetCoord() ));
	    return _PORT_DEF(p);
	}
    default:
 	return NULL;
    }
}

	void parse_init()
	{
		currentModule = NULL;
		currentBlock  = NULL;
	//	yydebug = 1;
	}

